@@ -3,8 +3,8 @@
 import abc
 import numpy as np
 import astropy.units as u
-from ...utils.fitting import Parameter, Parameters
-from ...spectrum.utils import integrate_spectrum
+from gammapy.modeling import Parameter, Parameters
+from gammapy.utils.integrate import integrate_spectrum
 
 __all__ = [
     "DMProfile",
@@ -26,10 +26,7 @@ class DMProfile(abc.ABC):
 
     def __call__(self, radius):
         """Call evaluate method of derived classes."""
-        kwargs = dict()
-        for par in self.parameters.parameters:
-            kwargs[par.name] = par.quantity
-
+        kwargs = {par.name: par.quantity for par in self.parameters}
         return self.evaluate(radius, **kwargs)
 
     def scale_to_local_density(self):
@@ -52,7 +49,7 @@ class DMProfile(abc.ABC):
         rmin, rmax : `~astropy.units.Quantity`
             Lower and upper bound of integration range.
         **kwargs : dict
-            Keyword arguments passed to :func:`~gammapy.spectrum.integrate_spectrum`
+            Keyword arguments passed to :func:`~gammapy.utils.integrate.integrate_spectrum`
         """
         integral = integrate_spectrum(self._eval_squared, rmin, rmax, **kwargs)
         return integral.to("GeV2 / cm5")
