@@ -1,18 +1,15 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """HESS Galactic plane survey (HGPS) catalog."""
-from collections import OrderedDict
 import numpy as np
 import astropy.units as u
-from astropy.table import Table
 from astropy.coordinates import Angle
 from astropy.modeling.models import Gaussian1D
-from ..utils.scripts import make_path
-from ..utils.table import table_row_to_dict
-from ..utils.interpolation import ScaledRegularGridInterpolator
-from ..spectrum import FluxPoints
-from ..spectrum.models import PowerLaw, ExponentialCutoffPowerLaw
-from ..image.models import SkyPointSource, SkyGaussian, SkyShell
-from ..cube.models import SkyModel, SkyModels
+from astropy.table import Table
+from gammapy.modeling.models import Model, SkyModel, SkyModels
+from gammapy.spectrum import FluxPoints
+from gammapy.utils.interpolation import ScaledRegularGridInterpolator
+from gammapy.utils.scripts import make_path
+from gammapy.utils.table import table_row_to_dict
 from .core import SourceCatalog, SourceCatalogObject
 
 __all__ = [
@@ -32,7 +29,7 @@ FLUX_TO_CRAB = 100 / 2.26e-11
 FLUX_TO_CRAB_DIFF = 100 / 3.5060459323111307e-11
 
 
-class SourceCatalogObjectHGPSComponent:
+class SourceCatalogObjectHGPSComponent(SourceCatalogObject):
     """One Gaussian component from the HGPS catalog.
 
     See Also
@@ -41,13 +38,12 @@ class SourceCatalogObjectHGPSComponent:
     """
 
     _source_name_key = "Component_ID"
-    _source_index_key = "row_index"
 
     def __init__(self, data):
-        self.data = OrderedDict(data)
+        self.data = data
 
     def __repr__(self):
-        return "{}({!r})".format(self.__class__.__name__, self.name)
+        return f"{self.__class__.__name__}({self.name!r})"
 
     def __str__(self):
         """Pretty-print source data"""
@@ -68,22 +64,21 @@ class SourceCatalogObjectHGPSComponent:
     @property
     def name(self):
         """Source name (str)"""
-        name = self.data[self._source_name_key]
-        return name.strip()
+        return self.data[self._source_name_key]
 
-    @property
-    def index(self):
-        """Row index of source in table (int)"""
-        return self.data[self._source_index_key]
-
-    @property
     def spatial_model(self):
-        """Component spatial model (`~gammapy.image.models.SkyGaussian`)."""
+        """Component spatial model (`~gammapy.modeling.models.GaussianSpatialModel`)."""
         d = self.data
-        model = SkyGaussian(lon_0=d["GLON"], lat_0=d["GLAT"], sigma=d["Size"])
-        model.parameters.set_parameter_errors(
-            dict(lon_0=d["GLON_Err"], lat_0=d["GLAT_Err"], sigma=d["Size_Err"])
-        )
+        tag = "GaussianSpatialModel"
+        pars = {
+            "lon_0": d["GLON"],
+            "lat_0": d["GLAT"],
+            "sigma": d["Size"],
+            "frame": "galactic",
+        }
+        errs = {"lon_0": d["GLON_Err"], "lat_0": d["GLAT_Err"], "sigma": d["Size_Err"]}
+        model = Model.create(tag, **pars)
+        model.parameters.set_error(**errs)
         return model
 
 
@@ -95,7 +90,7 @@ class SourceCatalogObjectHGPS(SourceCatalogObject):
     """
 
     def __repr__(self):
-        return "{}({!r})".format(self.__class__.__name__, self.name)
+        return f"{self.__class__.__name__}({self.name!r})"
 
     def __str__(self):
         return self.info()
@@ -133,8 +128,8 @@ class SourceCatalogObjectHGPS(SourceCatalogObject):
         """Print basic info."""
         d = self.data
         ss = "\n*** Basic info ***\n\n"
-        ss += "Catalog row index (zero-based) : {}\n".format(d["catalog_row_index"])
-        ss += "{:<20s} : {}\n".format("Source name", d["Source_Name"])
+        ss += "Catalog row index (zero-based) : {}\n".format(self.row_index)
+        ss += "{:<20s} : {}\n".format("Source name", self.name)
 
         ss += "{:<20s} : {}\n".format("Analysis reference", d["Analysis_Reference"])
         ss += "{:<20s} : {}\n".format("Source class", d["Source_Class"])
@@ -151,9 +146,7 @@ class SourceCatalogObjectHGPS(SourceCatalogObject):
 
     def _info_id(self):
         ss = "\n*** Source identification info ***\n\n"
-        ss += "\n".join(
-            "{}: {}".format(k, v) for k, v in self.identification_info.items()
-        )
+        ss += "\n".join(f"{k}: {v}" for k, v in self.identification_info.items())
         return ss + "\n"
 
     def _info_map(self):
@@ -252,12 +245,12 @@ class SourceCatalogObjectHGPS(SourceCatalogObject):
             "Flux correction (RSpec -> Total)",
             100 * d["Flux_Correction_RSpec_To_Total"],
         )
-        ss += "{:<35s} : {:5.1f} %\n".format(label, val)
+        ss += f"{label:<35s} : {val:5.1f} %\n"
         label, val = (
             "Flux correction (Total -> RSpec)",
             100 * (1 / d["Flux_Correction_RSpec_To_Total"]),
         )
-        ss += "{:<35s} : {:5.1f} %\n".format(label, val)
+        ss += f"{label:<35s} : {val:5.1f} %\n"
 
         return ss
 
@@ -420,21 +413,13 @@ class SourceCatalogObjectHGPS(SourceCatalogObject):
 
         return u.Quantity([emin, emax], "TeV")
 
-    @property
-    def spectral_model_type(self):
-        """Spectral model type (str).
-
-        One of: 'pl', 'ecpl'
-        """
-        return self.data["Spectral_Model"].strip().lower()
-
     def spectral_model(self, which="best"):
-        """Spectral model (`~gammapy.spectrum.models.SpectralModel`).
+        """Spectral model (`~gammapy.modeling.models.SpectralModel`).
 
         One of the following models (given by ``Spectral_Model`` in the catalog):
 
-        - ``PL`` : `~gammapy.spectrum.models.PowerLaw`
-        - ``ECPL`` : `~gammapy.spectrum.models.ExponentialCutoffPowerLaw`
+        - ``PL`` : `~gammapy.modeling.models.PowerLawSpectralModel`
+        - ``ECPL`` : `~gammapy.modeling.models.ExpCutoffPowerLawSpectralModel`
 
         Parameters
         ----------
@@ -444,85 +429,79 @@ class SourceCatalogObjectHGPS(SourceCatalogObject):
         data = self.data
 
         if which == "best":
-            spec_type = self.spectral_model_type
+            spec_type = self.data["Spectral_Model"].strip().lower()
         elif which in {"pl", "ecpl"}:
             spec_type = which
         else:
-            raise ValueError("Invalid selection: which = {!r}".format(which))
-
-        pars, errs = {}, {}
+            raise ValueError(f"Invalid selection: which = {which!r}")
 
         if spec_type == "pl":
-            pars["index"] = data["Index_Spec_PL"]
-            pars["amplitude"] = data["Flux_Spec_PL_Diff_Pivot"]
-            pars["reference"] = data["Energy_Spec_PL_Pivot"]
-            errs["amplitude"] = data["Flux_Spec_PL_Diff_Pivot_Err"]
-            errs["index"] = data["Index_Spec_PL_Err"]
-            model = PowerLaw(**pars)
+            tag = "PowerLawSpectralModel"
+            pars = {
+                "index": data["Index_Spec_PL"],
+                "amplitude": data["Flux_Spec_PL_Diff_Pivot"],
+                "reference": data["Energy_Spec_PL_Pivot"],
+            }
+            errs = {
+                "amplitude": data["Flux_Spec_PL_Diff_Pivot_Err"],
+                "index": data["Index_Spec_PL_Err"],
+            }
         elif spec_type == "ecpl":
-            pars["index"] = data["Index_Spec_ECPL"]
-            pars["amplitude"] = data["Flux_Spec_ECPL_Diff_Pivot"]
-            pars["reference"] = data["Energy_Spec_ECPL_Pivot"]
-            pars["lambda_"] = data["Lambda_Spec_ECPL"]
-            errs["index"] = data["Index_Spec_ECPL_Err"]
-            errs["amplitude"] = data["Flux_Spec_ECPL_Diff_Pivot_Err"]
-            errs["lambda_"] = data["Lambda_Spec_ECPL_Err"]
-            model = ExponentialCutoffPowerLaw(**pars)
+            tag = "ExpCutoffPowerLawSpectralModel"
+            pars = {
+                "index": data["Index_Spec_ECPL"],
+                "amplitude": data["Flux_Spec_ECPL_Diff_Pivot"],
+                "reference": data["Energy_Spec_ECPL_Pivot"],
+                "lambda_": data["Lambda_Spec_ECPL"],
+            }
+            errs = {
+                "index": data["Index_Spec_ECPL_Err"],
+                "amplitude": data["Flux_Spec_ECPL_Diff_Pivot_Err"],
+                "lambda_": data["Lambda_Spec_ECPL_Err"],
+            }
         else:
-            raise ValueError("Invalid spectral model: {}".format(spec_type))
+            raise ValueError(f"Invalid spec_type: {spec_type}")
 
-        model.parameters.set_parameter_errors(errs)
+        model = Model.create(tag, **pars)
+        model.parameters.set_error(**errs)
         return model
 
-    @property
-    def spatial_model_type(self):
-        """Spatial model type (str).
-
-        One of: 'point-like', 'shell', 'gaussian', '2-gaussian', '3-gaussian'
-        """
-        return self.data["Spatial_Model"].strip().lower()
-
-    @property
-    def is_pointlike(self):
-        """Source is pointlike? (bool)"""
-        d = self.data
-        has_size_ul = np.isfinite(d["Size_UL"])
-        pointlike = d["Spatial_Model"] == "Point-Like"
-        return pointlike or has_size_ul
-
-    @property
     def spatial_model(self):
-        """Spatial model (`~gammapy.image.models.SkySpatialModel`).
+        """Spatial model (`~gammapy.modeling.models.SpatialModel`).
 
         One of the following models (given by ``Spatial_Model`` in the catalog):
 
-        - ``Point-Like`` or has a size upper limit : `~gammapy.image.models.SkyPointSource`
-        - ``Gaussian``: `~gammapy.image.models.SkyGaussian`
+        - ``Point-Like`` or has a size upper limit : `~gammapy.modeling.models.PointSpatialModel`
+        - ``Gaussian``: `~gammapy.modeling.models.GaussianSpatialModel`
         - ``2-Gaussian`` or ``3-Gaussian``: composite model (using ``+`` with Gaussians)
-        - ``Shell``: `~gammapy.image.models.SkyShell`
+        - ``Shell``: `~gammapy.modeling.models.ShellSpatialModel`
         """
         d = self.data
-        glon = d["GLON"]
-        glat = d["GLAT"]
+        pars = {"lon_0": d["GLON"], "lat_0": d["GLAT"], "frame": "galactic"}
+        errs = {"lon_0": d["GLON_Err"], "lat_0": d["GLAT_Err"]}
 
-        spatial_type = self.spatial_model_type
+        spatial_type = self.data["Spatial_Model"]
 
-        if self.is_pointlike:
-            model = SkyPointSource(lon_0=glon, lat_0=glat)
-        elif spatial_type == "gaussian":
-            model = SkyGaussian(lon_0=glon, lat_0=glat, sigma=d["Size"])
-        elif spatial_type in {"2-gaussian", "3-gaussian"}:
+        if spatial_type == "Point-Like":
+            tag = "PointSpatialModel"
+        elif spatial_type == "Gaussian":
+            tag = "GaussianSpatialModel"
+            pars["sigma"] = d["Size"]
+            errs["sigma"] = d["Size_Err"]
+        elif spatial_type in {"2-Gaussian", "3-Gaussian"}:
             raise ValueError("For Gaussian or Multi-Gaussian models, use sky_model()!")
-        elif spatial_type == "shell":
+        elif spatial_type == "Shell":
             # HGPS contains no information on shell width
             # Here we assume a 5% shell width for all shells.
-            r_out = d["Size"]
-            radius = 0.95 * r_out
-            width = r_out - radius
-            model = SkyShell(lon_0=glon, lat_0=glat, width=width, radius=radius)
+            tag = "ShellSpatialModel"
+            pars["radius"] = 0.95 * d["Size"]
+            pars["width"] = d["Size"] - pars["radius"]
+            errs["radius"] = d["Size_Err"]
         else:
-            raise ValueError("Not a valid spatial model: {}".format(spatial_type))
+            raise ValueError(f"Invalid spatial_type: {spatial_type}")
 
+        model = Model.create(tag, **pars)
+        model.parameters.set_error(**errs)
         return model
 
     def sky_model(self, which="best"):
@@ -535,31 +514,30 @@ class SourceCatalogObjectHGPS(SourceCatalogObject):
 
         Returns
         -------
-        sky_model : `~gammapy.cube.models.SkyModel`
+        sky_model : `~gammapy.modeling.models.SkyModel`
             Sky model of the catalog object.
         """
-        if self.spatial_model_type in {"2-gaussian", "3-gaussian"}:
+        spatial_type = self.data["Spatial_Model"]
+        if spatial_type in {"2-Gaussian", "3-Gaussian"}:
             models = []
-
-            spectral_model = self.spectral_model(which=which)
             for component in self.components:
+                spectral_model = self.spectral_model(which=which)
                 weight = component.data["Flux_Map"] / self.data["Flux_Map"]
-                spectral_model_comp = spectral_model.copy()
-                # weight amplitude of the component
-                spectral_model_comp.parameters["amplitude"].value *= weight
-                models.append(
-                    SkyModel(
-                        component.spatial_model,
-                        spectral_model_comp,
-                        name=component.name,
-                    )
+                spectral_model.parameters["amplitude"].value *= weight
+                model = SkyModel(
+                    spatial_model=component.spatial_model(),
+                    spectral_model=spectral_model,
+                    name=component.name,
                 )
+                models.append(model)
 
             return SkyModels(models)
         else:
-            spatial_model = self.spatial_model
-            spectral_model = self.spectral_model(which=which)
-            return SkyModel(spatial_model, spectral_model, name=self.name)
+            return SkyModel(
+                spatial_model=self.spatial_model(),
+                spectral_model=self.spectral_model(which=which),
+                name=self.name,
+            )
 
     @property
     def flux_points(self):
@@ -588,8 +566,6 @@ class SourceCatalogHGPS(SourceCatalog):
 
     One source is represented by `SourceCatalogObjectHGPS`.
 
-    An extensive tutorial is available here: :gp-notebook:`hgps`
-
     Examples
     --------
     Let's assume you have downloaded the HGPS catalog FITS file, e.g. via:
@@ -620,8 +596,6 @@ class SourceCatalogHGPS(SourceCatalog):
 
     >>> source.components
     >>> cat.gaussian_component(83)
-
-    More examples here: :gp-notebook:`hgps`
     """
 
     name = "hgps"
@@ -632,11 +606,12 @@ class SourceCatalogHGPS(SourceCatalog):
 
     source_object_class = SourceCatalogObjectHGPS
 
-    def __init__(self, filename=None, hdu="HGPS_SOURCES"):
-        if not filename:
-            filename = "$GAMMAPY_DATA/catalogs/hgps_catalog_v1.fits.gz"
-
-        filename = str(make_path(filename))
+    def __init__(
+        self,
+        filename="$GAMMAPY_DATA/catalogs/hgps_catalog_v1.fits.gz",
+        hdu="HGPS_SOURCES",
+    ):
+        filename = make_path(filename)
         table = Table.read(filename, hdu=hdu)
 
         source_name_alias = ("Identified_Object",)
@@ -644,6 +619,7 @@ class SourceCatalogHGPS(SourceCatalog):
 
         self._table_components = Table.read(filename, hdu="HGPS_GAUSS_COMPONENTS")
         self._table_associations = Table.read(filename, hdu="HGPS_ASSOCIATIONS")
+        self._table_associations["Separation"].format = ".6f"
         self._table_identifications = Table.read(filename, hdu="HGPS_IDENTIFICATIONS")
         self._table_large_scale_component = Table.read(
             filename, hdu="HGPS_LARGE_SCALE_COMPONENT"
@@ -679,7 +655,7 @@ class SourceCatalogHGPS(SourceCatalog):
         source = super()._make_source_object(index)
 
         if source.data["Components"] != "":
-            self._attach_component_info(source)
+            source.components = list(self._get_gaussian_components(source))
 
         self._attach_association_info(source)
 
@@ -688,12 +664,10 @@ class SourceCatalogHGPS(SourceCatalog):
 
         return source
 
-    def _attach_component_info(self, source):
-        source.components = []
-        lookup = SourceCatalog(self.table_components, source_name_key="Component_ID")
+    def _get_gaussian_components(self, source):
         for name in source.data["Components"].split(", "):
-            component = SourceCatalogObjectHGPSComponent(data=lookup[name].data)
-            source.components.append(component)
+            row_index = int(name.split()[-1]) - 1
+            yield self.gaussian_component(row_index)
 
     def _attach_association_info(self, source):
         t = self.table_associations
@@ -708,7 +682,7 @@ class SourceCatalogHGPS(SourceCatalog):
     def gaussian_component(self, row_idx):
         """Gaussian component (`SourceCatalogObjectHGPSComponent`)."""
         data = table_row_to_dict(self.table_components[row_idx])
-        data["row_index"] = row_idx
+        data[SourceCatalogObject._row_index_key] = row_idx
         return SourceCatalogObjectHGPSComponent(data=data)
 
 
@@ -755,7 +729,7 @@ class SourceCatalogLargeScaleHGPS:
 
         Parameters
         ----------
-        glon : `~astropy.coordinates.Longitude`
+        glon : `~astropy.coordinates.Angle`
             Galactic Longitude.
         """
         return self._interpolate_parameter("Surface_Brightness", glon)
@@ -765,7 +739,7 @@ class SourceCatalogLargeScaleHGPS:
 
         Parameters
         ----------
-        glon : `~astropy.coordinates.Longitude` or `~astropy.coordinates.Angle`
+        glon : `~astropy.coordinates.Angle`
             Galactic Longitude.
         """
         return self._interpolate_parameter("Surface_Brightness_Err", glon)
@@ -775,7 +749,7 @@ class SourceCatalogLargeScaleHGPS:
 
         Parameters
         ----------
-        glon : `~astropy.coordinates.Longitude` or `~astropy.coordinates.Angle`
+        glon : `~astropy.coordinates.Angle`
             Galactic Longitude.
         """
         return self._interpolate_parameter("Width", glon)
@@ -785,7 +759,7 @@ class SourceCatalogLargeScaleHGPS:
 
         Parameters
         ----------
-        glon : `~astropy.coordinates.Longitude` or `~astropy.coordinates.Angle`
+        glon : `~astropy.coordinates.Angle`
             Galactic Longitude.
         """
         return self._interpolate_parameter("Width_Err", glon)
@@ -795,7 +769,7 @@ class SourceCatalogLargeScaleHGPS:
 
         Parameters
         ----------
-        glon : `~astropy.coordinates.Longitude` or `~astropy.coordinates.Angle`
+        glon : `~astropy.coordinates.Angle`
             Galactic Longitude.
         """
         return self._interpolate_parameter("GLAT", glon)
@@ -805,7 +779,7 @@ class SourceCatalogLargeScaleHGPS:
 
         Parameters
         ----------
-        glon : `~astropy.coordinates.Longitude` or `~astropy.coordinates.Angle`
+        glon : `~astropy.coordinates.Angle`
             Galactic Longitude.
         """
         return self._interpolate_parameter("GLAT_Err", glon)
