@@ -1,19 +1,22 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+import collections.abc
 import logging
 import numpy as np
-from collections import OrderedDict
 from astropy.coordinates import SkyCoord
-from astropy.units import Quantity
 from astropy.time import Time
+from astropy.units import Quantity
+from gammapy.irf import Background3D
+from gammapy.utils.fits import earth_location_from_dict
+from gammapy.utils.table import table_row_to_dict
+from gammapy.utils.testing import Checker
+from gammapy.utils.time import time_ref_from_dict
+from ..irf import load_cta_irfs
 from .event_list import EventListChecker
-from ..utils.testing import Checker
-from ..utils.fits import earth_location_from_dict
-from ..utils.table import table_row_to_dict
-from ..utils.time import time_ref_from_dict
 from .filters import ObservationFilter
+from .gti import GTI
 from .pointing import FixedPointingInfo
 
-__all__ = ["DataStoreObservation", "Observations"]
+__all__ = ["DataStoreObservation", "Observations", "Observation"]
 
 log = logging.getLogger(__name__)
 
@@ -34,21 +37,21 @@ class DataStoreObservation:
     def __init__(self, obs_id, data_store, obs_filter=None):
         # Assert that `obs_id` is available
         if obs_id not in data_store.obs_table["OBS_ID"]:
-            raise ValueError("OBS_ID = {} not in obs index table.".format(obs_id))
+            raise ValueError(f"OBS_ID = {obs_id} not in obs index table.")
         if obs_id not in data_store.hdu_table["OBS_ID"]:
-            raise ValueError("OBS_ID = {} not in HDU index table.".format(obs_id))
+            raise ValueError(f"OBS_ID = {obs_id} not in HDU index table.")
 
         self.obs_id = obs_id
         self.data_store = data_store
         self.obs_filter = obs_filter or ObservationFilter()
 
     def __str__(self):
-        ss = "Info for OBS_ID = {}\n".format(self.obs_id)
-        ss += "- Start time: {:.2f}\n".format(self.tstart.mjd)
+        ss = f"Info for OBS_ID = {self.obs_id}\n"
+        ss += f"- Start time: {self.tstart.mjd:.2f}\n"
         ss += "- Pointing pos: RA {:.2f} / Dec {:.2f}\n".format(
             self.pointing_radec.ra, self.pointing_radec.dec
         )
-        ss += "- Observation duration: {}\n".format(self.observation_time_duration)
+        ss += f"- Observation duration: {self.observation_time_duration}\n"
         ss += "- Dead-time fraction: {:5.3f} %\n".format(
             100 * self.observation_dead_time_fraction
         )
@@ -136,7 +139,7 @@ class DataStoreObservation:
 
     @property
     def obs_info(self):
-        """Observation information (`~collections.OrderedDict`)."""
+        """Observation information (`dict`)."""
         row = self.data_store.obs_table.select_obs_id(obs_id=self.obs_id)[0]
         return table_row_to_dict(row)
 
@@ -229,9 +232,36 @@ class DataStoreObservation:
         """Observation muon efficiency."""
         return self.obs_info["MUONEFF"]
 
-    def peek(self):
+    def peek(self, figsize=(12, 10)):
         """Quick-look plots in a few panels."""
-        raise NotImplementedError
+        import matplotlib.pyplot as plt
+
+        fig, ((ax_aeff, ax_bkg), (ax_psf, ax_edisp)) = plt.subplots(
+            nrows=2,
+            ncols=2,
+            figsize=figsize,
+            gridspec_kw={"wspace": 0.25, "hspace": 0.25},
+        )
+
+        self.aeff.plot(ax=ax_aeff)
+
+        try:
+            if isinstance(self.bkg, Background3D):
+                bkg = self.bkg.to_2d()
+            else:
+                bkg = self.bkg
+
+            bkg.plot(ax=ax_bkg)
+        except IndexError:
+            logging.warning(f"No background model found for obs {self.obs_id}.")
+
+        self.psf.plot_containment_vs_energy(ax=ax_psf)
+        self.edisp.plot_bias(ax=ax_edisp, add_cbar=True)
+
+        ax_aeff.set_title("Effective area")
+        ax_bkg.set_title("Background rate")
+        ax_psf.set_title("Point spread function")
+        ax_edisp.set_title("Energy dispersion")
 
     def select_time(self, time_interval):
         """Select a time interval of the observation.
@@ -263,23 +293,26 @@ class DataStoreObservation:
         return checker.run(checks=checks)
 
 
-class Observations:
+class Observations(collections.abc.Sequence):
     """Container class that holds a list of observations.
 
     Parameters
     ----------
-    obs_list : list
+    observations : list
         A list of `~gammapy.data.DataStoreObservation`
     """
 
-    def __init__(self, obs_list=None):
-        self.list = obs_list or []
+    def __init__(self, observations=None):
+        self._observations = observations or []
 
     def __getitem__(self, key):
-        return self.list[key]
+        if isinstance(key, str):
+            key = self.ids.index(key)
+
+        return self._observations[key]
 
     def __len__(self):
-        return len(self.list)
+        return len(self._observations)
 
     def __str__(self):
         s = self.__class__.__name__ + "\n"
@@ -288,29 +321,213 @@ class Observations:
             s += str(obs)
         return s
 
-    def select_time(self, time_interval):
+    @property
+    def ids(self):
+        """List of obs IDs (`list`)"""
+        return [str(obs.obs_id) for obs in self]
+
+    def select_time(self, time_intervals):
         """Select a time interval of the observations.
 
         Parameters
         ----------
-        time_interval : `astropy.time.Time`
-            Start and stop time of the selected time interval.
-            For now we only support a single time interval.
+        time_intervals : `astropy.time.Time` or list of `astropy.time.Time`
+            list of Start and stop time of the time intervals or one Time interval
 
         Returns
         -------
         new_observations : `~gammapy.data.Observations`
-            A new observations instance of the specified time interval
+            A new Observations instance of the specified time intervals
         """
         new_obs_list = []
-        for obs in self:
-            new_obs = obs.select_time(time_interval)
-            if len(new_obs.gti.table) > 0:
-                new_obs_list.append(new_obs)
+        if isinstance(time_intervals, Time):
+            time_intervals = [time_intervals]
+
+        for time_interval in time_intervals:
+            for obs in self:
+                if (obs.tstart < time_interval[1]) & (obs.tstop > time_interval[0]):
+                    new_obs = obs.select_time(time_interval)
+                    new_obs_list.append(new_obs)
 
         return self.__class__(new_obs_list)
 
 
+class Observation:
+    """In-memory observation.
+
+    Parameters
+    ----------
+    obs_id : int
+        Observation ID as identifier
+    pointing : `~astropy.coordinates.SkyCoord`
+        Pointing position in icrs coordinates
+    aeff : `~gammapy.irf.EffectiveAreaTable2D`
+        Effective area used for simulating the observation
+    edisp : `~gammapy.irf.EnergyDispersion2D`
+        Energy dispersion IRF for simulating the observation
+    psf : `~gammapy.irf.PSF3D`
+        PSF IRF  used for simulating the observation
+    bkg : `~gammapy.irf.Background3D`
+        Background rate model
+    gti : `~gammapy.data.GTI`
+        Table with GTI start and stop time
+    deadtime : float, optional
+        Deadtime fraction, defaults to 0
+    """
+
+    def __init__(
+        self,
+        obs_id=None,
+        gti=None,
+        pointing=None,
+        aeff=None,
+        edisp=None,
+        psf=None,
+        bkg=None,
+        deadtime_fraction=0.0,
+    ):
+        self.obs_id = obs_id
+        self.pointing_radec = pointing
+        self.aeff = aeff
+        self.edisp = edisp
+        self.psf = psf
+        self.bkg = bkg
+        self.gti = gti
+        self.observation_dead_time_fraction = deadtime_fraction
+
+    def __str__(self):
+        ss = "Info for OBS_ID = {}\n".format(self.obs_id)
+
+        ss += "- Pointing pos: RA {:.2f} / Dec {:.2f}\n".format(
+            self.pointing_radec.ra, self.pointing_radec.dec
+        )
+
+        ss += "- Livetime duration: {}\n".format(self.observation_live_time_duration)
+        return ss
+
+    @property
+    def tstart(self):
+        return self.gti.time_start[0]
+
+    @property
+    def tstop(self):
+        return self.gti.time_stop[0]
+
+    @property
+    def observation_time_duration(self):
+        return self.gti.time_delta[0]
+
+    @property
+    def observation_live_time_duration(self):
+        return self.observation_time_duration * (
+            1 - self.observation_dead_time_fraction
+        )
+
+    @classmethod
+    def create(
+        cls,
+        pointing,
+        obs_id=None,
+        livetime=None,
+        tstart=None,
+        tstop=None,
+        irfs=None,
+        deadtime_fraction=0.0,
+    ):
+        """Create an observation.
+
+        User must either provide the livetime, or the start and stop times.
+
+        Parameters
+        ----------
+        pointing : `~astropy.coordinates.SkyCoord`
+            Pointing position
+        obs_id : int
+            Observation ID as identifier
+        livetime : ~astropy.units.Quantity`
+            Livetime exposure of the simulated observation
+        tstart : `~astropy.units.Quantity`
+            Start time of observation
+        tstop : `~astropy.units.Quantity`
+            Stop time of observation
+        irfs : dict
+            IRFs used for simulating the observation: `bkg`, `aeff`, `psf`, `edisp`
+        deadtime_fraction : float, optional
+            Deadtime fraction, defaults to 0
+
+        Returns
+        -------
+        obs : `gammapy.data.Observation`
+        """
+        tstart = tstart or Quantity(0.0, "hr")
+        tstop = (tstart + livetime) or tstop
+        gti = GTI.create([tstart], [tstop])
+        obs_id = obs_id or 1
+
+        return cls(
+            obs_id=obs_id,
+            pointing=pointing.icrs,
+            gti=gti,
+            aeff=irfs.get("aeff"),
+            bkg=irfs.get("bkg"),
+            edisp=irfs.get("edisp"),
+            psf=irfs.get("psf"),
+            deadtime_fraction=deadtime_fraction,
+        )
+
+    @classmethod
+    def from_caldb(
+        cls,
+        pointing,
+        obs_id=None,
+        livetime=None,
+        tstart=None,
+        tstop=None,
+        caldb="prod2",
+        irf="South0.5hr",
+        deadtime_fraction=0.0,
+    ):
+        """Create an observation using IRFs from a given CTA CALDB.
+
+        Parameters
+        ----------
+        pointing : `~astropy.coordinates.SkyCoord`
+            Pointing position
+        obs_id : int
+            Observation ID as identifier
+        livetime : ~astropy.units.Quantity`
+            Livetime exposure of the simulated observation
+        tstart : `~astropy.units.Quantity`
+            Start time of observation
+        tstop : `~astropy.units.Quantity`
+            Stop time of observation
+        caldb : str
+            Calibration database
+        irf : str
+            Type of Instrumental response function.
+        deadtime_fraction : float, optional
+            Deadtime fraction, defaults to 0
+
+        Returns
+        -------
+        obs : `gammapy.data.Observation`
+        """
+        from .data_store import CalDBIRF
+
+        irf_loc = CalDBIRF("CTA", caldb, irf)
+        filename = irf_loc.file_dir + irf_loc.file_name
+        irfs = load_cta_irfs(filename)
+        cls.create(
+            pointing=pointing,
+            obs_id=obs_id,
+            livetime=livetime,
+            tstart=tstart,
+            tstop=tstop,
+            irfs=irfs,
+            deadtime_fraction=deadtime_fraction,
+        )
+
+
 class ObservationChecker(Checker):
     """Check an observation.
 
@@ -358,9 +575,7 @@ class ObservationChecker(Checker):
         columns_required = ["START", "STOP"]
         for name in columns_required:
             if name not in gti.table.colnames:
-                yield self._record(
-                    level="error", msg="Missing table column: {!r}".format(name)
-                )
+                yield self._record(level="error", msg=f"Missing table column: {name!r}")
 
         # TODO: Check that header keywords agree with table entries
         # TSTART, TSTOP, MJDREFI, MJDREFF
@@ -381,9 +596,10 @@ class ObservationChecker(Checker):
         """
         # http://fermi.gsfc.nasa.gov/ssc/data/analysis/documentation/Cicerone/Cicerone_Data/Time_in_ScienceTools.html
         # https://hess-confluence.desy.de/confluence/display/HESS/HESS+FITS+data+-+References+and+checks#HESSFITSdata-Referencesandchecks-Time
-        telescope_met_refs = OrderedDict(
-            FERMI=Time("2001-01-01T00:00:00"), HESS=Time("2001-01-01T00:00:00")
-        )
+        telescope_met_refs = {
+            "FERMI": Time("2001-01-01T00:00:00"),
+            "HESS": Time("2001-01-01T00:00:00"),
+        }
 
         meta = self.dset.event_list.table.meta
         telescope = meta["TELESCOP"]
