@@ -1,15 +1,13 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """Functions to compute TS images."""
+import functools
 import logging
-import contextlib
 import warnings
-from functools import partial
-from multiprocessing import Pool
 import numpy as np
-from scipy.optimize import newton, brentq
+import scipy.optimize
 from astropy.convolution import CustomKernel, Kernel2D
-from ..stats import cash, cash_sum_cython
-from ..utils.array import shape_2N, symmetric_crop_pad_width
+from gammapy.stats import cash, cash_sum_cython
+from gammapy.utils.array import shape_2N, symmetric_crop_pad_width
 from ._test_statistics_cython import (
     _amplitude_bounds_cython,
     _f_cash_root_cython,
@@ -72,7 +70,6 @@ def f_cash(x, counts, background, model):
 class TSMapEstimator:
     r"""Compute TS map using different optimization methods.
 
-
     The map is computed fitting by a single parameter amplitude fit. The fit is
     simplified by finding roots of the the derivative of the fit statistics using
     various root finding algorithms. The approach is sescribed in Appendix A
@@ -100,8 +97,6 @@ class TSMapEstimator:
         Upper limit estimation method.
     ul_sigma : int (2)
         Sigma for flux upper limits.
-    n_jobs : int
-        Number of parallel jobs to use for the computation.
     threshold : float (None)
         If the TS value corresponding to the initial flux estimate is not above
         this threshold, the optimizing step is omitted to save computing time.
@@ -135,16 +130,15 @@ class TSMapEstimator:
         error_sigma=1,
         ul_method="covar",
         ul_sigma=2,
-        n_jobs=1,
         threshold=None,
         rtol=0.001,
     ):
 
         if method not in ["root brentq", "root newton", "leastsq iter"]:
-            raise ValueError("Not a valid method: '{}'".format(method))
+            raise ValueError(f"Not a valid method: '{method}'")
 
         if error_method not in ["covar", "conf"]:
-            raise ValueError("Not a valid error method '{}'".format(error_method))
+            raise ValueError(f"Not a valid error method '{error_method}'")
 
         self.parameters = {
             "method": method,
@@ -152,7 +146,6 @@ class TSMapEstimator:
             "error_sigma": error_sigma,
             "ul_method": ul_method,
             "ul_sigma": ul_sigma,
-            "n_jobs": n_jobs,
             "threshold": threshold,
             "rtol": rtol,
         }
@@ -316,7 +309,7 @@ class TSMapEstimator:
         error_method = p["error_method"] if "flux_err" in which else "none"
         ul_method = p["ul_method"] if "flux_ul" in which else "none"
 
-        wrap = partial(
+        wrap = functools.partial(
             _ts_value,
             counts=counts,
             exposure=exposure,
@@ -336,11 +329,7 @@ class TSMapEstimator:
         x, y = np.where(mask.data)
         positions = list(zip(x, y))
 
-        with contextlib.closing(Pool(processes=p["n_jobs"])) as pool:
-            log.info("Using {} jobs to compute TS map.".format(p["n_jobs"]))
-            results = pool.map(wrap, positions)
-
-        pool.join()
+        results = list(map(wrap, positions))
 
         # Set TS values at given positions
         j, i = zip(*positions)
@@ -372,7 +361,7 @@ class TSMapEstimator:
         info = self.__class__.__name__
         info += "\n\nParameters:\n\n"
         for key in p:
-            info += "\t{key:13s}: {value}\n".format(key=key, value=p[key])
+            info += f"\t{key:13s}: {p[key]}\n"
         return info
 
 
@@ -454,7 +443,7 @@ def _ts_value(
             counts_, background_, model, rtol=rtol
         )
     else:
-        raise ValueError("Invalid method: {}".format(method))
+        raise ValueError(f"Invalid method: {method}")
 
     with np.errstate(invalid="ignore", divide="ignore"):
         c_1 = f_cash(amplitude, counts_, background_, model)
@@ -553,7 +542,7 @@ def _root_amplitude(counts, background, model, flux, rtol=RTOL):
         warnings.simplefilter("ignore")
         try:
             return (
-                newton(
+                scipy.optimize.newton(
                     _f_cash_root_cython, flux, args=args, maxiter=MAX_NITER, tol=rtol
                 ),
                 0,
@@ -595,7 +584,7 @@ def _root_amplitude_brentq(counts, background, model, rtol=RTOL):
     with warnings.catch_warnings():
         warnings.simplefilter("ignore")
         try:
-            result = brentq(
+            result = scipy.optimize.brentq(
                 _f_cash_root_cython,
                 amplitude_min,
                 amplitude_max,
@@ -633,7 +622,7 @@ def _compute_flux_err_conf(amplitude, counts, background, model, c_1, error_sigm
     with warnings.catch_warnings():
         warnings.simplefilter("ignore")
         try:
-            result = brentq(
+            result = scipy.optimize.brentq(
                 ts_diff,
                 amplitude,
                 amplitude_max,
