@@ -1,17 +1,15 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from collections import OrderedDict
 import numpy as np
-from scipy.special import erf
-from astropy.io import fits
+import scipy.special
 from astropy.coordinates import Angle
-from astropy.units import Quantity
+from astropy.io import fits
 from astropy.table import Table
-from ..maps import MapAxis
-from ..maps.utils import edges_from_lo_hi
-from ..utils.energy import EnergyBounds, Energy
-from ..utils.scripts import make_path
-from ..utils.nddata import NDDataArray
-from ..utils.fits import energy_axis_to_ebounds
+from astropy.units import Quantity
+from gammapy.maps import MapAxis
+from gammapy.maps.utils import edges_from_lo_hi
+from gammapy.utils.fits import energy_axis_to_ebounds
+from gammapy.utils.nddata import NDDataArray
+from gammapy.utils.scripts import make_path
 
 __all__ = ["EnergyDispersion", "EnergyDispersion2D"]
 
@@ -79,11 +77,11 @@ class EnergyDispersion:
         self.data = NDDataArray(
             axes=[e_true_axis, e_reco_axis], data=data, interp_kwargs=interp_kwargs
         )
-        self.meta = OrderedDict(meta) if meta else OrderedDict()
+        self.meta = meta or {}
 
     def __str__(self):
         ss = self.__class__.__name__
-        ss += "\n{}".format(self.data)
+        ss += f"\n{self.data}"
         return ss
 
     def apply(self, data):
@@ -105,9 +103,7 @@ class EnergyDispersion:
         """
         if len(data) != self.e_true.nbin:
             raise ValueError(
-                "Input size {} does not match true energy axis {}".format(
-                    len(data), self.e_true.nbin
-                )
+                f"Input size {len(data)} does not match true energy axis {self.e_true.nbin}"
             )
         return np.dot(data, self.data.data)
 
@@ -142,7 +138,11 @@ class EnergyDispersion:
         """
         data = self.pdf_matrix.copy()
         energy = self.e_reco.edges
-        idx = np.where((energy[:-1] < lo_threshold) | (energy[1:] > hi_threshold))
+
+        if lo_threshold is None and hi_threshold is None:
+            idx = slice(None)
+        else:
+            idx = (energy[:-1] < lo_threshold) | (energy[1:] > hi_threshold)
         data[:, idx] = 0
         return data
 
@@ -260,14 +260,19 @@ class EnergyDispersion:
                     ] = l.field("MATRIX")[m_start : m_start + l.field("N_CHAN")[k]]
                     m_start += l.field("N_CHAN")[k]
 
-        e_reco = EnergyBounds.from_ebounds(ebounds_hdu)
-        e_true = EnergyBounds.from_rmf_matrix(matrix_hdu)
+        unit = ebounds_hdu.header.get("TUNIT2")
+        e_reco_lo = Quantity(ebounds_hdu.data["E_MIN"], unit=unit)
+        e_reco_hi = Quantity(ebounds_hdu.data["E_MAX"], unit=unit)
+
+        unit = matrix_hdu.header.get("TUNIT1")
+        e_true_lo = Quantity(matrix_hdu.data["ENERG_LO"], unit=unit)
+        e_true_hi = Quantity(matrix_hdu.data["ENERG_HI"], unit=unit)
 
         return cls(
-            e_true_lo=e_true.lower_bounds,
-            e_true_hi=e_true.upper_bounds,
-            e_reco_lo=e_reco.lower_bounds,
-            e_reco_hi=e_reco.upper_bounds,
+            e_true_lo=e_true_lo,
+            e_true_hi=e_true_hi,
+            e_reco_lo=e_reco_lo,
+            e_reco_hi=e_reco_hi,
             data=pdf_matrix,
         )
 
@@ -284,11 +289,8 @@ class EnergyDispersion:
         hdu2 : str, optional
             HDU containing the energy axis information, default, EBOUNDS
         """
-        filename = make_path(filename)
-        with fits.open(str(filename), memmap=False) as hdulist:
-            edisp = cls.from_hdulist(hdulist, hdu1=hdu1, hdu2=hdu2)
-
-        return edisp
+        with fits.open(make_path(filename), memmap=False) as hdulist:
+            return cls.from_hdulist(hdulist, hdu1=hdu1, hdu2=hdu2)
 
     def to_hdulist(self, use_sherpa=False, **kwargs):
         """Convert RMF to FITS HDU list format.
@@ -325,10 +327,10 @@ class EnergyDispersion:
 
         cols = table.columns
         c0 = fits.Column(
-            name=cols[0].name, format="E", array=cols[0], unit="{}".format(cols[0].unit)
+            name=cols[0].name, format="E", array=cols[0], unit=str(cols[0].unit)
         )
         c1 = fits.Column(
-            name=cols[1].name, format="E", array=cols[1], unit="{}".format(cols[1].unit)
+            name=cols[1].name, format="E", array=cols[1], unit=str(cols[1].unit)
         )
         c2 = fits.Column(name=cols[2].name, format="I", array=cols[2])
         c3 = fits.Column(name=cols[3].name, format="PI()", array=cols[3])
@@ -397,26 +399,24 @@ class EnergyDispersion:
         table["N_CHAN"] = n_chan
         table["MATRIX"] = matrix
 
-        table.meta = OrderedDict(
-            [
-                ("name", "MATRIX"),
-                ("chantype", "PHA"),
-                ("hduclass", "OGIP"),
-                ("hduclas1", "RESPONSE"),
-                ("hduclas2", "RSP_MATRIX"),
-                ("detchans", self.e_reco.nbin),
-                ("numgrp", numgrp),
-                ("numelt", numelt),
-                ("tlmin4", 0),
-            ]
-        )
+        table.meta = {
+            "name": "MATRIX",
+            "chantype": "PHA",
+            "hduclass": "OGIP",
+            "hduclas1": "RESPONSE",
+            "hduclas2": "RSP_MATRIX",
+            "detchans": self.e_reco.nbin,
+            "numgrp": numgrp,
+            "numelt": numelt,
+            "tlmin4": 0,
+        }
 
         return table
 
     def write(self, filename, use_sherpa=False, **kwargs):
         """Write to file."""
         filename = make_path(filename)
-        self.to_hdulist(use_sherpa=use_sherpa).writeto(str(filename), **kwargs)
+        self.to_hdulist(use_sherpa=use_sherpa).writeto(filename, **kwargs)
 
     def get_resolution(self, e_true):
         """Get energy resolution for a given true energy.
@@ -454,9 +454,9 @@ class EnergyDispersion:
     def get_bias_energy(self, bias, emin=None, emax=None):
         """Find energy corresponding to a given bias.
 
-        In case the solution is not unique, provide the `emin` or `emax` arguments
+        In case the solution is not unique, provide the ``emin`` or ``emax`` arguments
         to limit the solution to the given range.  By default the peak energy of the
-        bias is chosen as `emin`.
+        bias is chosen as ``emin``.
 
         Parameters
         ----------
@@ -469,10 +469,10 @@ class EnergyDispersion:
 
         Returns
         -------
-        bias_energy : `~astropy.unit.Quantity`
+        bias_energy : `~astropy.units.Quantity`
             Reconstructed energy corresponding to the given bias.
         """
-        from ..spectrum.models import TableModel
+        from gammapy.modeling.models import TemplateSpectralModel
 
         e_true = self.e_true.center
         values = self.get_bias(e_true)
@@ -483,7 +483,7 @@ class EnergyDispersion:
         if emax is None:
             emax = e_true[-1]
 
-        bias_spectrum = TableModel(e_true, values)
+        bias_spectrum = TemplateSpectralModel(e_true, values)
         e_true_bias = bias_spectrum.inverse(Quantity(bias), emin=emin, emax=emax)
 
         # return reconstructed energy
@@ -535,52 +535,6 @@ class EnergyDispersion:
 
         return var / norm
 
-    def to_sherpa(self, name):
-        """Convert to `sherpa.astro.data.DataRMF`.
-
-        Parameters
-        ----------
-        name : str
-            Instance name
-        """
-        from sherpa.astro.data import DataRMF
-        from sherpa.utils import SherpaUInt, SherpaFloat
-
-        # Need to modify RMF data
-        # see https://github.com/sherpa/sherpa/blob/master/sherpa/astro/io/pyfits_backend.py#L727
-
-        table = self.to_table()
-        n_grp = table["N_GRP"].data.astype(SherpaUInt)
-        f_chan = table["F_CHAN"].data
-        n_chan = table["N_CHAN"].data
-        matrix = table["MATRIX"].data
-
-        good = n_grp > 0
-        matrix = matrix[good]
-        matrix = np.concatenate([row for row in matrix])
-        matrix = matrix.astype(SherpaFloat)
-
-        good = n_grp > 0
-        f_chan = f_chan[good]
-        f_chan = np.concatenate([row for row in f_chan]).astype(SherpaUInt)
-        n_chan = n_chan[good]
-        n_chan = np.concatenate([row for row in n_chan]).astype(SherpaUInt)
-
-        energy = self.e_reco.edges.to_value("keV")
-        return DataRMF(
-            name=name,
-            energ_lo=table["ENERG_LO"].quantity.to_value("keV").astype(SherpaFloat),
-            energ_hi=table["ENERG_HI"].quantity.to_value("keV").astype(SherpaFloat),
-            matrix=matrix,
-            n_grp=n_grp,
-            n_chan=n_chan,
-            f_chan=f_chan,
-            detchans=self.e_reco.nbin,
-            e_min=energy[:-1],
-            e_max=energy[1:],
-            offset=0,
-        )
-
     def plot_matrix(self, ax=None, show_energy=None, add_cbar=False, **kwargs):
         """Plot PDF matrix.
 
@@ -622,8 +576,8 @@ class EnergyDispersion:
             label = "Probability density (A.U.)"
             cbar = ax.figure.colorbar(caxes, ax=ax, label=label)
 
-        ax.set_xlabel(r"$E_\mathrm{{True}}$ [{unit}]".format(unit=e_true.unit))
-        ax.set_ylabel(r"$E_\mathrm{{Reco}}$ [{unit}]".format(unit=e_reco.unit))
+        ax.set_xlabel(fr"$E_\mathrm{{True}}$ [{e_true.unit}]")
+        ax.set_ylabel(fr"$E_\mathrm{{Reco}}$ [{e_reco.unit}]")
         ax.set_xscale("log")
         ax.set_yscale("log")
         ax.set_xlim(x.min(), x.max())
@@ -683,29 +637,16 @@ class EnergyDispersion2D:
     --------
     Read energy dispersion IRF from disk:
 
+    >>> from gammapy.maps import MapAxis
     >>> from gammapy.irf import EnergyDispersion2D
-    >>> from gammapy.utils.energy import EnergyBounds
-    >>> filename = '$GAMMAPY_DATA/tests/irf/hess/pa/hess_edisp_2d_023523.fits.gz'
-    >>> edisp2d = EnergyDispersion2D.read(filename, hdu='ENERGY DISPERSION')
-    >>> print(edisp2d)
-    EnergyDispersion2D
-    NDDataArray summary info
-    e_true         : size =    15, min =  0.125 TeV, max = 80.000 TeV
-    migra          : size =   100, min =  0.051, max = 10.051
-    offset         : size =     6, min =  0.125 deg, max =  2.500 deg
-    Data           : size =  9000, min =  0.000, max =  3.405
+    >>> filename = '$GAMMAPY_DATA/hess-dl3-dr1/data/hess_dl3_dr1_obs_id_020136.fits.gz'
+    >>> edisp2d = EnergyDispersion2D.read(filename, hdu="EDISP")
 
     Create energy dispersion matrix (`~gammapy.irf.EnergyDispersion`)
     for a given field of view offset and energy binning:
 
-    >>> energy = EnergyBounds.equal_log_spacing(0.1,20,60, 'TeV')
+    >>> energy = MapAxis.from_bounds(0.1, 20, nbin=60, unit="TeV", interp="log").edges
     >>> edisp = edisp2d.to_energy_dispersion(offset='1.2 deg', e_reco=energy, e_true=energy)
-    >>> print(edisp)
-    EnergyDispersion
-    NDDataArray summary info
-    e_true         : size =    60, min =  0.105 TeV, max = 19.136 TeV
-    e_reco         : size =    60, min =  0.105 TeV, max = 19.136 TeV
-    Data           : size =  3600, min =  0.000, max =  0.266
 
     See Also
     --------
@@ -748,11 +689,11 @@ class EnergyDispersion2D:
         axes = [e_true_axis, migra_axis, offset_axis]
 
         self.data = NDDataArray(axes=axes, data=data, interp_kwargs=interp_kwargs)
-        self.meta = OrderedDict(meta) if meta else OrderedDict()
+        self.meta = meta or {}
 
     def __str__(self):
         ss = self.__class__.__name__
-        ss += "\n{}".format(self.data)
+        ss += f"\n{self.data}"
         return ss
 
     @classmethod
@@ -782,9 +723,9 @@ class EnergyDispersion2D:
         pdf_threshold : float, optional
             Zero suppression threshold
         """
-        e_true = EnergyBounds(e_true)
+        e_true = Quantity(e_true)
         # erf does not work with Quantities
-        true = e_true.log_centers.to_value("TeV")
+        true = MapAxis.from_edges(e_true, interp="log").center.to_value("TeV")
 
         true2d, migra2d = np.meshgrid(true, migra)
 
@@ -795,7 +736,7 @@ class EnergyDispersion2D:
         s = np.sqrt(2) * sigma
         t1 = (migra2d_hi - 1 - bias) / s
         t2 = (migra2d_lo - 1 - bias) / s
-        pdf = (erf(t1) - erf(t2)) / 2
+        pdf = (scipy.special.erf(t1) - scipy.special.erf(t2)) / 2
 
         pdf_array = pdf.T[:, :, np.newaxis] * np.ones(len(offset) - 1)
 
@@ -856,11 +797,8 @@ class EnergyDispersion2D:
         filename : str
             File name
         """
-        filename = make_path(filename)
-        with fits.open(str(filename), memmap=False) as hdulist:
-            edisp = cls.from_hdulist(hdulist, hdu)
-
-        return edisp
+        with fits.open(make_path(filename), memmap=False) as hdulist:
+            return cls.from_hdulist(hdulist, hdu)
 
     def to_energy_dispersion(self, offset, e_true=None, e_reco=None):
         """Detector response R(Delta E_reco, Delta E_true)
@@ -872,9 +810,9 @@ class EnergyDispersion2D:
         ----------
         offset : `~astropy.coordinates.Angle`
             Offset
-        e_true : `~gammapy.utils.energy.EnergyBounds`, None
+        e_true : `~astropy.units.Quantity`, None
             True energy axis
-        e_reco : `~gammapy.utils.energy.EnergyBounds`
+        e_reco : `~astropy.units.Quantity`
             Reconstructed energy axis
 
         Returns
@@ -885,11 +823,9 @@ class EnergyDispersion2D:
         offset = Angle(offset)
         e_true = self.data.axis("e_true").edges if e_true is None else e_true
         e_reco = self.data.axis("e_true").edges if e_reco is None else e_reco
-        e_true = EnergyBounds(e_true)
-        e_reco = EnergyBounds(e_reco)
 
         data = []
-        for energy in e_true.log_centers:
+        for energy in MapAxis.from_edges(e_true, interp="log").center:
             vec = self.get_response(offset=offset, e_true=energy, e_reco=e_reco)
             data.append(vec)
 
@@ -914,9 +850,9 @@ class EnergyDispersion2D:
 
         Parameters
         ----------
-        e_true : `~gammapy.utils.energy.Energy`
+        e_true : `~astropy.units.Quantity`
             True energy
-        e_reco : `~gammapy.utils.energy.EnergyBounds`, None
+        e_reco : `~astropy.units.Quantity`, None
             Reconstructed energy axis
         offset : `~astropy.coordinates.Angle`
             Offset
@@ -928,14 +864,14 @@ class EnergyDispersion2D:
         rv : `~numpy.ndarray`
             Redistribution vector
         """
-        e_true = Energy(e_true)
+        e_true = Quantity(e_true)
 
         if e_reco is None:
             # Default: e_reco nodes = migra nodes * e_true nodes
-            e_reco = EnergyBounds(self.data.axis("migra").edges * e_true)
+            e_reco = self.data.axis("migra").edges * e_true
         else:
             # Translate given e_reco binning to migra at bin center
-            e_reco = EnergyBounds(e_reco)
+            e_reco = Quantity(e_reco)
 
         # migration value of e_reco bounds
         migra_e_reco = e_reco / e_true
@@ -972,9 +908,9 @@ class EnergyDispersion2D:
             Axis
         offset : `~astropy.coordinates.Angle`, optional
             Offset
-        e_true : `~gammapy.utils.energy.Energy`, optional
+        e_true : `~astropy.units.Quantity`, optional
             True energy
-        migra : `~numpy.array`, list, optional
+        migra : `~numpy.ndarray`, optional
             Migration nodes
 
         Returns
@@ -990,16 +926,18 @@ class EnergyDispersion2D:
             offset = Angle([1], "deg")
         else:
             offset = np.atleast_1d(Angle(offset))
+
         if e_true is None:
-            e_true = Energy([0.1, 1, 10], "TeV")
+            e_true = Quantity([0.1, 1, 10], "TeV")
         else:
-            e_true = np.atleast_1d(Energy(e_true))
+            e_true = np.atleast_1d(Quantity(e_true))
+
         migra = self.data.axis("migra").center if migra is None else migra
 
         for ener in e_true:
             for off in offset:
                 disp = self.data.evaluate(offset=off, e_true=ener, migra=migra)
-                label = "offset = {:.1f}\nenergy = {:.1f}".format(off, ener)
+                label = f"offset = {off:.1f}\nenergy = {ener:.1f}"
                 ax.plot(migra, disp, label=label, **kwargs)
 
         ax.set_xlabel(r"$E_\mathrm{{Reco}} / E_\mathrm{{True}}$")
@@ -1055,7 +993,7 @@ class EnergyDispersion2D:
             label = "Probability density (A.U.)"
             ax.figure.colorbar(caxes, ax=ax, label=label)
 
-        ax.set_xlabel(r"$E_\mathrm{{True}}$ [{unit}]".format(unit=e_true.unit))
+        ax.set_xlabel(fr"$E_\mathrm{{True}}$ [{e_true.unit}]")
         ax.set_ylabel(r"$E_\mathrm{{Reco}} / E_\mathrm{{True}}$")
         ax.set_xlim(x.min(), x.max())
         ax.set_ylim(y.min(), y.max())
