@@ -2,11 +2,11 @@
 import pytest
 import numpy as np
 from numpy.testing import assert_allclose, assert_equal
-from astropy.coordinates import Angle
 import astropy.units as u
-from ...utils.testing import requires_dependency, requires_data, mpl_plot_check
-from ...utils.energy import energy_logspace
-from ...irf import EnergyDispersion, EnergyDispersion2D
+from astropy.coordinates import Angle
+from gammapy.irf import EnergyDispersion, EnergyDispersion2D
+from gammapy.maps import MapAxis
+from gammapy.utils.testing import mpl_plot_check, requires_data, requires_dependency
 
 
 class TestEnergyDispersion:
@@ -53,13 +53,12 @@ class TestEnergyDispersion:
     def test_apply(self):
         counts = np.arange(len(self.e_true) - 1)
         actual = self.edisp.apply(counts)
-        assert_allclose(actual[0], 1.8612999017723058, atol=1e-3)
+        assert_allclose(actual[0], 1.862299, atol=1e-3)
 
         counts = np.arange(len(self.e_true) - 4)
-        with pytest.raises(ValueError) as excinfo:
+        with pytest.raises(ValueError):
             self.edisp.apply(counts)
-        assert "Input size 97 does not match true energy axis 100" == str(excinfo.value)
-        assert_allclose(actual[0], 1.8612999017723058, atol=1e-3)
+        assert_allclose(actual[0], 1.862299, atol=1e-3)
 
     def test_get_bias(self):
         bias = self.edisp.get_bias(3.34 * u.TeV)
@@ -69,12 +68,11 @@ class TestEnergyDispersion:
         resolution = self.edisp.get_resolution(3.34 * u.TeV)
         assert_allclose(resolution, self.resolution, atol=1e-2)
 
-    def test_io(self, tmpdir):
+    def test_io(self, tmp_path):
         indices = np.array([[1, 3, 6], [3, 3, 2]])
         desired = self.edisp.pdf_matrix[indices]
-        writename = str(tmpdir / "rmf_test.fits")
-        self.edisp.write(writename)
-        edisp2 = EnergyDispersion.read(writename)
+        self.edisp.write(tmp_path / "tmp.fits")
+        edisp2 = EnergyDispersion.read(tmp_path / "tmp.fits")
         actual = edisp2.pdf_matrix[indices]
         assert_allclose(actual, desired)
 
@@ -96,11 +94,10 @@ class TestEnergyDispersion:
 
 @requires_data()
 class TestEnergyDispersion2D:
-    def setup(self):
-        # TODO: use from_gauss method to create know edisp (see below)
-        # At the moment only 1 test uses it (test_get_response)
-        filename = "$GAMMAPY_DATA/tests/irf/hess/pa/hess_edisp_2d_023523.fits.gz"
-        self.edisp = EnergyDispersion2D.read(filename, hdu="ENERGY DISPERSION")
+    @classmethod
+    def setup_class(cls):
+        filename = "$GAMMAPY_DATA/hess-dl3-dr1/data/hess_dl3_dr1_obs_id_020136.fits.gz"
+        cls.edisp = EnergyDispersion2D.read(filename, hdu="EDISP")
 
         # Make a test case
         e_true = np.logspace(-1.0, 2.0, 51) * u.TeV
@@ -108,25 +105,12 @@ class TestEnergyDispersion2D:
         offset = np.linspace(0.0, 2.5, 5) * u.deg
         sigma = 0.15 / (e_true[:-1] / (1 * u.TeV)).value ** 0.3
         bias = 1e-3 * (e_true[:-1] - 1 * u.TeV).value
-        self.edisp2 = EnergyDispersion2D.from_gauss(e_true, migra, bias, sigma, offset)
+        cls.edisp2 = EnergyDispersion2D.from_gauss(e_true, migra, bias, sigma, offset)
 
     def test_str(self):
         assert "EnergyDispersion2D" in str(self.edisp)
 
     def test_evaluation(self):
-        # TODO: Move to tests for NDDataArray
-        # Check that nodes are evaluated correctly
-        e_node = 12
-        off_node = 3
-        m_node = 5
-        offset = self.edisp.data.axis("offset").center[off_node]
-        energy = self.edisp.data.axis("e_true").center[e_node]
-        migra = self.edisp.data.axis("migra").center[m_node]
-        actual = self.edisp.data.evaluate(offset=offset, e_true=energy, migra=migra)
-        desired = self.edisp.data.data[e_node, m_node, off_node]
-        assert_allclose(actual, desired, rtol=1e-06)
-        assert_allclose(actual, 0.09388659149, rtol=1e-06)
-
         # Check output shape
         energy = [1, 2] * u.TeV
         migra = np.array([0.98, 0.97, 0.7])
@@ -155,8 +139,8 @@ class TestEnergyDispersion2D:
     def test_exporter(self):
         # Check RMF exporter
         offset = Angle(0.612, "deg")
-        e_reco = energy_logspace(1, 10, 7, "TeV")
-        e_true = energy_logspace(0.8, 5, 5, "TeV")
+        e_reco = MapAxis.from_energy_bounds(1, 10, 7, "TeV").edges
+        e_true = MapAxis.from_energy_bounds(0.8, 5, 5, "TeV").edges
         rmf = self.edisp.to_energy_dispersion(offset, e_true=e_true, e_reco=e_reco)
         assert_allclose(rmf.data.data[2, 3], 0.08, atol=5e-2)  # same tolerance as above
         actual = rmf.pdf_matrix[2]
@@ -205,3 +189,12 @@ class TestEnergyDispersion2D:
     def test_peek(self):
         with mpl_plot_check():
             self.edisp.peek()
+
+
+@requires_data("gammapy-data")
+def test_get_bias_energy():
+    """Obs read from file"""
+    rmffile = "$GAMMAPY_DATA/joint-crab/spectra/hess/rmf_obs23523.fits"
+    edisp = EnergyDispersion.read(rmffile)
+    thresh_lo = edisp.get_bias_energy(0.1)
+    assert_allclose(thresh_lo.to("TeV").value, 0.9174, rtol=1e-4)
