@@ -2,12 +2,15 @@
 import pytest
 import numpy as np
 from numpy.testing import assert_allclose, assert_almost_equal
-from astropy.utils.data import get_pkg_data_filename
-from astropy.io import fits
 from astropy import units as u
-from ...utils.testing import requires_data
-from ..psf_gauss import EnergyDependentMultiGaussPSF
-from ..psf_gauss import multi_gauss_psf_kernel, HESSMultiGaussPSF
+from astropy.io import fits
+from astropy.utils.data import get_pkg_data_filename
+from gammapy.irf.psf_gauss import (
+    EnergyDependentMultiGaussPSF,
+    HESSMultiGaussPSF,
+    multi_gauss_psf_kernel,
+)
+from gammapy.utils.testing import requires_data
 
 
 def make_test_psf(energy_bins=15, theta_bins=12):
@@ -72,22 +75,14 @@ class TestEnergyDependentMultiGaussPSF:
         return EnergyDependentMultiGaussPSF.read(filename, hdu="POINT SPREAD FUNCTION")
 
     def test_info(self, psf):
-        filename = get_pkg_data_filename("data/psf_info.txt")
-        info_str = open(filename, "r").read()
+        info_str = open(get_pkg_data_filename("data/psf_info.txt")).read()
 
         assert psf.info() == info_str
 
-    def test_write(self, tmpdir, psf):
-        # Write it back to disk
-        filename = str(tmpdir / "multigauss_psf_test.fits")
-        psf.write(filename)
+    def test_write(self, tmp_path, psf):
+        psf.write(tmp_path / "tmp.fits")
 
-        # Verify checksum
-        with fits.open(filename) as hdu_list:
-            # TODO: replace this assert with something else.
-            # For unknown reasons this verify_checksum fails non-deterministically
-            # see e.g. https://travis-ci.org/gammapy/gammapy/jobs/31056341#L1162
-            # assert hdu_list[1].verify_checksum() == 1
+        with fits.open(tmp_path / "tmp.fits", memmap=False) as hdu_list:
             assert len(hdu_list) == 2
 
     def test_to_table_psf(self, psf):
