@@ -1,17 +1,22 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """Utilities for dealing with HEALPix projections and mappings."""
-from collections import OrderedDict
-import re
 import copy
+import re
 import numpy as np
-from astropy.io import fits
 from astropy.coordinates import SkyCoord
+from astropy.io import fits
 from astropy.units import Quantity
+from .geom import (
+    Geom,
+    MapCoord,
+    coordsys_to_frame,
+    find_and_read_bands,
+    make_axes,
+    pix_tuple_to_idx,
+    skycoord_to_lonlat,
+)
 from .utils import INVALID_INDEX
 from .wcs import WcsGeom
-from .geom import MapGeom, MapCoord, pix_tuple_to_idx
-from .geom import coordsys_to_frame, skycoord_to_lonlat
-from .geom import find_and_read_bands, make_axes
 
 # Not sure if we should expose this in the docs or not:
 # HPX_FITS_CONVENTIONS, HpxConv
@@ -37,7 +42,7 @@ class HpxConv:
         self.coordsys = kwargs.get("coordsys", "COORDSYS")
 
     def colname(self, indx):
-        return "{}{}".format(self.colstring, indx)
+        return f"{self.colstring}{indx}"
 
     @classmethod
     def create(cls, convname="gadf"):
@@ -179,7 +184,7 @@ def get_pix_size_from_nside(nside):
     """
     order = nside_to_order(nside)
     if np.any(order < 0) or np.any(order > 13):
-        raise ValueError("HEALPIX order must be 0 to 13. Got: {!r}".format(order))
+        raise ValueError(f"HEALPIX order must be 0 to 13. Got: {order!r}")
 
     return HPX_ORDER_TO_PIXSIZE[order]
 
@@ -323,7 +328,7 @@ def parse_hpxregion(region):
     m = re.match(r"([A-Za-z\_]*?)\((.*?)\)", region)
 
     if m is None:
-        raise ValueError("Failed to parse hpx region string: {!r}".format(region))
+        raise ValueError(f"Failed to parse hpx region string: {region!r}")
 
     if not m.group(1):
         return re.split(",", m.group(2))
@@ -360,13 +365,13 @@ def get_hpxregion_dir(region, coordsys):
         elif tokens[1] == "RING":
             nest_pix = False
         else:
-            raise ValueError("Invalid ordering scheme: {!r}".format(tokens[1]))
+            raise ValueError(f"Invalid ordering scheme: {tokens[1]!r}")
         theta, phi = hp.pix2ang(nside_pix, ipix_pix, nest_pix)
         lat = np.degrees((np.pi / 2) - theta)
         lon = np.degrees(phi)
         return SkyCoord(lon, lat, frame=frame, unit="deg")
     else:
-        raise ValueError("Invalid region type: {!r}".format(tokens[0]))
+        raise ValueError(f"Invalid region type: {tokens[0]!r}")
 
 
 def get_hpxregion_size(region):
@@ -379,7 +384,7 @@ def get_hpxregion_size(region):
         pix_size = get_pix_size_from_nside(int(tokens[2]))
         return 2.0 * pix_size
     else:
-        raise ValueError("Invalid region type: {!r}".format(tokens[0]))
+        raise ValueError(f"Invalid region type: {tokens[0]!r}")
 
 
 def is_power2(n):
@@ -518,7 +523,7 @@ def get_subpixels(idx, nside_superpix, nside_subpix, nest=True):
     return idx
 
 
-class HpxGeom(MapGeom):
+class HpxGeom(Geom):
     """Geometry class for HEALPIX maps.
 
     This class performs mapping between partial-sky indices (pixel
@@ -552,8 +557,6 @@ class HpxGeom(MapGeom):
         encompassed by the geometry.
     axes : list
         Axes for non-spatial dimensions.
-    conv : str
-        Convention for FITS serialization format.
     sparse : bool
         If True defer allocation of partial- to all-sky index mapping
         arrays.  This option is only compatible with partial-sky maps
@@ -563,21 +566,14 @@ class HpxGeom(MapGeom):
     is_hpx = True
 
     def __init__(
-        self,
-        nside,
-        nest=True,
-        coordsys="CEL",
-        region=None,
-        axes=None,
-        conv="gadf",
-        sparse=False,
+        self, nside, nest=True, coordsys="CEL", region=None, axes=None, sparse=False
     ):
 
         # FIXME: Figure out what to do when sparse=True
         # FIXME: Require NSIDE to be power of two when nest=True
 
         self._nside = np.array(nside, ndmin=1)
-        self._axes = make_axes(axes, conv)
+        self._axes = make_axes(axes)
         if self.nside.size > 1 and self.nside.shape != self.shape_axes:
             raise ValueError(
                 "Wrong dimensionality for nside. nside must "
@@ -603,7 +599,6 @@ class HpxGeom(MapGeom):
                 self._rmap[ipix] = i
 
         self._npix = self._npix * np.ones(self.shape_axes, dtype=int)
-        self._conv = conv
         self._center_skydir = self._get_ref_dir()
         lon, lat, frame = skycoord_to_lonlat(self._center_skydir)
         self._center_coord = tuple(
@@ -665,7 +660,7 @@ class HpxGeom(MapGeom):
             self._npix = self._maxpix
 
         else:
-            raise ValueError("Invalid region string: {!r}".format(region))
+            raise ValueError(f"Invalid region string: {region!r}")
 
     def local_to_global(self, idx_local):
         """Compute a local index (partial-sky) from a global (all-sky) index.
@@ -711,7 +706,7 @@ class HpxGeom(MapGeom):
 
         if self._rmap is not None:
             retval = np.full(idx.size, -1, "i")
-            m = np.in1d(idx.flat, self._ipix)
+            m = np.isin(idx.flat, self._ipix)
             retval[m] = np.searchsorted(self._ipix, idx.flat[m])
             retval = retval.reshape(idx.shape)
         else:
@@ -884,7 +879,7 @@ class HpxGeom(MapGeom):
         else:
             region = self.region
 
-        return self.__class__(nside, self.nest, self.coordsys, region, axes, self.conv)
+        return self.__class__(nside, self.nest, self.coordsys, region, axes)
 
     @property
     def axes(self):
@@ -933,15 +928,6 @@ class HpxGeom(MapGeom):
         """
         return self._npix
 
-    @property
-    def conv(self):
-        """Name of default FITS convention associated with this geometry."""
-        return self._conv
-
-    @property
-    def hpx_conv(self):
-        return HPX_FITS_CONVENTIONS[self.conv]
-
     @property
     def coordsys(self):
         return self._coordsys
@@ -1016,12 +1002,7 @@ class HpxGeom(MapGeom):
 
         axes = copy.deepcopy(self.axes)
         return self.__class__(
-            2 ** order,
-            self.nest,
-            coordsys=self.coordsys,
-            region=self.region,
-            axes=axes,
-            conv=self.conv,
+            2 ** order, self.nest, coordsys=self.coordsys, region=self.region, axes=axes
         )
 
     def to_swapped(self):
@@ -1039,16 +1020,11 @@ class HpxGeom(MapGeom):
             coordsys=self.coordsys,
             region=self.region,
             axes=axes,
-            conv=self.conv,
         )
 
     def to_image(self):
         return self.__class__(
-            np.max(self.nside),
-            self.nest,
-            coordsys=self.coordsys,
-            region=self.region,
-            conv=self.conv,
+            np.max(self.nside), self.nest, coordsys=self.coordsys, region=self.region
         )
 
     def to_cube(self, axes):
@@ -1058,7 +1034,6 @@ class HpxGeom(MapGeom):
             self.nest,
             coordsys=self.coordsys,
             region=self.region,
-            conv=self.conv,
             axes=axes,
         )
 
@@ -1083,11 +1058,7 @@ class HpxGeom(MapGeom):
         idx_nb = ravel_hpx_index(idx_nb, self._maxpix)
 
         for _ in range(pad_width):
-            # Mask of neighbors that are not contained in the geometry
-            # TODO: change this to numpy.isin when we require Numpy 1.13+
-            # Here and everywhere in Gamampy -> search for "isin"
-            # see https://github.com/gammapy/gammapy/pull/1371
-            mask_edge = np.in1d(idx_nb, idx_r, invert=True).reshape(idx_nb.shape)
+            mask_edge = np.isin(idx_nb, idx_r, invert=True)
             idx_edge = idx_nb[mask_edge]
             idx_edge = np.unique(idx_edge)
             idx_r = np.sort(np.concatenate((idx_r, idx_edge)))
@@ -1101,7 +1072,6 @@ class HpxGeom(MapGeom):
             self.nest,
             coordsys=self.coordsys,
             region=idx,
-            conv=self.conv,
             axes=copy.deepcopy(self.axes),
         )
 
@@ -1119,8 +1089,7 @@ class HpxGeom(MapGeom):
         for _ in range(crop_width):
             # Mask of pixels that have at least one neighbor not
             # contained in the geometry
-            mask_edge = np.in1d(idx_nb, idx_r, invert=True)
-            mask_edge = np.any(mask_edge, axis=0)
+            mask_edge = np.any(np.isin(idx_nb, idx_r, invert=True), axis=0)
             idx_r = idx_r[~mask_edge]
             idx_nb = idx_nb[:, ~mask_edge]
 
@@ -1130,7 +1099,6 @@ class HpxGeom(MapGeom):
             self.nest,
             coordsys=self.coordsys,
             region=idx,
-            conv=self.conv,
             axes=copy.deepcopy(self.axes),
         )
 
@@ -1144,7 +1112,6 @@ class HpxGeom(MapGeom):
                 self.nest,
                 coordsys=self.coordsys,
                 region=self.region,
-                conv=self.conv,
                 axes=copy.deepcopy(self.axes),
             )
 
@@ -1161,7 +1128,6 @@ class HpxGeom(MapGeom):
             self.nest,
             coordsys=self.coordsys,
             region=tuple(idx),
-            conv=self.conv,
             axes=copy.deepcopy(self.axes),
         )
 
@@ -1175,7 +1141,6 @@ class HpxGeom(MapGeom):
                 self.nest,
                 coordsys=self.coordsys,
                 region=self.region,
-                conv=self.conv,
                 axes=copy.deepcopy(self.axes),
             )
 
@@ -1188,7 +1153,6 @@ class HpxGeom(MapGeom):
             self.nest,
             coordsys=self.coordsys,
             region=tuple(idx),
-            conv=self.conv,
             axes=copy.deepcopy(self.axes),
         )
 
@@ -1201,7 +1165,6 @@ class HpxGeom(MapGeom):
         coordsys="CEL",
         region=None,
         axes=None,
-        conv="gadf",
         skydir=None,
         width=None,
     ):
@@ -1232,8 +1195,6 @@ class HpxGeom(MapGeom):
             ``skydir``.
         axes : list
             List of axes for non-spatial dimensions.
-        conv : str
-            Convention for FITS file format.
 
         Returns
         -------
@@ -1262,14 +1223,12 @@ class HpxGeom(MapGeom):
         elif isinstance(skydir, SkyCoord):
             lon, lat, frame = skycoord_to_lonlat(skydir, coordsys=coordsys)
         else:
-            raise ValueError("Invalid type for skydir: {!r}".format(type(skydir)))
+            raise ValueError(f"Invalid type for skydir: {type(skydir)!r}")
 
         if region is None and width is not None:
-            region = "DISK({:f},{:f},{:f})".format(lon, lat, width / 2.0)
+            region = f"DISK({lon},{lat},{width/2})"
 
-        return cls(
-            nside, nest=nest, coordsys=coordsys, region=region, axes=axes, conv=conv
-        )
+        return cls(nside, nest=nest, coordsys=coordsys, region=region, axes=axes)
 
     @staticmethod
     def identify_hpx_convention(header):
@@ -1338,7 +1297,7 @@ class HpxGeom(MapGeom):
 
         if header["PIXTYPE"] != "HEALPIX":
             raise ValueError(
-                "Header PIXTYPE must be 'HEALPIX'. Got: {}".format(header["PIXTYPE"])
+                f"Invalid header PIXTYPE: {header['PIXTYPE']} (must be HEALPIX)"
             )
 
         if header["ORDERING"] == "RING":
@@ -1347,9 +1306,7 @@ class HpxGeom(MapGeom):
             nest = True
         else:
             raise ValueError(
-                "Header ORDERING must be RING or NESTED. Got: {}".format(
-                    header["ORDERING"]
-                )
+                f"Invalid header ORDERING: {header['ORDERING']} (must be RING or NESTED)"
             )
 
         if hdu_bands is not None and "NSIDE" in hdu_bands.columns.names:
@@ -1374,9 +1331,7 @@ class HpxGeom(MapGeom):
             except KeyError:
                 region = None
 
-        return cls(
-            nside, nest, coordsys=coordsys, region=region, axes=axes, conv=convname
-        )
+        return cls(nside, nest, coordsys=coordsys, region=region, axes=axes)
 
     @classmethod
     def from_hdu(cls, hdu, hdu_bands=None):
@@ -1405,10 +1360,10 @@ class HpxGeom(MapGeom):
 
         return cls.from_header(hdu.header, hdu_bands=hdu_bands, pix=pix)
 
-    def make_header(self, **kwargs):
+    def make_header(self, conv="gadf", **kwargs):
         """Build and return FITS header for this HEALPIX map."""
         header = fits.Header()
-        conv = kwargs.get("conv", HPX_FITS_CONVENTIONS[self.conv])
+        conv = kwargs.get("conv", HPX_FITS_CONVENTIONS[conv])
 
         # FIXME: For some sparse maps we may want to allow EXPLICIT
         # with an empty region string
@@ -1495,10 +1450,10 @@ class HpxGeom(MapGeom):
             elif tokens[1] == "RING":
                 ipix_ring = int(tokens[3])
             else:
-                raise ValueError("Invalid ordering scheme: {!r}".format(tokens[1]))
+                raise ValueError(f"Invalid ordering scheme: {tokens[1]!r}")
             ilist = match_hpx_pix(nside, nest, nside_pix, ipix_ring)
         else:
-            raise ValueError("Invalid region type: {!r}".format(reg_type))
+            raise ValueError(f"Invalid region type: {reg_type!r}")
 
         return ilist
 
@@ -1592,7 +1547,7 @@ class HpxGeom(MapGeom):
 
     def get_idx(self, idx=None, local=False, flat=False):
         if idx is not None and np.any(np.array(idx) >= np.array(self.shape_axes)):
-            raise ValueError("Image index out of range: {!r}".format(idx))
+            raise ValueError(f"Image index out of range: {idx!r}")
 
         # Regular all- and partial-sky maps
         if self.is_regular:
@@ -1684,7 +1639,7 @@ class HpxGeom(MapGeom):
     def get_coord(self, idx=None, flat=False):
         pix = self.get_idx(idx=idx, flat=flat)
         coords = self.pix_to_coord(pix)
-        cdict = OrderedDict([("lon", coords[0]), ("lat", coords[1])])
+        cdict = {"lon": coords[0], "lat": coords[1]}
 
         for i, axis in enumerate(self.axes):
             cdict[axis.name] = coords[i + 2]
@@ -1712,19 +1667,20 @@ class HpxGeom(MapGeom):
         return Quantity(hp.nside2pixarea(self.nside), "sr")
 
     def __repr__(self):
-        str_ = self.__class__.__name__
-        str_ += "\n\n"
         axes = ["skycoord"] + [_.name for _ in self.axes]
-        str_ += "\taxes       : {}\n".format(", ".join(axes))
-        str_ += "\tshape      : {}\n".format(self.data_shape[::-1])
-        str_ += "\tndim       : {}\n".format(self.ndim)
-        str_ += "\tnside      : {nside[0]}\n".format(nside=self.nside)
-        str_ += "\tnested     : {}\n".format(self.nest)
-        str_ += "\tcoordsys   : {}\n".format(self.coordsys)
-        str_ += "\tprojection : {}\n".format(self.projection)
         lon, lat = self.center_skydir.data.lon.deg, self.center_skydir.data.lat.deg
-        str_ += "\tcenter     : {lon:.1f} deg, {lat:.1f} deg\n".format(lon=lon, lat=lat)
-        return str_
+
+        return (
+            f"{self.__class__.__name__}\n\n"
+            f"\taxes       : {axes}\n"
+            f"\tshape      : {self.data_shape[::-1]}\n"
+            f"\tndim       : {self.ndim}\n"
+            f"\tnside      : {self.nside[0]}\n"
+            f"\tnested     : {self.nest}\n"
+            f"\tcoordsys   : {self.coordsys}\n"
+            f"\tprojection : {self.projection}\n"
+            f"\tcenter     : {lon:.1f} deg, {lat:.1f} deg\n"
+        )
 
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
