@@ -1,11 +1,10 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 import pytest
-from collections import OrderedDict
 import numpy as np
 from numpy.testing import assert_allclose, assert_equal
 from astropy import units as u
 from astropy.coordinates import SkyCoord
-from ..geom import MapAxis, MapCoord
+from gammapy.maps import MapAxis, MapCoord
 
 mapaxis_geoms = [
     (np.array([0.25, 0.75, 1.0, 2.0]), "lin"),
@@ -136,6 +135,7 @@ def test_mapcoords_create():
     assert_allclose(coords.lat, lat)
     assert coords.coordsys == "CEL"
     assert coords.ndim == 2
+
     coords = MapCoord.create((skycoord_gal,))
     assert_allclose(coords.lon, lon)
     assert_allclose(coords.lat, lat)
@@ -154,30 +154,29 @@ def test_mapcoords_create():
     assert coords.coordsys == "GAL"
     assert coords.ndim == 2
 
-    # 2D Dict w/ vectors
+    # 2D dict w/ vectors
     coords = MapCoord.create(dict(lon=lon, lat=lat))
     assert_allclose(coords.lon, lon)
     assert_allclose(coords.lat, lat)
     assert coords.ndim == 2
 
-    # 3D Dict w/ vectors
+    # 3D dict w/ vectors
     coords = MapCoord.create(dict(lon=lon, lat=lat, energy=energy))
     assert_allclose(coords.lon, lon)
     assert_allclose(coords.lat, lat)
     assert_allclose(coords["energy"], energy)
     assert coords.ndim == 3
 
-    # 3D Dict w/ SkyCoord
+    # 3D dict w/ SkyCoord
     coords = MapCoord.create(dict(skycoord=skycoord_cel, energy=energy))
     assert_allclose(coords.lon, lon)
     assert_allclose(coords.lat, lat)
     assert_allclose(coords["energy"], energy)
+    assert coords.coordsys == "CEL"
     assert coords.ndim == 3
 
-    # 3D OrderedDict w/ vectors
-    coords = MapCoord.create(
-        OrderedDict([("energy", energy), ("lat", lat), ("lon", lon)])
-    )
+    # 3D dict  w/ vectors
+    coords = MapCoord.create({"energy": energy, "lat": lat, "lon": lon})
     assert_allclose(coords.lon, lon)
     assert_allclose(coords.lat, lat)
     assert_allclose(coords["energy"], energy)
@@ -370,3 +369,8 @@ def test_group_table_outside_range(energy_axis_ref):
 
     with pytest.raises(ValueError):
         energy_axis_ref.group_table(e_edges)
+
+
+def test_map_axis_single_bin():
+    with pytest.raises(ValueError):
+        _ = MapAxis.from_nodes([1])
