@@ -2,12 +2,11 @@
 import pytest
 import numpy as np
 from numpy.testing import assert_allclose
-from astropy.io import fits
-from astropy.coordinates import SkyCoord, Angle
 import astropy.units as u
-from ..wcs import WcsGeom, _check_width
-from ..geom import MapAxis
-from ..base import Map
+from astropy.coordinates import Angle, SkyCoord
+from astropy.io import fits
+from gammapy.maps import Map, MapAxis, WcsGeom
+from gammapy.maps.wcs import _check_width
 
 axes1 = [MapAxis(np.logspace(0.0, 3.0, 3), interp="log", name="energy")]
 axes2 = [
@@ -80,17 +79,19 @@ def test_wcsgeom_test_coord_to_idx(npix, binsz, coordsys, proj, skydir, axes):
 
     if not geom.is_allsky:
         coords = geom.center_coord[:2] + tuple([ax.center[0] for ax in geom.axes])
-        coords[0][...] += 2.0 * np.max(geom.width[0].to_value("deg"))
+        coords[0][...] += 2.0 * np.max(geom.width[0])
         idx = geom.coord_to_idx(coords)
-        assert_allclose(np.full_like(coords[0], -1, dtype=int), idx[0])
+        assert_allclose(np.full_like(coords[0].value, -1, dtype=int), idx[0])
         idx = geom.coord_to_idx(coords, clip=True)
-        assert np.all(np.not_equal(np.full_like(coords[0], -1, dtype=int), idx[0]))
+        assert np.all(
+            np.not_equal(np.full_like(coords[0].value, -1, dtype=int), idx[0])
+        )
 
 
 @pytest.mark.parametrize(
     ("npix", "binsz", "coordsys", "proj", "skydir", "axes"), wcs_test_geoms
 )
-def test_wcsgeom_read_write(tmpdir, npix, binsz, coordsys, proj, skydir, axes):
+def test_wcsgeom_read_write(tmp_path, npix, binsz, coordsys, proj, skydir, axes):
     geom0 = WcsGeom.create(
         npix=npix, binsz=binsz, proj=proj, coordsys=coordsys, axes=axes
     )
@@ -99,11 +100,10 @@ def test_wcsgeom_read_write(tmpdir, npix, binsz, coordsys, proj, skydir, axes):
     hdu_prim = fits.PrimaryHDU()
     hdu_prim.header.update(geom0.make_header())
 
-    filename = str(tmpdir / "wcsgeom.fits")
     hdulist = fits.HDUList([hdu_prim, hdu_bands])
-    hdulist.writeto(filename, overwrite=True)
+    hdulist.writeto(tmp_path / "tmp.fits")
 
-    with fits.open(filename) as hdulist:
+    with fits.open(tmp_path / "tmp.fits", memmap=False) as hdulist:
         geom1 = WcsGeom.from_header(hdulist[0].header, hdulist["BANDS"])
 
     assert_allclose(geom0.npix, geom1.npix)
@@ -232,17 +232,76 @@ def test_cutout():
     )
     position = SkyCoord(0.1, 0.2, unit="deg", frame="galactic")
     cutout_geom = geom.cutout(position=position, width=2 * 0.3 * u.deg, mode="trim")
-    assert_allclose(cutout_geom.center_coord, (0.1, 0.2, 2.0))
+
+    center_coord = cutout_geom.center_coord
+    assert_allclose(center_coord[0].value, 0.1)
+    assert_allclose(center_coord[1].value, 0.2)
+    assert_allclose(center_coord[2].value, 2.0)
+
     assert cutout_geom.data_shape == (2, 6, 6)
 
 
+def test_cutout_info():
+    geom = WcsGeom.create(skydir=(0, 0), npix=10)
+    position = SkyCoord(0, 0, unit="deg")
+    cutout_geom = geom.cutout(position=position, width="2 deg")
+    assert cutout_geom.cutout_info["parent-slices"][0].start == 3
+    assert cutout_geom.cutout_info["parent-slices"][1].start == 3
+
+    assert cutout_geom.cutout_info["cutout-slices"][0].start == 0
+    assert cutout_geom.cutout_info["cutout-slices"][1].start == 0
+
+    header = cutout_geom.make_header()
+    assert "PSLICE1" in header
+    assert "PSLICE2" in header
+    assert "CSLICE1" in header
+    assert "CSLICE2" in header
+
+    geom = WcsGeom.from_header(header)
+    assert geom.cutout_info["parent-slices"][0].start == 3
+    assert geom.cutout_info["parent-slices"][1].start == 3
+
+    assert geom.cutout_info["cutout-slices"][0].start == 0
+    assert geom.cutout_info["cutout-slices"][1].start == 0
+
+
 def test_wcsgeom_get_coord():
     geom = WcsGeom.create(
         skydir=(0, 0), npix=(4, 3), binsz=1, coordsys="GAL", proj="CAR"
     )
     coord = geom.get_coord(mode="edges")
-    assert_allclose(coord.lon[0, 0], 2)
-    assert_allclose(coord.lat[0, 0], -1.5)
+    assert_allclose(coord.lon[0, 0].value, 2)
+    assert coord.lon[0, 0].unit == "deg"
+    assert_allclose(coord.lat[0, 0].value, -1.5)
+    assert coord.lat[0, 0].unit == "deg"
+
+
+def test_wcsgeom_instance_cache():
+    geom_1 = WcsGeom.create(npix=(3, 3))
+    geom_2 = WcsGeom.create(npix=(3, 3))
+
+    coord_1, coord_2 = geom_1.get_coord(), geom_2.get_coord()
+
+    assert geom_1.get_coord.cache_info().misses == 1
+    assert geom_2.get_coord.cache_info().misses == 1
+
+    coord_1_cached, coord_2_cached = geom_1.get_coord(), geom_2.get_coord()
+
+    assert geom_1.get_coord.cache_info().hits == 1
+    assert geom_2.get_coord.cache_info().hits == 1
+
+    assert geom_1.get_coord.cache_info().currsize == 1
+    assert geom_2.get_coord.cache_info().currsize == 1
+
+    assert id(coord_1) == id(coord_1_cached)
+    assert id(coord_2) == id(coord_2_cached)
+
+
+def test_wcsgeom_squash():
+    axis = MapAxis.from_nodes([1, 2, 3], name="test-axis")
+    geom = WcsGeom.create(npix=(3, 3), axes=[axis])
+    geom_squashed = geom.squash(axis="test-axis")
+    assert geom_squashed.data_shape == (1, 3, 3)
 
 
 def test_wcsgeom_get_pix_coords():
@@ -392,7 +451,7 @@ def test_wcs_geom_equal(npix, binsz, coordsys, proj, skypos, axes, result):
 
 @pytest.mark.parametrize("node_type", ["edges", "center"])
 @pytest.mark.parametrize("interp", ["log", "lin", "sqrt"])
-def test_read_write(tmpdir, node_type, interp):
+def test_read_write(tmp_path, node_type, interp):
     # Regression test for MapAxis interp and node_type FITS serialization
     # https://github.com/gammapy/gammapy/issues/1887
     e_ax = MapAxis([1, 2], interp, "energy", node_type, "TeV")
@@ -405,7 +464,20 @@ def test_read_write(tmpdir, node_type, interp):
     assert header["INTERP2"] == interp
 
     # Check that all MapAxis properties are preserved on FITS I/O
-    filename = str(tmpdir / "geom_test.fits")
-    m.write(filename, overwrite=True)
-    m2 = Map.read(filename)
+    m.write(tmp_path / "tmp.fits", overwrite=True)
+    m2 = Map.read(tmp_path / "tmp.fits")
     assert m2.geom == m.geom
+
+
+@pytest.mark.parametrize(
+    ("npix", "binsz", "coordsys", "proj", "skypos", "axes", "result"),
+    compatibility_test_geoms,
+)
+def test_wcs_geom_to_binsz(npix, binsz, coordsys, proj, skypos, axes, result):
+    geom = WcsGeom.create(
+        skydir=skydir, npix=10, binsz=0.1, proj="CAR", coordsys="GAL", axes=test_axis1
+    )
+
+    geom_new = geom.to_binsz(binsz=0.5)
+
+    assert_allclose(geom_new.pixel_scales.value, 0.5)
