@@ -2,22 +2,23 @@
 import pytest
 import numpy as np
 from numpy.testing import assert_allclose, assert_equal
-from astropy.io import fits
-from astropy.coordinates import SkyCoord
-from astropy.convolution import Gaussian2DKernel
 import astropy.units as u
+from astropy.convolution import Gaussian2DKernel
+from astropy.coordinates import SkyCoord
+from astropy.io import fits
+from astropy.table import Table
 from regions import CircleSkyRegion
-from ...utils.testing import requires_dependency, requires_data, mpl_plot_check
-from ...cube import PSFKernel
-from ...irf import EnergyDependentMultiGaussPSF
-from ..utils import fill_poisson
-from ..geom import MapAxis, MapCoord, coordsys_to_frame
-from ..base import Map
-from ..wcs import WcsGeom
-from ..hpx import HpxGeom
-from ..wcsnd import WcsNDMap
-
-pytest.importorskip("reproject")
+from gammapy.cube import PSFKernel
+from gammapy.cube.fit import MapEvaluator
+from gammapy.irf import EnergyDependentMultiGaussPSF
+from gammapy.maps import Map, MapAxis, MapCoord, WcsGeom, WcsNDMap
+from gammapy.maps.utils import fill_poisson
+from gammapy.modeling.models import (
+    GaussianSpatialModel,
+    PowerLawSpectralModel,
+    SkyModel,
+)
+from gammapy.utils.testing import mpl_plot_check, requires_data, requires_dependency
 
 axes1 = [MapAxis(np.logspace(0.0, 3.0, 3), interp="log", name="spam")]
 axes2 = [
@@ -62,62 +63,67 @@ def test_wcsndmap_init(npix, binsz, coordsys, proj, skydir, axes):
 @pytest.mark.parametrize(
     ("npix", "binsz", "coordsys", "proj", "skydir", "axes"), wcs_test_geoms
 )
-def test_wcsndmap_read_write(tmpdir, npix, binsz, coordsys, proj, skydir, axes):
+def test_wcsndmap_read_write(tmp_path, npix, binsz, coordsys, proj, skydir, axes):
     geom = WcsGeom.create(
         npix=npix, binsz=binsz, proj=proj, coordsys=coordsys, axes=axes
     )
-    filename = str(tmpdir / "map.fits")
+    path = tmp_path / "tmp.fits"
 
     m0 = WcsNDMap(geom)
     fill_poisson(m0, mu=0.5)
-    m0.write(filename, overwrite=True)
-    m1 = WcsNDMap.read(filename)
-    m2 = Map.read(filename)
-    m3 = Map.read(filename, map_type="wcs")
+    m0.write(path, overwrite=True)
+    m1 = WcsNDMap.read(path)
+    m2 = Map.read(path)
+    m3 = Map.read(path, map_type="wcs")
     assert_allclose(m0.data, m1.data)
     assert_allclose(m0.data, m2.data)
     assert_allclose(m0.data, m3.data)
 
-    m0.write(filename, sparse=True, overwrite=True)
-    m1 = WcsNDMap.read(filename)
-    m2 = Map.read(filename)
-    m3 = Map.read(filename, map_type="wcs")
+    m0.write(path, sparse=True, overwrite=True)
+    m1 = WcsNDMap.read(path)
+    m2 = Map.read(path)
+    m3 = Map.read(path, map_type="wcs")
     assert_allclose(m0.data, m1.data)
     assert_allclose(m0.data, m2.data)
     assert_allclose(m0.data, m3.data)
 
     # Specify alternate HDU name for IMAGE and BANDS table
-    m0.write(filename, hdu="IMAGE", hdu_bands="TEST", overwrite=True)
-    m1 = WcsNDMap.read(filename)
-    m2 = Map.read(filename)
-    m3 = Map.read(filename, map_type="wcs")
+    m0.write(path, hdu="IMAGE", hdu_bands="TEST", overwrite=True)
+    m1 = WcsNDMap.read(path)
+    m2 = Map.read(path)
+    m3 = Map.read(path, map_type="wcs")
 
 
-def test_wcsndmap_read_write_fgst(tmpdir):
-    filename = str(tmpdir / "map.fits")
+def test_wcsndmap_read_write_fgst(tmp_path):
+    path = tmp_path / "tmp.fits"
 
     axis = MapAxis.from_bounds(100.0, 1000.0, 4, name="energy", unit="MeV")
     geom = WcsGeom.create(npix=10, binsz=1.0, proj="AIT", coordsys="GAL", axes=[axis])
 
     # Test Counts Cube
     m = WcsNDMap(geom)
-    m.write(filename, conv="fgst-ccube", overwrite=True)
-    with fits.open(filename) as h:
-        assert "EBOUNDS" in h
+    m.write(path, conv="fgst-ccube", overwrite=True)
+    with fits.open(path, memmap=False) as hdulist:
+        assert "EBOUNDS" in hdulist
 
-    m2 = Map.read(filename)
-    assert m2.geom.conv == "fgst-ccube"
+    m2 = Map.read(path)
+    assert m2.geom.axes[0].name == "energy"
 
     # Test Model Cube
-    m.write(filename, conv="fgst-template", overwrite=True)
-    with fits.open(filename) as h:
-        assert "ENERGIES" in h
-
-    m2 = Map.read(filename)
-    assert m2.geom.conv == "fgst-template"
+    m.write(path, conv="fgst-template", overwrite=True)
+    with fits.open(path, memmap=False) as hdulist:
+        assert "ENERGIES" in hdulist
 
 
-def test_wcs_nd_map_data_transpose_issue(tmpdir):
+@requires_data()
+def test_wcsndmap_read_ccube():
+    counts = Map.read("$GAMMAPY_DATA/fermi-3fhl-gc/fermi-3fhl-gc-counts-cube.fits.gz")
+    energy_axis = counts.geom.get_axis_by_name("energy")
+    # for the 3FGL data the lower energy threshold should be at 10 GeV
+    assert_allclose(energy_axis.edges.min().to_value("GeV"), 10, rtol=1e-3)
+
+
+def test_wcs_nd_map_data_transpose_issue(tmp_path):
     # Regression test for https://github.com/gammapy/gammapy/issues/1346
 
     # Our test case: a little map with WCS shape (3, 2), i.e. numpy array shape (2, 3)
@@ -135,14 +141,14 @@ def test_wcs_nd_map_data_transpose_issue(tmpdir):
     assert_equal(m.data, data)
 
     # Data should be unmodified after write / read to normal image format
-    filename = str(tmpdir / "normal.fits.gz")
-    m.write(filename)
-    assert_equal(Map.read(filename).data, data)
+    m.write(tmp_path / "normal.fits.gz")
+    m2 = Map.read(tmp_path / "normal.fits.gz")
+    assert_equal(m2.data, data)
 
     # Data should be unmodified after write / read to sparse image format
-    filename = str(tmpdir / "sparse.fits.gz")
-    m.write(filename)
-    assert_equal(Map.read(filename).data, data)
+    m.write(tmp_path / "sparse.fits.gz")
+    m2 = Map.read(tmp_path / "sparse.fits.gz")
+    assert_equal(m2.data, data)
 
 
 @pytest.mark.parametrize(
@@ -156,7 +162,7 @@ def test_wcsndmap_set_get_by_pix(npix, binsz, coordsys, proj, skydir, axes):
     coords = m.geom.get_coord()
     pix = m.geom.get_idx()
     m.set_by_pix(pix, coords[0])
-    assert_allclose(coords[0], m.get_by_pix(pix))
+    assert_allclose(coords[0].value, m.get_by_pix(pix))
 
 
 def test_get_by_coord_bool_int():
@@ -181,23 +187,17 @@ def test_wcsndmap_set_get_by_coord(npix, binsz, coordsys, proj, skydir, axes):
     m = WcsNDMap(geom)
     coords = m.geom.get_coord()
     m.set_by_coord(coords, coords[0])
-    assert_allclose(coords[0], m.get_by_coord(coords))
-
-    if not geom.is_allsky:
-        coords[1][...] = 0.0
-        assert_allclose(np.nan * np.ones(coords[0].shape), m.get_by_coord(coords))
+    assert_allclose(coords[0].value, m.get_by_coord(coords))
 
     # Test with SkyCoords
     m = WcsNDMap(geom)
     coords = m.geom.get_coord()
-    skydir = SkyCoord(
-        coords[0], coords[1], unit="deg", frame=coordsys_to_frame(geom.coordsys)
-    )
+    skydir = coords.skycoord
     skydir_cel = skydir.transform_to("icrs")
     skydir_gal = skydir.transform_to("galactic")
 
     m.set_by_coord((skydir_gal,) + tuple(coords[2:]), coords[0])
-    assert_allclose(coords[0], m.get_by_coord(coords))
+    assert_allclose(coords[0].value, m.get_by_coord(coords))
     assert_allclose(
         m.get_by_coord((skydir_cel,) + tuple(coords[2:])),
         m.get_by_coord((skydir_gal,) + tuple(coords[2:])),
@@ -212,7 +212,7 @@ def test_wcsndmap_set_get_by_coord(npix, binsz, coordsys, proj, skydir, axes):
             coords_dict[ax.name] = coords[i + 2]
     map_coords = MapCoord.create(coords_dict, coordsys=coordsys)
     m.set_by_coord(map_coords, coords[0])
-    assert_allclose(coords[0], m.get_by_coord(map_coords))
+    assert_allclose(coords[0].value, m.get_by_coord(map_coords))
 
 
 def test_set_get_by_coord_quantities():
@@ -227,6 +227,11 @@ def test_set_get_by_coord_quantities():
     assert_allclose(42, m.get_by_coord(coords_dict))
 
 
+def qconcatenate(q_1, q_2):
+    """Concatenate quantity"""
+    return u.Quantity(np.concatenate((q_1.value, q_2.value)), unit=q_1.unit)
+
+
 @pytest.mark.parametrize(
     ("npix", "binsz", "coordsys", "proj", "skydir", "axes"), wcs_test_geoms
 )
@@ -236,24 +241,23 @@ def test_wcsndmap_fill_by_coord(npix, binsz, coordsys, proj, skydir, axes):
     )
     m = WcsNDMap(geom)
     coords = m.geom.get_coord()
-    fill_coords = tuple([np.concatenate((t, t)) for t in coords])
+    fill_coords = tuple([qconcatenate(t, t) for t in coords])
+
     fill_vals = fill_coords[1]
-    m.fill_by_coord(fill_coords, fill_vals)
-    assert_allclose(m.get_by_coord(coords), 2.0 * coords[1])
+    m.fill_by_coord(fill_coords, fill_vals.value)
+    assert_allclose(m.get_by_coord(coords), 2.0 * coords[1].value)
 
     # Test with SkyCoords
     m = WcsNDMap(geom)
     coords = m.geom.get_coord()
-    skydir = SkyCoord(
-        coords[0], coords[1], unit="deg", frame=coordsys_to_frame(geom.coordsys)
-    )
+    skydir = coords.skycoord
     skydir_cel = skydir.transform_to("icrs")
     skydir_gal = skydir.transform_to("galactic")
     fill_coords_cel = (skydir_cel,) + tuple(coords[2:])
     fill_coords_gal = (skydir_gal,) + tuple(coords[2:])
-    m.fill_by_coord(fill_coords_cel, coords[1])
-    m.fill_by_coord(fill_coords_gal, coords[1])
-    assert_allclose(m.get_by_coord(coords), 2.0 * coords[1])
+    m.fill_by_coord(fill_coords_cel, coords[1].value)
+    m.fill_by_coord(fill_coords_gal, coords[1].value)
+    assert_allclose(m.get_by_coord(coords), 2.0 * coords[1].value)
 
 
 @pytest.mark.parametrize(
@@ -267,8 +271,8 @@ def test_wcsndmap_coadd(npix, binsz, coordsys, proj, skydir, axes):
     m1 = WcsNDMap(geom.upsample(2))
     coords = m0.geom.get_coord()
     m1.fill_by_coord(
-        tuple([np.concatenate((t, t)) for t in coords]),
-        np.concatenate((coords[1], coords[1])),
+        tuple([qconcatenate(t, t) for t in coords]),
+        qconcatenate(coords[1], coords[1]).value,
     )
     m0.coadd(m1)
     assert_allclose(np.nansum(m0.data), np.nansum(m1.data), rtol=1e-4)
@@ -283,12 +287,14 @@ def test_wcsndmap_interp_by_coord(npix, binsz, coordsys, proj, skydir, axes):
     )
     m = WcsNDMap(geom)
     coords = m.geom.get_coord(flat=True)
-    m.set_by_coord(coords, coords[1])
-    assert_allclose(coords[1], m.interp_by_coord(coords, interp="nearest"))
-    assert_allclose(coords[1], m.interp_by_coord(coords, interp="linear"))
-    assert_allclose(coords[1], m.interp_by_coord(coords, interp=1))
+    m.set_by_coord(coords, coords[1].value)
+    assert_allclose(coords[1].value, m.interp_by_coord(coords, interp="nearest"))
+    assert_allclose(coords[1].value, m.interp_by_coord(coords, interp="linear"))
+    assert_allclose(coords[1].value, m.interp_by_coord(coords, interp=1))
     if geom.is_regular and not geom.is_allsky:
-        assert_allclose(coords[1], m.interp_by_coord(coords, interp="cubic"))
+        assert_allclose(
+            coords[1].to_value("deg"), m.interp_by_coord(coords, interp="cubic")
+        )
 
 
 def test_interp_by_coord_quantities():
@@ -329,64 +335,13 @@ def test_wcsndmap_sum_over_axes(npix, binsz, coordsys, proj, skydir, axes, keepd
     )
     m = WcsNDMap(geom)
     coords = m.geom.get_coord()
-    m.fill_by_coord(coords, coords[0])
+    m.fill_by_coord(coords, coords[0].value)
     msum = m.sum_over_axes(keepdims=keepdims)
 
     if m.geom.is_regular:
         assert_allclose(np.nansum(m.data), np.nansum(msum.data))
 
 
-def test_wcsndmap_reproject():
-    skydir = SkyCoord(110.0, 75.0, unit="deg", frame="icrs")
-    geom = WcsGeom.create(npix=10, binsz=1.0, coordsys="GAL", proj="AIT", skydir=skydir)
-    geom_new = geom.downsample(2)
-
-    data = np.arange(np.prod(geom.data_shape)).reshape(geom.data_shape)
-    m = WcsNDMap(data=data, geom=geom, unit="m2")
-
-    m_reprojected = m.reproject(geom_new, order=1)
-    assert m.unit == m_reprojected.unit
-
-    assert_allclose(m_reprojected.data[0, 0], 5.5)
-    assert_allclose(m_reprojected.data[4, 4], 93.5)
-
-    energy_axis = MapAxis.from_bounds(0.1, 10, 2, name="energy", interp="log")
-    geom_3d = geom.to_cube([energy_axis])
-
-    data = np.arange(np.prod(geom_3d.data_shape)).reshape(geom_3d.data_shape)
-    m = WcsNDMap(data=data, geom=geom_3d, unit="m2")
-
-    m_reprojected = m.reproject(geom_new, order=1)
-    assert m.unit == m_reprojected.unit
-
-    assert_allclose(m_reprojected.data[0, 0, 0], 5.5)
-    assert_allclose(m_reprojected.data[1, 4, 4], 193.5)
-
-    assert m_reprojected.geom.axes[0].name == "energy"
-
-
-@requires_dependency("healpy")
-def test_wcsndmap_reproject_allsky_car():
-    geom = WcsGeom.create(binsz=10.0, proj="CAR", coordsys="CEL")
-    m = WcsNDMap(geom)
-    coords = m.geom.get_coord()
-    m.set_by_coord(coords, coords[0])
-
-    geom0 = WcsGeom.create(
-        binsz=1.0, proj="CAR", coordsys="CEL", skydir=(180.0, 0.0), width=30.0
-    )
-    m0 = m.reproject(geom0, order=1)
-    coords0 = m0.geom.get_coord()
-    assert_allclose(m0.get_by_coord(coords0), coords0[0])
-
-    geom1 = HpxGeom.create(binsz=5.0, coordsys="CEL")
-    m1 = m.reproject(geom1, order=1)
-    coords1 = m1.geom.get_coord()
-
-    m = (coords1[0] > 10.0) & (coords1[0] < 350.0)
-    assert_allclose(m1.get_by_coord((coords1[0][m], coords1[1][m])), coords1[0][m])
-
-
 @pytest.mark.parametrize(
     ("npix", "binsz", "coordsys", "proj", "skydir", "axes"), wcs_test_geoms
 )
@@ -405,6 +360,15 @@ def test_wcsndmap_pad(npix, binsz, coordsys, proj, skydir, axes):
     m.pad(1, mode="interp")
 
 
+def test_wcsndmap_pad_cval():
+    geom = WcsGeom.create(npix=(5, 5))
+    m = WcsNDMap.from_geom(geom)
+
+    cval = 1.1
+    m_padded = m.pad(1, mode="constant", cval=cval)
+    assert_allclose(m_padded.data[0, 0], cval)
+
+
 @pytest.mark.parametrize(
     ("npix", "binsz", "coordsys", "proj", "skydir", "axes"), wcs_test_geoms
 )
@@ -552,9 +516,8 @@ def test_convolve_pixel_scale_error():
 
     kernel = PSFKernel.from_gauss(kgeom, sigma=0.1 * u.deg, max_radius=1.5 * u.deg)
 
-    with pytest.raises(ValueError) as excinfo:
+    with pytest.raises(ValueError):
         m.convolve(kernel)
-    assert "Pixel size of kernel and map not compatible." == str(excinfo.value)
 
 
 @requires_dependency("matplotlib")
@@ -589,3 +552,54 @@ def test_get_spectrum():
 
     spec = m.get_spectrum(region=region, func=np.mean)
     assert_allclose(spec.data, [1.0, 1.0, 1.0])
+
+
+def get_npred_map():
+    position = SkyCoord(0.0, 0.0, frame="galactic", unit="deg")
+    energy_axis = MapAxis.from_bounds(
+        1, 100, nbin=30, unit="TeV", name="energy", interp="log"
+    )
+
+    exposure = Map.create(
+        binsz=0.02,
+        map_type="wcs",
+        skydir=position,
+        width="2 deg",
+        axes=[energy_axis],
+        coordsys="GAL",
+        unit="cm2 s",
+    )
+
+    spatial_model = GaussianSpatialModel(
+        lon_0="0 deg", lat_0="0 deg", sigma="0.2 deg", frame="galactic"
+    )
+    spectral_model = PowerLawSpectralModel(amplitude="1e-11 cm-2 s-1 TeV-1")
+    skymodel = SkyModel(spatial_model=spatial_model, spectral_model=spectral_model)
+
+    exposure.data = 1e14 * np.ones(exposure.data.shape)
+    evaluator = MapEvaluator(model=skymodel, exposure=exposure)
+
+    npred = evaluator.compute_npred()
+    return evaluator, npred
+
+
+def test_map_sampling():
+    eval, npred = get_npred_map()
+
+    nmap = WcsNDMap(geom=eval.geom, data=npred.data)
+    coords = nmap.sample_coord(n_events=2, random_state=0)
+    skycoord = coords.skycoord
+
+    events = Table()
+    events["RA_TRUE"] = skycoord.icrs.ra
+    events["DEC_TRUE"] = skycoord.icrs.dec
+    events["ENERGY_TRUE"] = coords["energy"]
+
+    assert len(events) == 2
+    assert_allclose(events["RA_TRUE"].data, [266.307081, 266.442255], rtol=1e-5)
+    assert_allclose(events["DEC_TRUE"].data, [-28.753408, -28.742696], rtol=1e-5)
+    assert_allclose(events["ENERGY_TRUE"].data, [2.755397, 1.72316], rtol=1e-5)
+
+    assert coords["lon"].unit == "deg"
+    assert coords["lat"].unit == "deg"
+    assert coords["energy"].unit == "TeV"
