@@ -1,7 +1,7 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 import numpy as np
 from astropy.io import fits
-from ..utils.random import get_random_state
+from gammapy.utils.random import get_random_state
 
 
 class InvalidValue:
@@ -19,9 +19,7 @@ class InvalidValue:
         elif np.issubdtype(dtype, np.dtype(bool).type):
             return self.bool
         else:
-            raise ValueError(
-                "No invalid value placeholder defined for {}".format(dtype)
-            )
+            raise ValueError(f"No invalid value placeholder defined for {dtype}")
 
 
 class InvalidIndex:
@@ -159,3 +157,12 @@ def edges_from_lo_hi(edges_lo, edges_hi):
     except AttributeError:
         edges = np.insert(edges, len(edges), edges_hi[-1])
     return edges
+
+
+def slice_to_str(slice_):
+    return f"{slice_.start}:{slice_.stop}"
+
+
+def str_to_slice(slice_str):
+    start, stop = slice_str.split(":")
+    return slice(int(start), int(stop))
