@@ -1,17 +1,24 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 import copy
-from collections import OrderedDict
+from functools import lru_cache
 import numpy as np
-from astropy.wcs import WCS
+import astropy.units as u
+from astropy.coordinates import Angle, SkyCoord
 from astropy.io import fits
 from astropy.nddata import Cutout2D
-from astropy.coordinates import SkyCoord, Angle
+from astropy.wcs import WCS
 from astropy.wcs.utils import proj_plane_pixel_scales
-import astropy.units as u
 from regions import SkyRegion
-from .geom import MapGeom, MapCoord, pix_tuple_to_idx, skycoord_to_lonlat
-from .geom import get_shape, make_axes, find_and_read_bands, axes_pix_to_coord
-from .utils import INVALID_INDEX
+from .geom import (
+    Geom,
+    MapCoord,
+    find_and_read_bands,
+    get_shape,
+    make_axes,
+    pix_tuple_to_idx,
+    skycoord_to_lonlat,
+)
+from .utils import INVALID_INDEX, slice_to_str, str_to_slice
 
 __all__ = ["WcsGeom"]
 
@@ -127,7 +134,7 @@ def _make_image_header(
     elif coordsys == "GAL":
         ctype1, ctype2 = "GLON-", "GLAT-"
     else:
-        raise ValueError("Unsupported coordsys: {!r}".format(coordsys))
+        raise ValueError(f"Unsupported coordsys: {coordsys!r}")
 
     pars = {
         "NAXIS": 2,
@@ -151,7 +158,7 @@ def _make_image_header(
     return header
 
 
-class WcsGeom(MapGeom):
+class WcsGeom(Geom):
     """Geometry class for WCS maps.
 
     This class encapsulates both the WCS transformation object and the
@@ -171,21 +178,19 @@ class WcsGeom(MapGeom):
         Reference pixel coordinate in each image plane.
     axes : list
         Axes for non-spatial dimensions
-    conv : {'gadf', 'fgst-ccube', 'fgst-template'}
-        Serialization format convention.  This sets the default format
-        that will be used when writing this geometry to a file.
+    cutout_info : dict
+        Dict with cutout info, if the `WcsGeom` was created by `WcsGeom.cutout()`
     """
 
     _slice_spatial_axes = slice(0, 2)
     _slice_non_spatial_axes = slice(2, None)
     is_hpx = False
 
-    def __init__(self, wcs, npix, cdelt=None, crpix=None, axes=None, conv="gadf"):
+    def __init__(self, wcs, npix, cdelt=None, crpix=None, axes=None, cutout_info=None):
         self._wcs = wcs
         self._coordsys = get_coordys(wcs)
         self._projection = get_projection(wcs)
-        self._conv = conv
-        self._axes = make_axes(axes, conv)
+        self._axes = make_axes(axes)
 
         if cdelt is None:
             cdelt = tuple(np.abs(self.wcs.wcs.cdelt))
@@ -200,6 +205,13 @@ class WcsGeom(MapGeom):
             crpix = tuple(1.0 + (np.array(self._npix) - 1.0) / 2.0)
 
         self._crpix = crpix
+        self._cutout_info = cutout_info
+
+        # define cached methods
+        self.get_coord = lru_cache()(self.get_coord)
+        self.solid_angle = lru_cache()(self.solid_angle)
+        self.bin_volume = lru_cache()(self.bin_volume)
+        self.to_image = lru_cache()(self.to_image)
 
     @property
     def data_shape(self):
@@ -236,6 +248,11 @@ class WcsGeom(MapGeom):
         """
         return self._coordsys
 
+    @property
+    def cutout_info(self):
+        """Cutout info dict."""
+        return self._cutout_info
+
     @property
     def projection(self):
         """Map projection."""
@@ -279,11 +296,6 @@ class WcsGeom(MapGeom):
         """Tuple with image dimension in pixels in longitude and latitude."""
         return self._npix
 
-    @property
-    def conv(self):
-        """Name of default FITS convention associated with this geometry."""
-        return self._conv
-
     @property
     def axes(self):
         """List of non-spatial axes."""
@@ -348,7 +360,6 @@ class WcsGeom(MapGeom):
         axes=None,
         skydir=None,
         width=None,
-        conv="gadf",
     ):
         """Create a WCS geometry object.
 
@@ -390,9 +401,6 @@ class WcsGeom(MapGeom):
         refpix : tuple
             Reference pixel of the projection.  If None this will be
             set to the center of the map.
-        conv : string, optional
-            FITS format convention ('fgst-ccube', 'fgst-template',
-            'gadf').  Default is 'gadf'.
 
         Returns
         -------
@@ -416,7 +424,7 @@ class WcsGeom(MapGeom):
         elif isinstance(skydir, SkyCoord):
             xref, yref, frame = skycoord_to_lonlat(skydir, coordsys=coordsys)
         else:
-            raise ValueError("Invalid type for skydir: {!r}".format(type(skydir)))
+            raise ValueError(f"Invalid type for skydir: {type(skydir)!r}")
 
         if width is not None:
             width = _check_width(width)
@@ -452,7 +460,7 @@ class WcsGeom(MapGeom):
             yrefpix=refpix[1],
         )
         wcs = WCS(header)
-        return cls(wcs, npix, cdelt=binsz, axes=axes, conv=conv)
+        return cls(wcs, npix, cdelt=binsz, axes=axes)
 
     @classmethod
     def from_header(cls, header, hdu_bands=None):
@@ -475,16 +483,8 @@ class WcsGeom(MapGeom):
         for i in range(naxis - 2):
             wcs = wcs.dropaxis(2)
 
-        axes = find_and_read_bands(hdu_bands, header)
+        axes = find_and_read_bands(hdu_bands)
         shape = tuple([ax.nbin for ax in axes])
-        conv = "gadf"
-
-        # Discover FITS convention
-        if hdu_bands is not None:
-            if hdu_bands.name == "EBOUNDS":
-                conv = "fgst-ccube"
-            elif hdu_bands.name == "ENERGIES":
-                conv = "fgst-template"
 
         if hdu_bands is not None and "NPIX" in hdu_bands.columns.names:
             npix = hdu_bands.data.field("NPIX").reshape(shape + (2,))
@@ -499,7 +499,20 @@ class WcsGeom(MapGeom):
             npix = (header["NAXIS1"], header["NAXIS2"])
             cdelt = None
 
-        return cls(wcs, npix, cdelt=cdelt, axes=axes, conv=conv)
+        if "PSLICE1" in header:
+            cutout_info = {}
+            cutout_info["parent-slices"] = (
+                str_to_slice(header["PSLICE2"]),
+                str_to_slice(header["PSLICE1"]),
+            )
+            cutout_info["cutout-slices"] = (
+                str_to_slice(header["CSLICE2"]),
+                str_to_slice(header["CSLICE1"]),
+            )
+        else:
+            cutout_info = None
+
+        return cls(wcs, npix, cdelt=cdelt, axes=axes, cutout_info=cutout_info)
 
     def _make_bands_cols(self, hdu=None, conv=None):
 
@@ -540,8 +553,18 @@ class WcsGeom(MapGeom):
         self._fill_header_from_axes(header)
         shape = "{},{}".format(np.max(self.npix[0]), np.max(self.npix[1]))
         for ax in self.axes:
-            shape += ",{}".format(ax.nbin)
-        header["WCSSHAPE"] = "({})".format(shape)
+            shape += f",{ax.nbin}"
+        header["WCSSHAPE"] = f"({shape})"
+
+        if self.cutout_info is not None:
+            slices_parent = self.cutout_info["parent-slices"]
+            slices_cutout = self.cutout_info["cutout-slices"]
+
+            header["PSLICE1"] = (slice_to_str(slices_parent[1]), "Parent slice")
+            header["PSLICE2"] = (slice_to_str(slices_parent[0]), "Parent slice")
+            header["CSLICE1"] = (slice_to_str(slices_cutout[1]), "Cutout slice")
+            header["CSLICE2"] = (slice_to_str(slices_cutout[0]), "Cutout slice")
+
         return header
 
     def get_image_shape(self, idx):
@@ -597,7 +620,7 @@ class WcsGeom(MapGeom):
             _[~m] = INVALID_INDEX.float
         return pix
 
-    def get_coord(self, idx=None, flat=False, mode="center"):
+    def get_coord(self, idx=None, flat=False, mode="center", coordsys=None):
         """Get map coordinates from the geometry.
 
         Parameters
@@ -618,9 +641,12 @@ class WcsGeom(MapGeom):
             coords = tuple([c[is_finite] for c in coords])
 
         axes_names = ["lon", "lat"] + [ax.name for ax in self.axes]
-        cdict = OrderedDict(zip(axes_names, coords))
+        cdict = dict(zip(axes_names, coords))
 
-        return MapCoord.create(cdict, coordsys=self.coordsys)
+        if coordsys is None:
+            coordsys = self.coordsys
+
+        return MapCoord.create(cdict, coordsys=self.coordsys).to_coordsys(coordsys)
 
     def coord_to_pix(self, coords):
         coords = MapCoord.create(coords, coordsys=self.coordsys)
@@ -659,7 +685,14 @@ class WcsGeom(MapGeom):
         else:
             coords = self._wcs.wcs_pix2world(pix[0], pix[1], 0)
 
-        coords += axes_pix_to_coord(self.axes, pix[self._slice_non_spatial_axes])
+        coords = [
+            u.Quantity(coords[0], unit="deg", copy=False),
+            u.Quantity(coords[1], unit="deg", copy=False),
+        ]
+
+        for ax, t in zip(self.axes, pix[self._slice_non_spatial_axes]):
+            coords += [ax.pix_to_coord(t)]
+
         return tuple(coords)
 
     def pix_to_idx(self, pix, clip=False):
@@ -696,13 +729,52 @@ class WcsGeom(MapGeom):
     def to_image(self):
         npix = (np.max(self._npix[0]), np.max(self._npix[1]))
         cdelt = (np.max(self._cdelt[0]), np.max(self._cdelt[1]))
-        return self.__class__(self._wcs, npix, cdelt=cdelt)
+        return self.__class__(
+            self._wcs, npix, cdelt=cdelt, cutout_info=self.cutout_info
+        )
 
     def to_cube(self, axes):
         npix = (np.max(self._npix[0]), np.max(self._npix[1]))
         cdelt = (np.max(self._cdelt[0]), np.max(self._cdelt[1]))
         axes = copy.deepcopy(self.axes) + axes
-        return self.__class__(self._wcs.deepcopy(), npix, cdelt=cdelt, axes=axes)
+        return self.__class__(
+            self._wcs.deepcopy(),
+            npix,
+            cdelt=cdelt,
+            axes=axes,
+            cutout_info=self.cutout_info,
+        )
+
+    def squash(self, axis):
+        """Squash geom axis.
+
+        Parameters
+        ----------
+        axis : str
+            Axis to squash.
+
+        Returns
+        -------
+        geom : `Geom`
+            Geom with squashed axis.
+        """
+        _ = self.get_axis_by_name(axis)
+        npix = (np.max(self._npix[0]), np.max(self._npix[1]))
+        cdelt = (np.max(self._cdelt[0]), np.max(self._cdelt[1]))
+
+        axes = []
+        for ax in copy.deepcopy(self.axes):
+            if ax.name == axis:
+                ax = ax.squash()
+            axes.append(ax)
+
+        return self.__class__(
+            self._wcs.deepcopy(),
+            npix,
+            cdelt=cdelt,
+            axes=axes,
+            cutout_info=self.cutout_info,
+        )
 
     def pad(self, pad_width):
         if np.isscalar(pad_width):
@@ -728,8 +800,8 @@ class WcsGeom(MapGeom):
         if axis is None:
             if np.any(np.mod(self.npix, factor) > 0):
                 raise ValueError(
-                    "Spatial shape not divisible by factor {!r} in all axes."
-                    " You need to pad prior to calling downsample.".format(factor)
+                    f"Spatial shape not divisible by factor {factor!r} in all axes."
+                    f" You need to pad prior to calling downsample."
                 )
 
             npix = (self.npix[0] / factor, self.npix[1] / factor)
@@ -739,8 +811,7 @@ class WcsGeom(MapGeom):
         else:
             if not self.is_regular:
                 raise NotImplementedError(
-                    "Upsampling in non-spatial axes not"
-                    " supported for irregular geometries"
+                    "Upsampling in non-spatial axes not supported for irregular geometries"
                 )
 
             axes = copy.deepcopy(self.axes)
@@ -757,14 +828,35 @@ class WcsGeom(MapGeom):
         else:
             if not self.is_regular:
                 raise NotImplementedError(
-                    "Upsampling in non-spatial axes not"
-                    " supported for irregular geometries"
+                    "Upsampling in non-spatial axes not supported for irregular geometries"
                 )
             axes = copy.deepcopy(self.axes)
             idx = self.get_axis_index_by_name(axis)
             axes[idx] = axes[idx].upsample(factor)
             return self._init_copy(axes=axes)
 
+    def to_binsz(self, binsz):
+        """Change pixel size of the geometry.
+
+        Parameters
+        ----------
+        binsz : float or tuple or list
+            New pixel size in degree.
+
+        Returns
+        -------
+        geom : `WcsGeom`
+            Geometry with new pixel size.
+        """
+        return self.create(
+            skydir=self.center_skydir,
+            binsz=binsz,
+            width=self.width,
+            proj=self.projection,
+            coordsys=self.coordsys,
+            axes=copy.deepcopy(self.axes),
+        )
+
     def solid_angle(self):
         """Solid angle array (`~astropy.units.Quantity` in ``sr``).
 
@@ -802,6 +894,17 @@ class WcsGeom(MapGeom):
 
         return u.Quantity(area_low_right + area_up_left, "sr", copy=False)
 
+    def bin_volume(self):
+        """Bin volume (`~astropy.units.Quantity`)"""
+        bin_volume = self.to_image().solid_angle()
+
+        for idx, ax in enumerate(self.axes):
+            shape = self.ndim * [1]
+            shape[-(idx + 3)] = -1
+            bin_volume = bin_volume * ax.bin_width.reshape(tuple(shape))
+
+        return bin_volume
+
     def separation(self, center):
         """Compute sky separation wrt a given center.
 
@@ -848,7 +951,14 @@ class WcsGeom(MapGeom):
             mode=mode,
         )
 
-        return self._init_copy(wcs=c2d.wcs, npix=c2d.shape[::-1])
+        cutout_info = {
+            "parent-slices": c2d.slices_original,
+            "cutout-slices": c2d.slices_cutout,
+        }
+
+        return self._init_copy(
+            wcs=c2d.wcs, npix=c2d.shape[::-1], cutout_info=cutout_info
+        )
 
     def region_mask(self, regions, inside=True):
         """Create a mask from a given list of regions
@@ -914,21 +1024,42 @@ class WcsGeom(MapGeom):
         return mask
 
     def __repr__(self):
-        str_ = self.__class__.__name__
-        str_ += "\n\n"
         axes = ["lon", "lat"] + [_.name for _ in self.axes]
-        str_ += "\taxes       : {}\n".format(", ".join(axes))
-        str_ += "\tshape      : {}\n".format(self.data_shape[::-1])
-        str_ += "\tndim       : {}\n".format(self.ndim)
-        str_ += "\tcoordsys   : {}\n".format(self.coordsys)
-        str_ += "\tprojection : {}\n".format(self.projection)
         lon = self.center_skydir.data.lon.deg
         lat = self.center_skydir.data.lat.deg
-        str_ += "\tcenter     : {:.1f} deg, {:.1f} deg\n".format(lon, lat)
-        str_ += "\twidth      : {width[0][0]:.1f} x {width[1][0]:.1f}\n".format(
-            width=self.width
+
+        return (
+            f"{self.__class__.__name__}\n\n"
+            f"\taxes       : {axes}\n"
+            f"\tshape      : {self.data_shape[::-1]}\n"
+            f"\tndim       : {self.ndim}\n"
+            f"\tcoordsys   : {self.coordsys}\n"
+            f"\tprojection : {self.projection}\n"
+            f"\tcenter     : {lon:.1f} deg, {lat:.1f} deg\n"
+            f"\twidth      : {self.width[0][0]:.1f} x {self.width[1][0]:.1f}\n"
         )
-        return str_
+
+    def is_aligned(self, other, tolerance=1e-6):
+        """Check if WCS and extra axes are aligned.
+
+        Parameters
+        ----------
+        other : `WcsGeom`
+            Other geom.
+        tolerance : float
+            Tolerance for the comparison.
+
+        Returns
+        -------
+        aligned : bool
+            Whether geometries are aligned
+        """
+        for axis, otheraxis in zip(self.axes, other.axes):
+            if axis != otheraxis:
+                return False
+
+        # check WCS consistency with a priori tolerance of 1e-6
+        return self.wcs.wcs.compare(other.wcs.wcs, cmp=2, tolerance=tolerance)
 
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
@@ -948,6 +1079,9 @@ class WcsGeom(MapGeom):
     def __ne__(self, other):
         return not self.__eq__(other)
 
+    def __hash__(self):
+        return id(self)
+
 
 def pix2world(wcs, cdelt, crpix, pix):
     """Perform pixel to world coordinate transformation.
