@@ -1,11 +1,10 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import absolute_import, division, print_function, unicode_literals
 import json
 import numpy as np
 from astropy.io import fits
 from .base import Map
+from .utils import find_bands_hdu, find_hdu
 from .wcs import WcsGeom
-from .utils import find_hdu, find_bands_hdu
 
 __all__ = ["WcsMap"]
 
@@ -34,7 +33,6 @@ class WcsMap(Map):
         axes=None,
         skydir=None,
         dtype="float32",
-        conv="gadf",
         meta=None,
         unit="",
     ):
@@ -78,7 +76,7 @@ class WcsMap(Map):
             Data type, default is float32
         conv : {'fgst-ccube','fgst-template','gadf'}, optional
             FITS format convention.  Default is 'gadf'.
-        meta : `~collections.OrderedDict`
+        meta : `dict`
             Dictionary to store meta data.
         unit : str or `~astropy.units.Unit`
             The unit of the map
@@ -90,8 +88,6 @@ class WcsMap(Map):
         """
         from .wcsnd import WcsNDMap
 
-        # from .wcssparse import WcsMapSparse
-
         geom = WcsGeom.create(
             npix=npix,
             binsz=binsz,
@@ -101,7 +97,6 @@ class WcsMap(Map):
             coordsys=coordsys,
             refpix=refpix,
             axes=axes,
-            conv=conv,
         )
 
         if map_type == "wcs":
@@ -109,7 +104,7 @@ class WcsMap(Map):
         elif map_type == "wcs-sparse":
             raise NotImplementedError
         else:
-            raise ValueError("Invalid map type: {!r}".format(map_type))
+            raise ValueError(f"Invalid map type: {map_type!r}")
 
     @classmethod
     def from_hdulist(cls, hdu_list, hdu=None, hdu_bands=None):
@@ -142,7 +137,7 @@ class WcsMap(Map):
 
         return cls.from_hdu(hdu, hdu_bands)
 
-    def to_hdulist(self, hdu=None, hdu_bands=None, sparse=False, conv=None):
+    def to_hdulist(self, hdu=None, hdu_bands=None, sparse=False, conv="gadf"):
         """Convert to `~astropy.io.fits.HDUList`.
 
         Parameters
@@ -154,9 +149,8 @@ class WcsMap(Map):
         sparse : bool
             Sparsify the map by only writing pixels with non-zero
             amplitude.
-        conv : {'fgst-ccube','fgst-template','gadf',None}, optional
-            FITS format convention.  If None this will be set to the
-            default convention of the map.
+        conv : {'gadf', 'fgst-ccube','fgst-template'}
+            FITS format convention.
 
         Returns
         -------
@@ -171,6 +165,12 @@ class WcsMap(Map):
         if sparse and hdu == "PRIMARY":
             raise ValueError("Sparse maps cannot be written to the PRIMARY HDU.")
 
+        if conv in ["fgst-ccube", "fgst-template"]:
+            if self.geom.axes[0].name != "energy" or len(self.geom.axes) > 1:
+                raise ValueError(
+                    "All 'fgst' formats don't support extra axes except for energy."
+                )
+
         if self.geom.axes:
             hdu_bands_out = self.geom.make_bands_hdu(
                 hdu=hdu_bands, hdu_skymap=hdu, conv=conv
