@@ -1,23 +1,20 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 import logging
+from collections import OrderedDict
 import numpy as np
-from astropy.io import fits
+import scipy.interpolate
+import scipy.ndimage
+import scipy.signal
 import astropy.units as u
-from astropy.nddata import Cutout2D
 from astropy.convolution import Tophat2DKernel
-from scipy.ndimage import gaussian_filter, uniform_filter, convolve
-from scipy.signal import fftconvolve
-from scipy.interpolate import griddata
-from scipy.ndimage import map_coordinates
-from ..extern.skimage import block_reduce
-from ..utils.units import unit_from_fits_image_hdu
-from ..utils.interpolation import ScaledRegularGridInterpolator
-from .geom import pix_tuple_to_idx
-from .wcs import _check_width
-from .utils import interp_to_order, INVALID_INDEX
+from astropy.io import fits
+from gammapy.extern.skimage import block_reduce
+from gammapy.utils.interpolation import ScaledRegularGridInterpolator
+from gammapy.utils.random import InverseCDFSampler, get_random_state
+from gammapy.utils.units import unit_from_fits_image_hdu
+from .geom import MapCoord, pix_tuple_to_idx
+from .utils import INVALID_INDEX, interp_to_order
 from .wcsmap import WcsGeom, WcsMap
-from .reproject import reproject_car_to_hpx, reproject_car_to_wcs
-
 
 __all__ = ["WcsNDMap"]
 
@@ -39,7 +36,7 @@ class WcsNDMap(WcsMap):
         Data array. If none then an empty array will be allocated.
     dtype : str, optional
         Data type, default is float32
-    meta : `~collections.OrderedDict`
+    meta : `dict`
         Dictionary to store meta data.
     unit : str or `~astropy.units.Unit`
         The map unit
@@ -58,36 +55,13 @@ class WcsNDMap(WcsMap):
     @staticmethod
     def _make_default_data(geom, shape_np, dtype):
         # Check whether corners of each image plane are valid
-        coords = []
-        if not geom.is_regular:
-            for idx in np.ndindex(geom.shape_axes):
-                pix = (
-                    np.array([0.0, float(geom.npix[0][idx] - 1)]),
-                    np.array([0.0, float(geom.npix[1][idx] - 1)]),
-                )
-                pix += tuple([np.array(2 * [t]) for t in idx])
-                coords += geom.pix_to_coord(pix)
 
-        else:
-            pix = (
-                np.array([0.0, float(geom.npix[0] - 1)]),
-                np.array([0.0, float(geom.npix[1] - 1)]),
-            )
-            pix += tuple([np.array(2 * [0.0]) for i in range(geom.ndim - 2)])
-            coords += geom.pix_to_coord(pix)
+        data = np.zeros(shape_np, dtype=dtype)
 
-        if np.all(np.isfinite(np.vstack(coords))):
-            if geom.is_regular:
-                data = np.zeros(shape_np, dtype=dtype)
-            else:
-                data = np.full(shape_np, np.nan, dtype=dtype)
-                for idx in np.ndindex(geom.shape_axes):
-                    data[idx, slice(geom.npix[0][idx]), slice(geom.npix[1][idx])] = 0.0
-        else:
-            data = np.full(shape_np, np.nan, dtype=dtype)
-            idx = geom.get_idx()
-            m = np.all(np.stack([t != INVALID_INDEX.int for t in idx]), axis=0)
-            data[m] = 0.0
+        if not geom.is_regular or geom.is_allsky:
+            coords = geom.get_coord()
+            is_nan = np.isnan(coords.lon)
+            data[is_nan] = np.nan
 
         return data
 
@@ -108,10 +82,10 @@ class WcsNDMap(WcsMap):
 
         meta = cls._get_meta_from_header(hdu.header)
         unit = unit_from_fits_image_hdu(hdu.header)
-        map_out = cls(geom, meta=meta, unit=unit)
 
         # TODO: Should we support extracting slices?
         if isinstance(hdu, fits.BinTableHDU):
+            map_out = cls(geom, meta=meta, unit=unit)
             pix = hdu.data.field("PIX")
             pix = np.unravel_index(pix, shape_wcs[::-1])
             vals = hdu.data.field("VALUE")
@@ -124,7 +98,7 @@ class WcsNDMap(WcsMap):
 
             map_out.set_by_idx(idx[::-1], vals)
         else:
-            map_out.data = hdu.data
+            map_out = cls(geom=geom, meta=meta, data=hdu.data, unit=unit)
 
         return map_out
 
@@ -154,7 +128,7 @@ class WcsNDMap(WcsMap):
         elif order == 2 or order == 3:
             return self._interp_by_pix_map_coordinates(pix, order=order)
         else:
-            raise ValueError("Invalid interpolation order: {!r}".format(order))
+            raise ValueError(f"Invalid interpolation order: {order!r}")
 
     def _interp_by_pix_linear_grid(self, pix, order=1, fill_value=None):
         # TODO: Cache interpolator
@@ -162,7 +136,7 @@ class WcsNDMap(WcsMap):
         try:
             method = method_lookup[order]
         except KeyError:
-            raise ValueError("Invalid interpolation order: {!r}".format(order))
+            raise ValueError(f"Invalid interpolation order: {order!r}")
 
         grid_pix = [np.arange(n, dtype=float) for n in self.data.shape[::-1]]
 
@@ -186,22 +160,26 @@ class WcsNDMap(WcsMap):
                 for x in pix
             ]
         )
-        return map_coordinates(self.data.T, pix, order=order, mode="nearest")
+        return scipy.ndimage.map_coordinates(
+            self.data.T, pix, order=order, mode="nearest"
+        )
 
     def _interp_by_coord_griddata(self, coords, interp=None):
         order = interp_to_order(interp)
         method_lookup = {0: "nearest", 1: "linear", 3: "cubic"}
         method = method_lookup.get(order, None)
         if method is None:
-            raise ValueError("Invalid interp: {!r}".format(interp))
+            raise ValueError(f"Invalid interp: {interp!r}")
 
         grid_coords = tuple(self.geom.get_coord(flat=True))
         data = self.data[np.isfinite(self.data)]
-        vals = griddata(grid_coords, data, tuple(coords), method=method)
+        vals = scipy.interpolate.griddata(
+            grid_coords, data, tuple(coords), method=method
+        )
 
         m = ~np.isfinite(vals)
         if np.any(m):
-            vals_fill = griddata(
+            vals_fill = scipy.interpolate.griddata(
                 grid_coords, data, tuple([c[m] for c in coords]), method="nearest"
             )
             vals[m] = vals_fill
@@ -241,75 +219,35 @@ class WcsNDMap(WcsMap):
         map_out : WcsNDMap
             Map with non-spatial axes summed over
         """
+        return self.reduce_over_axes(func=np.add, keepdims=keepdims)
+
+    def reduce_over_axes(self, func, keepdims=False):
+        """Reduce map over all non spatial axes
+
+        Parameters
+        ----------
+        func : ~numpy.ufunc
+            Function to use for reducing the data.
+        keepdims : bool, optional
+            If this is set to true, the axes which are summed over are left in
+            the map with a single bin
+
+        Returns
+        -------
+        map_out : WcsNDMap
+            Map with non-spatial axes reduced
+        """
         axis = tuple(range(self.data.ndim - 2))
         geom = self.geom.to_image()
         if keepdims:
             for ax in self.geom.axes:
                 geom = geom.to_cube([ax.squash()])
-        data = np.nansum(self.data, axis=axis, keepdims=keepdims)
+        data = func.reduce(
+            self.data, axis=axis, keepdims=keepdims, where=~np.isnan(self.data)
+        )
         # TODO: summing over the axis can change the unit, handle this correctly
         return self._init_copy(geom=geom, data=data)
 
-    def _reproject_to_wcs(self, geom, mode="interp", order=1):
-        from reproject import reproject_interp, reproject_exact
-
-        data = np.empty(geom.data_shape)
-
-        for img, idx in self.iter_by_image():
-            # TODO: Create WCS object for image plane if
-            # multi-resolution geom
-            shape_out = geom.get_image_shape(idx)[::-1]
-
-            if self.geom.projection == "CAR" and self.geom.is_allsky:
-                vals, footprint = reproject_car_to_wcs(
-                    (img, self.geom.wcs), geom.wcs, shape_out=shape_out
-                )
-            elif mode == "interp":
-                vals, footprint = reproject_interp(
-                    (img, self.geom.wcs), geom.wcs, shape_out=shape_out
-                )
-            elif mode == "exact":
-                vals, footprint = reproject_exact(
-                    (img, self.geom.wcs), geom.wcs, shape_out=shape_out
-                )
-            else:
-                raise TypeError(
-                    "mode must be 'interp' or 'exact'. Got: {!r}".format(mode)
-                )
-
-            data[idx] = vals
-
-        return self._init_copy(geom=geom, data=data)
-
-    def _reproject_to_hpx(self, geom, mode="interp", order=1):
-        from reproject import reproject_to_healpix
-
-        data = np.empty(geom.data_shape)
-        coordsys = "galactic" if geom.coordsys == "GAL" else "icrs"
-
-        for img, idx in self.iter_by_image():
-            # TODO: For partial-sky HPX we need to map from full- to
-            # partial-sky indices
-            if self.geom.projection == "CAR" and self.geom.is_allsky:
-                vals, footprint = reproject_car_to_hpx(
-                    (img, self.geom.wcs),
-                    coordsys,
-                    nside=geom.nside,
-                    nested=geom.nest,
-                    order=order,
-                )
-            else:
-                vals, footprint = reproject_to_healpix(
-                    (img, self.geom.wcs),
-                    coordsys,
-                    nside=geom.nside,
-                    nested=geom.nest,
-                    order=order,
-                )
-            data[idx] = vals
-
-        return self._init_copy(geom=geom, data=data)
-
     def pad(self, pad_width, mode="constant", cval=0, order=1):
         if np.isscalar(pad_width):
             pad_width = (pad_width, pad_width)
@@ -332,7 +270,7 @@ class WcsNDMap(WcsMap):
             kwargs["constant_values"] = cval
 
         pad_width = [(t, t) for t in pad_width]
-        data = np.pad(self.data, pad_width[::-1], mode)
+        data = np.pad(self.data, pad_width[::-1], mode, **kwargs)
         return self._init_copy(geom=geom, data=data)
 
     def _pad_coadd(self, geom, pad_width, mode, cval, order):
@@ -355,7 +293,7 @@ class WcsNDMap(WcsMap):
             vals = self.interp_by_coord(coords, interp=order)
             map_out.set_by_coord(coords, vals)
         else:
-            raise ValueError("Invalid mode: {!r}".format(mode))
+            raise ValueError(f"Invalid mode: {mode!r}")
 
         return map_out
 
@@ -394,7 +332,9 @@ class WcsNDMap(WcsMap):
             idx_ax = self.geom.get_axis_index_by_name(axis)
             pix[idx_ax] = (pix[idx_ax] - 0.5 * (factor - 1)) / factor
 
-        data = map_coordinates(self.data.T, tuple(pix), order=order, mode="nearest")
+        data = scipy.ndimage.map_coordinates(
+            self.data.T, tuple(pix), order=order, mode="nearest"
+        )
 
         if preserve_counts:
             if axis is None:
@@ -499,12 +439,14 @@ class WcsNDMap(WcsMap):
         ax.coords.frame.set_linewidth(0)
 
         # Set plot axis limits
-        ymax, xmax = self.data.shape
-        xmargin, _ = self.geom.coord_to_pix({"lon": 180, "lat": 0})
-        _, ymargin = self.geom.coord_to_pix({"lon": 0, "lat": -90})
+        xmin, _ = self.geom.coord_to_pix({"lon": 180, "lat": 0})
+        xmax, _ = self.geom.coord_to_pix({"lon": -180, "lat": 0})
 
-        ax.set_xlim(xmargin, xmax - xmargin)
-        ax.set_ylim(ymargin, ymax - ymargin)
+        _, ymin = self.geom.coord_to_pix({"lon": 0, "lat": -90})
+        _, ymax = self.geom.coord_to_pix({"lon": 0, "lat": 90})
+
+        ax.set_xlim(xmin, xmax)
+        ax.set_ylim(ymin, ymax)
 
         ax.text(0, ymax, self.geom.coordsys + " coords")
 
@@ -515,6 +457,9 @@ class WcsNDMap(WcsMap):
         lat.set_ticks(spacing=glat_spacing * u.deg)
         lon.set_ticks_visible(False)
 
+        lon.set_major_formatter("d")
+        lat.set_major_formatter("d")
+
         lon.set_ticklabel(color="w", alpha=0.8)
         lon.grid(alpha=0.2, linestyle="solid", color="w")
         lat.grid(alpha=0.2, linestyle="solid", color="w")
@@ -555,15 +500,15 @@ class WcsNDMap(WcsMap):
         for img, idx in self.iter_by_image():
             img = img.astype(float)
             if kernel == "gauss":
-                data = gaussian_filter(img, width, **kwargs)
+                data = scipy.ndimage.gaussian_filter(img, width, **kwargs)
             elif kernel == "disk":
                 disk = Tophat2DKernel(width)
                 disk.normalize("integral")
-                data = convolve(img, disk.array, **kwargs)
+                data = scipy.ndimage.convolve(img, disk.array, **kwargs)
             elif kernel == "box":
-                data = uniform_filter(img, width, **kwargs)
+                data = scipy.ndimage.uniform_filter(img, width, **kwargs)
             else:
-                raise ValueError("Invalid kernel: {!r}".format(kernel))
+                raise ValueError(f"Invalid kernel: {kernel!r}")
             smoothed_data[idx] = data
 
         return self._init_copy(data=smoothed_data)
@@ -571,23 +516,23 @@ class WcsNDMap(WcsMap):
     def get_spectrum(self, region=None, func=np.nansum):
         """Extract spectrum in a given region.
 
-        The spectrum can be computed by summing (or, more generally, applying `func`)
-        along the spatial axes in each energy bin. This occurs only inside the `region`,
+        The spectrum can be computed by summing (or, more generally, applying ``func``)
+        along the spatial axes in each energy bin. This occurs only inside the ``region``,
         which by default is assumed to be the whole spatial extension of the map.
 
         Parameters
         ----------
         region: `~regions.Region`
              Region (pixel or sky regions accepted).
-        func : `~numpy.ufunc`
+        func : numpy.ufunc
             Function to reduce the data.
 
         Returns
         -------
-        spectrum : `CountsSpectrum`
+        spectrum : `~gammapy.spectrum.CountsSpectrum`
             Spectrum in the given region.
         """
-        from ..spectrum import CountsSpectrum
+        from gammapy.spectrum import CountsSpectrum
 
         energy_axis = self.geom.get_axis_by_name("energy")
 
@@ -626,9 +571,9 @@ class WcsNDMap(WcsMap):
         map : `WcsNDMap`
             Convolved map.
         """
-        from ..cube.psf_kernel import PSFKernel
+        from gammapy.cube import PSFKernel
 
-        conv_function = fftconvolve if use_fft else convolve
+        conv_function = scipy.signal.fftconvolve if use_fft else scipy.ndimage.convolve
         convolved_data = np.empty(self.data.shape, dtype=np.float32)
         if use_fft:
             kwargs.setdefault("mode", "same")
@@ -647,6 +592,28 @@ class WcsNDMap(WcsMap):
 
         return self._init_copy(data=convolved_data)
 
+    def apply_edisp(self, edisp):
+        """Apply energy dispersion to map. Requires energy axis.
+
+        Parameters
+        ----------
+        edisp : `gammapy.irf.EnergyDispersion`
+            Energy dispersion matrix
+
+        Returns
+        -------
+        map : `WcsNDMap`
+            Map with energy dispersion applied.
+        """
+        loc = self.geom.get_axis_index_by_name("energy")
+        data = np.rollaxis(self.data, loc, len(self.data.shape))
+        data = np.dot(data, edisp.pdf_matrix)
+        data = np.rollaxis(data, -1, loc)
+
+        e_reco_axis = edisp.e_reco.copy(name="energy")
+        geom_reco = self.geom.to_image().to_cube(axes=[e_reco_axis])
+        return self._init_copy(geom=geom_reco, data=data)
+
     def cutout(self, position, width, mode="trim"):
         """
         Create a cutout around a given position.
@@ -666,21 +633,70 @@ class WcsNDMap(WcsMap):
         cutout : `~gammapy.maps.WcsNDMap`
             Cutout map
         """
-        width = _check_width(width)
-        idx = (0,) * len(self.geom.axes)
-        c2d = Cutout2D(
-            data=self.data[idx],
-            wcs=self.geom.wcs,
-            position=position,
-            # Cutout2D takes size with order (lat, lon)
-            size=width[::-1] * u.deg,
-            mode=mode,
-        )
+        geom_cutout = self.geom.cutout(position=position, width=width, mode=mode)
 
-        # Create the slices with the non-spatial axis
-        cutout_slices = Ellipsis, c2d.slices_original[0], c2d.slices_original[1]
+        slices = geom_cutout.cutout_info["parent-slices"]
+        cutout_slices = Ellipsis, slices[0], slices[1]
 
-        geom = WcsGeom(c2d.wcs, c2d.shape[::-1], axes=self.geom.axes)
         data = self.data[cutout_slices]
 
-        return self._init_copy(geom=geom, data=data)
+        return self._init_copy(geom=geom_cutout, data=data)
+
+    def stack(self, other, weights=None):
+        """Stack cutout into map.
+
+        Parameters
+        ----------
+        other : `WcsNDMap`
+            Other map to stack
+        weights : `WcsNDMap`
+            Array to be used as weights.
+        """
+        if self.geom == other.geom:
+            parent_slices, cutout_slices = None, None
+        elif self.geom.is_aligned(other.geom):
+            slices = other.geom.cutout_info["parent-slices"]
+            parent_slices = Ellipsis, slices[0], slices[1]
+
+            slices = other.geom.cutout_info["cutout-slices"]
+            cutout_slices = Ellipsis, slices[0], slices[1]
+        else:
+            raise ValueError(
+                "Can only stack equivalent maps or cutout of the same map."
+            )
+
+        data = other.data[cutout_slices]
+
+        if weights is not None:
+            data = data * weights.data
+
+        self.data[parent_slices] += data
+
+    def sample_coord(self, n_events, random_state=0):
+        """Sample position and energy of events.
+
+        Parameters
+        ----------
+        n_events : int
+            Number of events to sample.
+        random_state : {int, 'random-seed', 'global-rng', `~numpy.random.RandomState`}
+            Defines random number generator initialisation.
+            Passed to `~gammapy.utils.random.get_random_state`.
+
+        Returns
+        -------
+        coords : `~gammapy.maps.MapCoord` object.
+            Sequence of coordinates and energies of the sampled events.
+        """
+
+        random_state = get_random_state(random_state)
+        sampler = InverseCDFSampler(pdf=self.data, random_state=random_state)
+
+        coords_pix = sampler.sample(n_events)
+        coords = self.geom.pix_to_coord(coords_pix[::-1])
+
+        # TODO: pix_to_coord should return a MapCoord object
+        axes_names = ["lon", "lat"] + [ax.name for ax in self.geom.axes]
+        cdict = OrderedDict(zip(axes_names, coords))
+
+        return MapCoord.create(cdict, coordsys=self.geom.coordsys)
