@@ -1,47 +1,48 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+import importlib
+import logging
 import os
 import platform
 import sys
 import warnings
-import logging
-import importlib
-from collections import OrderedDict
 import click
-from .. import version
+from gammapy import __version__
 
 log = logging.getLogger(__name__)
 
+# Should be in sync with `docs/install/dependencies.rst`
 GAMMAPY_DEPENDENCIES = [
+    # required
     "numpy",
     "scipy",
-    "matplotlib",
-    "cython",
     "astropy",
-    "astropy_healpix",
-    "reproject",
-    "sherpa",
-    "pytest",
-    "sphinx",
-    "healpy",
     "regions",
+    "click",
+    "yaml",
+    # "pydantic",  # has no __version__
+    # optional
+    "IPython",
+    # "jupyter",   # has no __version__
+    "jupyterlab",
+    "matplotlib",
+    "pandas",
+    "healpy",
     "iminuit",
+    "sherpa",
     "naima",
-    "uncertainties",
+    "emcee",
+    "corner",
+    "parfive",
 ]
 
-GAMMAPY_ENV_VARIABLES = [
-    "GAMMA_CAT",
-    "GAMMAPY_DATA",
-    "GAMMAPY_EXTRA",
-    "GAMMAPY_FERMI_LAT_DATA",
-]
+GAMMAPY_ENV_VARIABLES = ["GAMMAPY_DATA", "GAMMAPY_EXTRA"]
 
 
 @click.command(name="info")
 @click.option("--system/--no-system", default=True, help="Show system info")
-@click.option("--version/--no-version", default=True, help="Show version info")
+@click.option("--version/--no-version", default=True, help="Show version")
 @click.option(
-    "--dependencies/--no-dependencies", default=True, help="Show dependencies info"
+    "--dependencies/--no-dependencies", default=True, help="Show dependencies"
 )
 @click.option("--envvar/--no-envvar", default=True, help="Show environment variables")
 def cli_info(system, version, dependencies, envvar):
@@ -66,41 +67,39 @@ def cli_info(system, version, dependencies, envvar):
 
 def print_info(info, title):
     """Print Gammapy info."""
-    info_all = "\n{}:\n\n".format(title)
+    info_all = f"\n{title}:\n\n"
 
     for key, value in info.items():
-        info_all += "\t{:22s} : {:<10s} \n".format(key, value)
+        info_all += f"\t{key:22s} : {value:<10s} \n"
 
     print(info_all)
 
 
 def get_info_system():
     """Get info about user system"""
-    info = OrderedDict()
-    info["python_executable"] = sys.executable
-    info["python_version"] = platform.python_version()
-    info["machine"] = platform.machine()
-    info["system"] = platform.system()
-    return info
+    return {
+        "python_executable": sys.executable,
+        "python_version": platform.python_version(),
+        "machine": platform.machine(),
+        "system": platform.system(),
+    }
 
 
 def get_info_version():
     """Get detailed info about Gammapy version."""
-    info = OrderedDict()
+    info = {"version": __version__}
     try:
         path = sys.modules["gammapy"].__path__[0]
     except:
         path = "unknown"
     info["path"] = path
-    info["version"] = version.version
-    if not version.release:
-        info["githash"] = version.githash
+
     return info
 
 
 def get_info_dependencies():
     """Get info about Gammapy dependencies."""
-    info = OrderedDict()
+    info = {}
     for name in GAMMAPY_DEPENDENCIES:
         try:
             with warnings.catch_warnings():
@@ -116,7 +115,4 @@ def get_info_dependencies():
 
 def get_info_envvar():
     """Get info about Gammapy environment variables."""
-    info = OrderedDict()
-    for name in GAMMAPY_ENV_VARIABLES:
-        info[name] = os.environ.get(name, "not set")
-    return info
+    return {name: os.environ.get(name, "not set") for name in GAMMAPY_ENV_VARIABLES}
