@@ -1,60 +1,55 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 import logging
-from collections import OrderedDict
 import numpy as np
-from astropy.table import Table, vstack
 from astropy import units as u
 from astropy.io.registry import IORegistryError
-from ..utils.scripts import make_path
-from ..utils.table import table_standardise_units_copy, table_from_row_data
-from ..utils.interpolation import interpolate_likelihood_profile
-from ..utils.fitting import Dataset, Datasets, Fit
-from .models import PowerLaw, ScaleModel
+from astropy.table import Table, vstack
+from gammapy.modeling import Dataset, Datasets, Fit, Parameters
+from gammapy.modeling.models import (
+    PowerLawSpectralModel,
+    ScaleSpectralModel,
+    SkyModel,
+    SkyModels,
+)
+from gammapy.utils.interpolation import interpolate_profile
+from gammapy.utils.scripts import make_path
+from gammapy.utils.table import table_from_row_data, table_standardise_units_copy
 from .dataset import SpectrumDatasetOnOff
 
 __all__ = ["FluxPoints", "FluxPointsEstimator", "FluxPointsDataset"]
 
 log = logging.getLogger(__name__)
 
-REQUIRED_COLUMNS = OrderedDict(
-    [
-        ("dnde", ["e_ref", "dnde"]),
-        ("e2dnde", ["e_ref", "e2dnde"]),
-        ("flux", ["e_min", "e_max", "flux"]),
-        ("eflux", ["e_min", "e_max", "eflux"]),
-        # TODO: extend required columns
-        (
-            "likelihood",
-            [
-                "e_min",
-                "e_max",
-                "e_ref",
-                "ref_dnde",
-                "norm",
-                "norm_scan",
-                "dloglike_scan",
-            ],
-        ),
-    ]
-)
+REQUIRED_COLUMNS = {
+    "dnde": ["e_ref", "dnde"],
+    "e2dnde": ["e_ref", "e2dnde"],
+    "flux": ["e_min", "e_max", "flux"],
+    "eflux": ["e_min", "e_max", "eflux"],
+    # TODO: extend required columns
+    "likelihood": [
+        "e_min",
+        "e_max",
+        "e_ref",
+        "ref_dnde",
+        "norm",
+        "norm_scan",
+        "stat_scan",
+    ],
+}
 
-OPTIONAL_COLUMNS = OrderedDict(
-    [
-        ("dnde", ["dnde_err", "dnde_errp", "dnde_errn", "dnde_ul", "is_ul"]),
-        ("e2dnde", ["e2dnde_err", "e2dnde_errp", "e2dnde_errn", "e2dnde_ul", "is_ul"]),
-        ("flux", ["flux_err", "flux_errp", "flux_errn", "flux_ul", "is_ul"]),
-        ("eflux", ["eflux_err", "eflux_errp", "eflux_errn", "eflux_ul", "is_ul"]),
-    ]
-)
+OPTIONAL_COLUMNS = {
+    "dnde": ["dnde_err", "dnde_errp", "dnde_errn", "dnde_ul", "is_ul"],
+    "e2dnde": ["e2dnde_err", "e2dnde_errp", "e2dnde_errn", "e2dnde_ul", "is_ul"],
+    "flux": ["flux_err", "flux_errp", "flux_errn", "flux_ul", "is_ul"],
+    "eflux": ["eflux_err", "eflux_errp", "eflux_errn", "eflux_ul", "is_ul"],
+}
 
-DEFAULT_UNIT = OrderedDict(
-    [
-        ("dnde", u.Unit("cm-2 s-1 TeV-1")),
-        ("e2dnde", u.Unit("erg cm-2 s-1")),
-        ("flux", u.Unit("cm-2 s-1")),
-        ("eflux", u.Unit("erg cm-2 s-1")),
-    ]
-)
+DEFAULT_UNIT = {
+    "dnde": u.Unit("cm-2 s-1 TeV-1"),
+    "e2dnde": u.Unit("erg cm-2 s-1"),
+    "flux": u.Unit("cm-2 s-1"),
+    "eflux": u.Unit("erg cm-2 s-1"),
+}
 
 
 class FluxPoints:
@@ -97,10 +92,10 @@ class FluxPoints:
         from astropy import units as u
         from astropy.table import Table
         from gammapy.spectrum import FluxPoints
-        from gammapy.spectrum.models import PowerLaw
+        from gammapy.modeling.models import PowerLawSpectralModel
 
         table = Table()
-        pwl = PowerLaw()
+        pwl = PowerLawSpectralModel()
         e_ref = np.logspace(0, 2, 7) * u.TeV
         table['e_ref'] = e_ref
         table['dnde'] = pwl(e_ref)
@@ -144,8 +139,7 @@ class FluxPoints:
         self._validate_table(self.table, table.meta["SED_TYPE"])
 
     def __repr__(self):
-        fmt = '{}(sed_type="{}", n_points={})'
-        return fmt.format(self.__class__.__name__, self.sed_type, len(self.table))
+        return f"{self.__class__.__name__}(sed_type={self.sed_type!r}, n_points={len(self.table)})"
 
     @property
     def table_formatted(self):
@@ -156,7 +150,7 @@ class FluxPoints:
             if column.startswith(("dnde", "eflux", "flux", "e2dnde", "ref")):
                 table[column].format = ".3e"
             elif column.startswith(
-                ("e_min", "e_max", "e_ref", "sqrt_ts", "norm", "ts", "loglike")
+                ("e_min", "e_max", "e_ref", "sqrt_ts", "norm", "ts", "stat")
             ):
                 table[column].format = ".3f"
 
@@ -175,15 +169,30 @@ class FluxPoints:
         """
         filename = make_path(filename)
         try:
-            table = Table.read(str(filename), **kwargs)
+            table = Table.read(filename, **kwargs)
         except IORegistryError:
             kwargs.setdefault("format", "ascii.ecsv")
-            table = Table.read(str(filename), **kwargs)
+            table = Table.read(filename, **kwargs)
 
         if "SED_TYPE" not in table.meta.keys():
             sed_type = cls._guess_sed_type(table)
             table.meta["SED_TYPE"] = sed_type
 
+        # TODO: check sign and factor 2 here
+        # https://github.com/gammapy/gammapy/pull/2546#issuecomment-554274318
+        # The idea below is to support the format here:
+        # https://gamma-astro-data-formats.readthedocs.io/en/latest/spectra/flux_points/index.html#likelihood-columns
+        # but internally to go to the uniform "stat"
+
+        if "loglike" in table.colnames and "stat" not in table.colnames:
+            table["stat"] = 2 * table["loglike"]
+
+        if "loglike_null" in table.colnames and "stat_null" not in table.colnames:
+            table["stat_null"] = 2 * table["loglike_null"]
+
+        if "dloglike_scan" in table.colnames and "stat_scan" not in table.colnames:
+            table["stat_scan"] = 2 * table["dloglike_scan"]
+
         return cls(table=table)
 
     def write(self, filename, **kwargs):
@@ -198,10 +207,10 @@ class FluxPoints:
         """
         filename = make_path(filename)
         try:
-            self.table.write(str(filename), **kwargs)
+            self.table.write(filename, **kwargs)
         except IORegistryError:
             kwargs.setdefault("format", "ascii.ecsv")
-            self.table.write(str(filename), **kwargs)
+            self.table.write(filename, **kwargs)
 
     @classmethod
     def stack(cls, flux_points):
@@ -259,7 +268,7 @@ class FluxPoints:
 
     def _flux_to_dnde(self, e_ref, table, model, pwl_approx):
         if model is None:
-            model = PowerLaw()
+            model = PowerLawSpectralModel()
 
         e_min, e_max = self.e_min, self.e_max
 
@@ -320,7 +329,7 @@ class FluxPoints:
         ----------
         sed_type : {'dnde'}
              SED type to convert to.
-        model : `~gammapy.spectrum.models.SpectralModel`
+        model : `~gammapy.modeling.models.SpectralModel`
             Spectral model assumption.  Note that the value of the amplitude parameter
             does not matter. Still it is recommended to use something with the right
             scale and units. E.g. `amplitude = 1e-12 * u.Unit('cm-2 s-1 TeV-1')`
@@ -343,10 +352,10 @@ class FluxPoints:
         Examples
         --------
         >>> from gammapy.spectrum import FluxPoints
-        >>> from gammapy.spectrum.models import PowerLaw
+        >>> from gammapy.modeling.models import PowerLawSpectralModel
         >>> filename = '$GAMMAPY_DATA/tests/spectrum/flux_points/flux_points.fits'
         >>> flux_points = FluxPoints.read(filename)
-        >>> model = PowerLaw(index=2.2)
+        >>> model = PowerLawSpectralModel(index=2.2)
         >>> flux_points_dnde = flux_points.to_sed_type('dnde', model=model)
         """
         # TODO: implement other directions.
@@ -362,7 +371,7 @@ class FluxPoints:
                 # set e_ref that it represents the mean dnde in the given energy bin
                 e_ref = self._e_ref_lafferty(model, self.e_min, self.e_max)
             else:
-                raise ValueError("Invalid method: {}".format(method))
+                raise ValueError(f"Invalid method: {method}")
             table = self._flux_to_dnde(e_ref, table, model, pwl_approx)
 
         elif self.sed_type == "dnde" and sed_type == "e2dnde":
@@ -407,7 +416,7 @@ class FluxPoints:
 
         if pwl_approx:
             index = model.spectral_index(e_ref)
-            flux_model = PowerLaw.evaluate_integral(
+            flux_model = PowerLawSpectralModel.evaluate_integral(
                 emin=e_min,
                 emax=e_max,
                 index=index,
@@ -642,25 +651,25 @@ class FluxPoints:
                 xerr=x_err,
                 yerr=y_err,
                 uplims=True,
-                **kwargs
+                **kwargs,
             )
 
         ax.set_xscale("log", nonposx="clip")
         ax.set_yscale("log", nonposy="clip")
-        ax.set_xlabel("Energy ({})".format(energy_unit))
-        ax.set_ylabel("{} ({})".format(self.sed_type, y_unit))
+        ax.set_xlabel(f"Energy ({energy_unit})")
+        ax.set_ylabel(f"{self.sed_type} ({y_unit})")
         return ax
 
-    def plot_likelihood(
+    def plot_ts_profiles(
         self,
         ax=None,
         energy_unit="TeV",
         add_cbar=True,
         y_values=None,
         y_unit=None,
-        **kwargs
+        **kwargs,
     ):
-        """Plot likelihood SED profiles as a density plot..
+        """Plot fit statistic SED profiles as a density plot.
 
         Parameters
         ----------
@@ -669,7 +678,7 @@ class FluxPoints:
         energy_unit : str, `~astropy.units.Unit`, optional
             Unit of the energy axis
         y_values : `astropy.units.Quantity`
-            Array of y-values to use for the likelihood profile evaluation.
+            Array of y-values to use for the fit statistic profile evaluation.
         y_unit : str or `astropy.units.Unit`
             Unit to use for the y-axis.
         add_cbar : bool
@@ -701,15 +710,15 @@ class FluxPoints:
 
         x = self.e_edges.to(energy_unit)
 
-        # Compute likelihood "image" one energy bin at a time
+        # Compute fit statistic "image" one energy bin at a time
         # by interpolating e2dnde at the log bin centers
         z = np.empty((len(self.table), len(y_values)))
         for idx, row in enumerate(self.table):
             y_ref = self.table["ref_" + self.sed_type].quantity[idx]
             norm = (y_values / y_ref).to_value("")
             norm_scan = row["norm_scan"]
-            dloglike_scan = row["dloglike_scan"] - row["loglike"]
-            interp = interpolate_likelihood_profile(norm_scan, dloglike_scan)
+            ts_scan = row["stat_scan"] - row["stat"]
+            interp = interpolate_profile(norm_scan, ts_scan)
             z[idx] = interp((norm,))
 
         kwargs.setdefault("vmax", 0)
@@ -720,14 +729,14 @@ class FluxPoints:
 
         # clipped values are set to NaN so that they appear white on the plot
         z[-z < kwargs["vmin"]] = np.nan
-        caxes = ax.pcolormesh(x, y_values, -z.T, **kwargs)
+        caxes = ax.pcolormesh(x.value, y_values.value, -z.T, **kwargs)
         ax.set_xscale("log", nonposx="clip")
         ax.set_yscale("log", nonposy="clip")
-        ax.set_xlabel("Energy ({})".format(energy_unit))
-        ax.set_ylabel("{} ({})".format(self.sed_type, y_values.unit))
+        ax.set_xlabel(f"Energy ({energy_unit})")
+        ax.set_ylabel(f"{self.sed_type} ({y_values.unit})")
 
         if add_cbar:
-            label = "delta log-likelihood"
+            label = "fit statistic difference"
             ax.figure.colorbar(caxes, ax=ax, label=label)
 
         return ax
@@ -753,20 +762,20 @@ class FluxPointsEstimator:
 
     Parameters
     ----------
-    datasets : list of `~gammapy.spectrum.SpectrumDatatset`
+    datasets : list of `~gammapy.spectrum.SpectrumDataset`
         Spectrum datasets.
     e_edges : `~astropy.units.Quantity`
         Energy edges of the flux point bins.
     source : str
         For which source in the model to compute the flux points.
     norm_min : float
-        Minimum value for the norm used for the likelihood profile evaluation.
+        Minimum value for the norm used for the fit statistic profile evaluation.
     norm_max : float
-        Maximum value for the norm used for the likelihood profile evaluation.
+        Maximum value for the norm used for the fit statistic profile evaluation.
     norm_n_values : int
-        Number of norm values used for the likelihood profile.
+        Number of norm values used for the fit statistic profile.
     norm_values : `numpy.ndarray`
-        Array of norm values to be used for the likelihood profile.
+        Array of norm values to be used for the fit statistic profile.
     sigma : int
         Sigma to use for asymmetric error computation.
     sigma_ul : int
@@ -801,14 +810,15 @@ class FluxPointsEstimator:
         self.datasets = datasets.copy()
         self.e_edges = e_edges
 
-        dataset = self.datasets.datasets[0]
+        dataset = self.datasets[0]
 
-        if isinstance(dataset, SpectrumDatasetOnOff):
-            model = dataset.model
+        # TODO: this is complex and non-obvious behaviour. Simlify!
+        if len(dataset.models) > 1:
+            model = dataset.models[source].spectral_model
         else:
-            model = dataset.model[source].spectral_model
+            model = dataset.models[0].spectral_model
 
-        self.model = ScaleModel(model)
+        self.model = ScaleSpectralModel(model)
         self.model.norm.min = 0
         self.model.norm.max = 1e3
 
@@ -825,6 +835,7 @@ class FluxPointsEstimator:
         self.fit = Fit(self.datasets)
 
         self._set_scale_model()
+        self._contribute_to_stat = False
 
     def _freeze_parameters(self):
         # freeze other parameters
@@ -833,22 +844,22 @@ class FluxPointsEstimator:
                 par.frozen = True
 
     def _freeze_empty_background(self):
-        from ..cube import MapDataset
+        from gammapy.cube import MapDataset
 
         counts_all = self.estimate_counts()["counts"]
 
-        for counts, dataset in zip(counts_all, self.datasets.datasets):
+        for counts, dataset in zip(counts_all, self.datasets):
             if isinstance(dataset, MapDataset) and counts == 0:
                 if dataset.background_model is not None:
                     dataset.background_model.parameters.freeze_all()
 
     def _set_scale_model(self):
         # set the model on all datasets
-        for dataset in self.datasets.datasets:
-            if isinstance(dataset, SpectrumDatasetOnOff):
-                dataset.model = self.model
+        for dataset in self.datasets:
+            if len(dataset.models) > 1:
+                dataset.models[self.source].spectral_model = self.model
             else:
-                dataset.model[self.source].spectral_model = self.model
+                dataset.models[0].spectral_model = self.model
 
     @property
     def ref_model(self):
@@ -857,7 +868,7 @@ class FluxPointsEstimator:
     @property
     def e_groups(self):
         """Energy grouping table `~astropy.table.Table`"""
-        dataset = self.datasets.datasets[0]
+        dataset = self.datasets[0]
         if isinstance(dataset, SpectrumDatasetOnOff):
             energy_axis = dataset.counts.energy
         else:
@@ -865,7 +876,7 @@ class FluxPointsEstimator:
         return energy_axis.group_table(self.e_edges)
 
     def __str__(self):
-        s = "{}:\n".format(self.__class__.__name__)
+        s = f"{self.__class__.__name__}:\n"
         s += str(self.datasets) + "\n"
         s += str(self.e_edges) + "\n"
         s += str(self.model) + "\n"
@@ -883,7 +894,6 @@ class FluxPointsEstimator:
             and available options.
         """
         rows = []
-
         for e_group in self.e_groups:
             if e_group["bin_type"].strip() != "normal":
                 log.debug("Skipping under-/ overflow bin in flux point estimation.")
@@ -892,12 +902,11 @@ class FluxPointsEstimator:
             row = self.estimate_flux_point(e_group, steps=steps)
             rows.append(row)
 
-        meta = OrderedDict([("SED_TYPE", "likelihood")])
-        table = table_from_row_data(rows=rows, meta=meta)
+        table = table_from_row_data(rows=rows, meta={"SED_TYPE": "likelihood"})
         return FluxPoints(table).to_sed_type("dnde")
 
-    def _energy_mask(self, e_group):
-        energy_mask = np.zeros(self.datasets.datasets[0].data_shape)
+    def _energy_mask(self, e_group, dataset):
+        energy_mask = np.zeros(dataset.data_shape)
         energy_mask[e_group["idx_min"] : e_group["idx_max"] + 1] = 1
         return energy_mask.astype(bool)
 
@@ -915,7 +924,7 @@ class FluxPointsEstimator:
                 * "errn-errp": estimate asymmetric errors.
                 * "ul": estimate upper limits.
                 * "ts": estimate ts and sqrt(ts) values.
-                * "norm-scan": estimate likelihood profiles.
+                * "norm-scan": estimate fit statistic profiles.
 
             By default all steps are executed.
 
@@ -928,37 +937,24 @@ class FluxPointsEstimator:
         # Put at log center of the bin
         e_ref = np.sqrt(e_min * e_max)
 
-        result = OrderedDict(
-            [
-                ("e_ref", e_ref),
-                ("e_min", e_min),
-                ("e_max", e_max),
-                ("ref_dnde", self.ref_model(e_ref)),
-                ("ref_flux", self.ref_model.integral(e_min, e_max)),
-                ("ref_eflux", self.ref_model.energy_flux(e_min, e_max)),
-                ("ref_e2dnde", self.ref_model(e_ref) * e_ref ** 2),
-            ]
-        )
+        result = {
+            "e_ref": e_ref,
+            "e_min": e_min,
+            "e_max": e_max,
+            "ref_dnde": self.ref_model(e_ref),
+            "ref_flux": self.ref_model.integral(e_min, e_max),
+            "ref_eflux": self.ref_model.energy_flux(e_min, e_max),
+            "ref_e2dnde": self.ref_model(e_ref) * e_ref ** 2,
+        }
 
-        contribute_to_likelihood = False
-
-        for dataset in self.datasets.datasets:
-            dataset.mask_fit = self._energy_mask(e_group)
+        for dataset in self.datasets:
+            dataset.mask_fit = self._energy_mask(e_group=e_group, dataset=dataset)
             mask = dataset.mask_fit
 
             if dataset.mask_safe is not None:
                 mask &= dataset.mask_safe
 
-            contribute_to_likelihood |= mask.any()
-
-        if not contribute_to_likelihood:
-            raise ValueError(
-                "No dataset contributes to the likelihood between"
-                " {e_min:.3f} and {e_max:.3f}. Please adapt the "
-                "flux point energy edges or check the dataset masks.".format(
-                    e_min=e_min, e_max=e_max
-                )
-            )
+            self._contribute_to_stat |= mask.any()
 
         with self.datasets.parameters.restore_values:
 
@@ -1006,6 +1002,9 @@ class FluxPointsEstimator:
         result : dict
             Dict with asymmetric errors for the flux point norm.
         """
+        if not self._contribute_to_stat:
+            return {"norm_errp": np.nan, "norm_errn": np.nan}
+
         result = self.fit.confidence(parameter=self.model.norm, sigma=self.sigma)
         return {"norm_errp": result["errp"], "norm_errn": result["errn"]}
 
@@ -1017,6 +1016,9 @@ class FluxPointsEstimator:
         result : dict
             Dict with symmetric error for the flux point norm.
         """
+        if not self._contribute_to_stat:
+            return {"norm_err": np.nan}
+
         result = self.fit.covariance()
         norm_err = result.parameters.error(self.model.norm)
         return {"norm_err": norm_err}
@@ -1029,9 +1031,11 @@ class FluxPointsEstimator:
         result : dict
             Dict with an array with one entry per dataset with counts for the flux point.
         """
-        counts = []
+        if not self._contribute_to_stat:
+            return {"counts": np.zeros(len(self.datasets))}
 
-        for dataset in self.datasets.datasets:
+        counts = []
+        for dataset in self.datasets:
             mask = dataset.mask_fit
             if dataset.mask_safe is not None:
                 mask &= dataset.mask_safe
@@ -1048,9 +1052,12 @@ class FluxPointsEstimator:
         result : dict
             Dict with upper limit for the flux point norm.
         """
+        if not self._contribute_to_stat:
+            return {"norm_ul": np.nan}
+
         norm = self.model.norm
 
-        # TODO: the minuit backend has convergence problems when the likelihood is not
+        # TODO: the minuit backend has convergence problems when the fit statistic is not
         #  of parabolic shape, which is the case, when there are zero counts in the
         #  energy bin. For this case we change to the scipy backend.
         counts = self.estimate_counts()["counts"]
@@ -1075,7 +1082,10 @@ class FluxPointsEstimator:
         result : dict
             Dict with ts and sqrt(ts) for the flux point.
         """
-        loglike = self.datasets.likelihood()
+        if not self._contribute_to_stat:
+            return {"sqrt_ts": np.nan, "ts": np.nan}
+
+        stat = self.datasets.stat_sum()
 
         # store best fit amplitude, set amplitude of fit model to zero
         self.model.norm.value = 0
@@ -1084,26 +1094,29 @@ class FluxPointsEstimator:
         if self.reoptimize:
             _ = self.fit.optimize()
 
-        loglike_null = self.datasets.likelihood()
+        stat_null = self.datasets.stat_sum()
 
         # compute sqrt TS
-        ts = np.abs(loglike_null - loglike)
+        ts = np.abs(stat_null - stat)
         sqrt_ts = np.sqrt(ts)
         return {"sqrt_ts": sqrt_ts, "ts": ts}
 
     def estimate_norm_scan(self):
-        """Estimate likelihood profile for the norm parameter.
+        """Estimate fit statistic profile for the norm parameter.
 
         Returns
         -------
         result : dict
-            Dict with norm_scan and dloglike_scan for the flux point.
+            Keys: "norm_scan", "stat_scan"
         """
-        result = self.fit.likelihood_profile(
+        if not self._contribute_to_stat:
+            nans = np.nan * np.empty_like(self.norm_values)
+            return {"norm_scan": nans, "stat_scan": nans}
+
+        result = self.fit.stat_profile(
             self.model.norm, values=self.norm_values, reoptimize=self.reoptimize
         )
-        dloglike_scan = result["likelihood"]
-        return {"norm_scan": result["values"], "dloglike_scan": dloglike_scan}
+        return {"norm_scan": result["values"], "stat_scan": result["stat"]}
 
     def estimate_norm(self):
         """Fit norm of the flux point.
@@ -1111,8 +1124,11 @@ class FluxPointsEstimator:
         Returns
         -------
         result : dict
-            Dict with "norm" and "loglike" for the flux point.
+            Dict with "norm" and "stat" for the flux point.
         """
+        if not self._contribute_to_stat:
+            return {"norm": np.nan, "stat": np.nan, "success": False}
+
         # start optimization with norm=1
         self.model.norm.value = 1.0
         self.model.norm.frozen = False
@@ -1124,7 +1140,7 @@ class FluxPointsEstimator:
         else:
             norm = np.nan
 
-        return {"norm": norm, "loglike": result.total_stat, "success": result.success}
+        return {"norm": norm, "stat": result.total_stat, "success": result.success}
 
 
 class FluxPointsDataset(Dataset):
@@ -1133,14 +1149,12 @@ class FluxPointsDataset(Dataset):
 
     Parameters
     ----------
-    model : `~gammapy.spectrum.models.SpectralModel`
-        Spectral model
+    models : `~gammapy.modeling.models.SkyModels`
+        Models (only spectral part needs to be set)
     data : `~gammapy.spectrum.FluxPoints`
         Flux points.
     mask_fit : `numpy.ndarray`
-        Mask to apply to the likelihood for fitting.
-    likelihood : {"chi2", "chi2assym"}
-        Likelihood function to use for the fit.
+        Mask to apply for fitting
     mask_safe : `numpy.ndarray`
         Mask defining the safe data range.
 
@@ -1148,29 +1162,30 @@ class FluxPointsDataset(Dataset):
     --------
     Load flux points from file and fit with a power-law model::
 
-        from astropy import units as u
+        from gammapy.modeling import Fit
+        from gammapy.modeling.models import PowerLawSpectralModel, SkyModel
         from gammapy.spectrum import FluxPoints, FluxPointsDataset
-        from gammapy.utils.fitting import Fit
-        from gammapy.spectrum.models import PowerLaw
 
-        filename = '$GAMMAPY_DATA/tests/spectrum/flux_points/diff_flux_points.fits'
+        filename = "$GAMMAPY_DATA/tests/spectrum/flux_points/diff_flux_points.fits"
         flux_points = FluxPoints.read(filename)
 
-        model = PowerLaw()
+        model = SkyModel(spectral_model=PowerLawSpectralModel())
 
         dataset = FluxPointsDataset(model, flux_points)
-        fit = Fit(dataset)
+        fit = Fit([dataset])
         result = fit.run()
         print(result)
         print(result.parameters.to_table())
     """
 
-    def __init__(self, model, data, mask_fit=None, likelihood="chi2", mask_safe=None):
-        self.model = model
+    likelihood_type = "chi2"
+    tag = "FluxPointsDataset"
+
+    def __init__(self, models, data, mask_fit=None, mask_safe=None, name=""):
         self.data = data
         self.mask_fit = mask_fit
-        self.parameters = model.parameters
-
+        self.name = name
+        self.models = models
         if data.sed_type != "dnde":
             raise ValueError("Currently only flux points of type 'dnde' are supported.")
 
@@ -1179,22 +1194,106 @@ class FluxPointsDataset(Dataset):
 
         self.mask_safe = mask_safe
 
-        if likelihood in ["chi2", "chi2assym"]:
-            self._likelihood = likelihood
-        else:
-            raise ValueError(
-                "'{likelihood}' is not a valid fit statistic, please choose"
-                " either 'chi2' or 'chi2assym'"
-            )
+    @property
+    def models(self):
+        return self._models
 
-    def __repr__(self):
-        str_ = self.__class__.__name__
-        return str_
+    @models.setter
+    def models(self, value):
+        if isinstance(value, SkyModels):
+            models = value
+        elif isinstance(value, list):
+            models = SkyModels(value)
+        elif isinstance(value, SkyModel):
+            models = SkyModels([value])
+        else:
+            raise TypeError(f"Invalid: {value!r}")
+
+        self._models = models
+
+    @property
+    def parameters(self):
+        """List of parameters (`~gammapy.modeling.Parameters`)"""
+        parameters = []
+
+        for component in self.models:
+            parameters.append(component.spectral_model.parameters)
+
+        return Parameters.from_stack(parameters)
+
+    def write(self, filename, overwrite=True, **kwargs):
+        """Write flux point dataset to file.
+
+        Parameters
+        ----------
+        filename : str
+            Filename to write to.
+        overwrite : bool
+            Overwrite existing file.
+        **kwargs : dict
+             Keyword arguments passed to `~astropy.table.Table.write`.
+        """
+        table = self.data.table.copy()
+        if self.mask_fit is None:
+            mask_fit = self.mask_safe
+        else:
+            mask_fit = self.mask_fit
+
+        table["mask_fit"] = mask_fit
+        table["mask_safe"] = self.mask_safe
+        table.write(filename, overwrite=overwrite, **kwargs)
+
+    @classmethod
+    def from_dict(cls, data, components, models):
+        """Create flux point dataset from dict.
+
+        Parameters
+        ----------
+        data : dict
+            Dict containing data to create dataset from.
+        components : list of dict
+            Not used.
+        models : list of `SkyModel`
+            List of model components.
+
+        Returns
+        -------
+        dataset : `FluxPointsDataset`
+            Flux point datasets.
+        """
+        models = [model for model in models if model.name in data["models"]]
+        # TODO: assumes that the model is a skymodel
+        # so this will work only when this change will be effective
+        table = Table.read(data["filename"])
+        mask_fit = table["mask_fit"].data.astype("bool")
+        mask_safe = table["mask_safe"].data.astype("bool")
+        table.remove_columns(["mask_fit", "mask_safe"])
+        return cls(
+            models=models,
+            name=data["name"],
+            data=FluxPoints(table),
+            mask_fit=mask_fit,
+            mask_safe=mask_safe,
+        )
+
+    def to_dict(self, filename=""):
+        """Convert to dict for YAML serialization."""
+        if self.models is not None:
+            models = [_.name for _ in self.models]
+        else:
+            models = []
+
+        return {
+            "name": self.name,
+            "type": self.tag,
+            "models": models,
+            "filename": str(filename),
+        }
 
     def __str__(self):
-        str_ = "{}: \n".format(self.__class__.__name__)
+        str_ = f"{self.__class__.__name__}: \n"
         str_ += "\n"
-        if self.model is None:
+        if self.models is None:
             str_ += "\t{:32}:   {} \n".format("Model Name", "No Model")
         else:
             str_ += "\t{:32}:   {} \n".format("Total flux points", len(self.data.table))
@@ -1202,8 +1301,7 @@ class FluxPointsDataset(Dataset):
                 "Points used for the fit", self.mask.sum()
             )
             str_ += "\t{:32}:   {} \n".format(
-                "Excluded for safe energy range",
-                (~self.mask_safe).sum(),
+                "Excluded for safe energy range", (~self.mask_safe).sum()
             )
             if self.mask_fit is None:
                 str_ += "\t{:32}:   {} \n".format("Excluded by user", "0")
@@ -1212,16 +1310,14 @@ class FluxPointsDataset(Dataset):
                     "Excluded by user", (~self.mask_fit).sum()
                 )
             str_ += "\t{:32}:   {}\n".format(
-                "Model Name", self.model.__class__.__name__
-            )
-            str_ += "\t{:32}:   {}\n".format(
-                "N parameters", len(self.parameters.parameters)
+                "Model Name", self.models.__class__.__name__
             )
+            str_ += "\t{:32}:   {}\n".format("N parameters", len(self.parameters))
             str_ += "\t{:32}:   {}\n".format(
                 "N free parameters", len(self.parameters.free_parameters)
             )
             str_ += "\tList of parameters\n"
-            for par in self.parameters.parameters:
+            for par in self.parameters:
                 if par.frozen:
                     if par.name == "amplitude":
                         str_ += "\t \t {:14} (Frozen):   {:.2e} {} \n".format(
@@ -1240,45 +1336,26 @@ class FluxPointsDataset(Dataset):
                         str_ += "\t \t {:23}:   {:.2f} {} \n".format(
                             par.name, par.value, par.unit
                         )
-            str_ += "\t{:32}:   {}\n".format("Likelihood type", self._likelihood)
-            str_ += "\t{:32}:   {:.2f}\n".format("Likelihood value", self.likelihood())
+            str_ += "\t{:32}:   {:.2f}\n".format("Likelihood value", self.stat_sum())
         return str_
 
     def data_shape(self):
         """Shape of the flux points data (tuple)."""
         return self.data.e_ref.shape
 
-    @staticmethod
-    def _likelihood_chi2(data, model, sigma):
-        return ((data - model) / sigma).to_value("") ** 2
-
-    @staticmethod
-    def _likelihood_chi2_assym(data, model, sigma_n, sigma_p):
-        """Assymetric chi2 statistics for a list of flux points and model."""
-        is_p = model > data
-        sigma = sigma_n
-        sigma[is_p] = sigma_p[is_p]
-        return FluxPointsDataset._likelihood_chi2(data, model, sigma)
-
     def flux_pred(self):
         """Compute predicted flux."""
-        return self.model(self.data.e_ref)
+        flux = 0.0
+        for component in self.models:
+            flux += component.spectral_model(self.data.e_ref)
+        return flux
 
-    def likelihood_per_bin(self):
-        """Likelihood per bin given the current model parameters."""
+    def stat_array(self):
+        """Fit statistic array."""
         model = self.flux_pred()
         data = self.data.table["dnde"].quantity
-
-        if self._likelihood == "chi2":
-            sigma = self.data.table["dnde_err"].quantity
-            return self._likelihood_chi2(data, model, sigma)
-        elif self._likelihood == "chi2assym":
-            sigma_n = self.data.table["dnde_errn"].quantity
-            sigma_p = self.data.table["dnde_errp"].quantity
-            return self._likelihood_chi2_assym(data, model, sigma_n, sigma_p)
-        else:
-            # TODO: add likelihood profiles
-            pass
+        sigma = self.data.table["dnde_err"].quantity
+        return ((data - model) / sigma).to_value("") ** 2
 
     def residuals(self, method="diff"):
         """Compute the flux point residuals ().
@@ -1301,22 +1378,20 @@ class FluxPointsDataset(Dataset):
         fp = self.data
         data = fp.table[fp.sed_type]
 
-        model = self.model(fp.e_ref)
-
-        residuals = data - model
+        model = self.flux_pred()
 
         residuals = self._compute_residuals(data, model, method)
         # Remove residuals for upper_limits
         residuals[fp.is_ul] = np.nan
         return residuals
 
-    def peek(self, norm=None, **kwargs):
+    def peek(self, method="diff/model", **kwargs):
         """Plot flux points, best fit model and residuals.
 
         Parameters
         ----------
-        norm: `str`
-            normalization used to compute the spectral residuals. Choose between `None`, `model` and `sqrt_model`.
+        method : {"diff", "diff/model", "diff/sqrt(model)"}
+            Method used to compute the residuals, see `MapDataset.residuals()`
         """
         from matplotlib.gridspec import GridSpec
         import matplotlib.pyplot as plt
@@ -1329,7 +1404,7 @@ class FluxPointsDataset(Dataset):
         ax_spectrum.set_xticks([])
 
         ax_residuals = plt.subplot(gs[5:, :])
-        self.plot_residuals(ax=ax_residuals, norm=norm)
+        self.plot_residuals(ax=ax_residuals, method=method)
         return ax_spectrum, ax_residuals
 
     @property
@@ -1372,7 +1447,7 @@ class FluxPointsDataset(Dataset):
         if xerr is not None:
             xerr = xerr[0].to_value(self._e_unit), xerr[1].to_value(self._e_unit)
 
-        model = self.model(fp.e_ref)
+        model = self.flux_pred()
         yerr = fp._plot_get_flux_err(fp.sed_type)
 
         if method == "diff":
@@ -1394,8 +1469,8 @@ class FluxPointsDataset(Dataset):
 
         # format axes
         ax.axhline(0, color="black", lw=0.5)
-        ax.set_ylabel("Residuals   {}".format(unit.__str__()))
-        ax.set_xlabel("Energy ({})".format(self._e_unit))
+        ax.set_ylabel("Residuals {}".format(unit.__str__()))
+        ax.set_xlabel(f"Energy ({self._e_unit})")
         ax.set_xscale("log")
         ax.set_xlim(self._e_range.to_value(self._e_unit))
         y_max = 2 * np.nanmax(residuals).value
@@ -1426,10 +1501,11 @@ class FluxPointsDataset(Dataset):
         fp_kwargs = {} if fp_kwargs is None else fp_kwargs
         model_kwargs = {} if model_kwargs is None else model_kwargs
 
-        kwargs = {}
-        kwargs.setdefault("flux_unit", "erg-1 cm-2 s-1")
-        kwargs.setdefault("energy_unit", "TeV")
-        kwargs.setdefault("energy_power", 2)
+        kwargs = {
+            "flux_unit": "erg-1 cm-2 s-1",
+            "energy_unit": "TeV",
+            "energy_power": 2,
+        }
 
         # plot flux points
         plot_kwargs = kwargs.copy()
@@ -1442,13 +1518,17 @@ class FluxPointsDataset(Dataset):
         plot_kwargs.setdefault("zorder", 10)
         plot_kwargs.update(model_kwargs)
         plot_kwargs.setdefault("label", "Best fit model")
-        self.model.plot(ax=ax, **plot_kwargs)
+        for _ in self.models:
+            _.spectral_model.plot(ax=ax, **plot_kwargs)
 
         plot_kwargs.setdefault("color", ax.lines[-1].get_color())
         del plot_kwargs["label"]
 
-        if self.model.parameters.covariance is not None:
-            self.model.plot_error(ax=ax, **plot_kwargs)
+        if self.models.parameters.covariance is not None:
+            try:
+                self.models.plot_error(ax=ax, **plot_kwargs)
+            except AttributeError:
+                log.debug("Model does not support evaluation of errors")
 
         # format axes
         ax.set_xlim(self._e_range.to_value(self._e_unit))
