@@ -4,26 +4,33 @@ import numpy as np
 from numpy.testing import assert_allclose
 from astropy import units as u
 from astropy.coordinates import SkyCoord
-from ...utils.testing import requires_dependency, requires_data
-from ...irf import EffectiveAreaTable, load_cta_irfs
-from ..models import PowerLaw, ExponentialCutoffPowerLaw
-from ..dataset import SpectrumDatasetOnOff
-from ..utils import SpectrumEvaluator
-from ..flux_point import FluxPointsEstimator
-from ...cube import simulate_dataset
-from ...cube.models import SkyModel
-from ...image.models import SkyGaussian
-from ...maps import MapAxis, WcsGeom
+from gammapy.cube import MapDataset, MapDatasetMaker
+from gammapy.data import Observation
+from gammapy.irf import EffectiveAreaTable, load_cta_irfs
+from gammapy.maps import MapAxis, WcsGeom
+from gammapy.modeling.models import (
+    ExpCutoffPowerLawSpectralModel,
+    GaussianSpatialModel,
+    PowerLawSpectralModel,
+    SkyModel,
+)
+from gammapy.spectrum import FluxPointsEstimator, SpectrumDatasetOnOff
+from gammapy.spectrum.core import SpectrumEvaluator
+from gammapy.utils.testing import requires_data, requires_dependency
 
 
 # TODO: use pregenerate data instead
 def simulate_spectrum_dataset(model, random_state=0):
     energy = np.logspace(-0.5, 1.5, 21) * u.TeV
     aeff = EffectiveAreaTable.from_parametrization(energy=energy)
-    bkg_model = PowerLaw(index=2.5, amplitude="1e-12 cm-2 s-1 TeV-1")
+    bkg_model = SkyModel(
+        spectral_model=PowerLawSpectralModel(
+            index=2.5, amplitude="1e-12 cm-2 s-1 TeV-1"
+        )
+    )
 
     dataset = SpectrumDatasetOnOff(
-        aeff=aeff, model=model, livetime=100 * u.h, acceptance=1, acceptance_off=5
+        aeff=aeff, models=model, livetime=100 * u.h, acceptance=1, acceptance_off=5
     )
 
     eval = SpectrumEvaluator(model=bkg_model, aeff=aeff, livetime=100 * u.h)
@@ -34,9 +41,10 @@ def simulate_spectrum_dataset(model, random_state=0):
 
 
 def create_fpe(model):
+    model = SkyModel(spectral_model=model)
     dataset = simulate_spectrum_dataset(model)
     e_edges = [0.1, 1, 10, 100] * u.TeV
-    dataset.model = model
+    dataset.models = model
     return FluxPointsEstimator(datasets=[dataset], e_edges=e_edges, norm_n_values=11)
 
 
@@ -47,22 +55,25 @@ def simulate_map_dataset(random_state=0):
 
     skydir = SkyCoord("0 deg", "0 deg", frame="galactic")
     edges = np.logspace(-1, 2, 15) * u.TeV
-    energy_axis = MapAxis.from_edges(edges=edges, name="energy")
+    energy_axis = MapAxis.from_edges(edges=edges, name="energy", interp="log")
 
     geom = WcsGeom.create(
         skydir=skydir, width=(4, 4), binsz=0.1, axes=[energy_axis], coordsys="GAL"
     )
 
-    gauss = SkyGaussian("0 deg", "0 deg", "0.4 deg", frame="galactic")
-    pwl = PowerLaw(amplitude="1e-11 cm-2 s-1 TeV-1")
-    skymodel = SkyModel(spatial_model=gauss, spectral_model=pwl, name="source")
-    dataset = simulate_dataset(
-        skymodel=skymodel,
-        geom=geom,
-        pointing=skydir,
-        irfs=irfs,
-        random_state=random_state,
+    gauss = GaussianSpatialModel(
+        lon_0="0 deg", lat_0="0 deg", sigma="0.4 deg", frame="galactic"
     )
+    pwl = PowerLawSpectralModel(amplitude="1e-11 cm-2 s-1 TeV-1")
+    skymodel = SkyModel(spatial_model=gauss, spectral_model=pwl, name="source")
+
+    obs = Observation.create(pointing=skydir, livetime=1 * u.h, irfs=irfs)
+    empty = MapDataset.create(geom)
+    maker = MapDatasetMaker(selection=["exposure", "background", "psf", "edisp"])
+    dataset = maker.run(empty, obs)
+
+    dataset.models = skymodel
+    dataset.fake(random_state=random_state)
     return dataset
 
 
@@ -99,12 +110,12 @@ def fpe_map_pwl_reoptimize():
 
 @pytest.fixture(scope="session")
 def fpe_pwl():
-    return create_fpe(PowerLaw())
+    return create_fpe(PowerLawSpectralModel())
 
 
 @pytest.fixture(scope="session")
 def fpe_ecpl():
-    return create_fpe(ExponentialCutoffPowerLaw(lambda_="1 TeV-1"))
+    return create_fpe(ExpCutoffPowerLawSpectralModel(lambda_="1 TeV-1"))
 
 
 class TestFluxPointsEstimator:
@@ -140,7 +151,7 @@ class TestFluxPointsEstimator:
         actual = fp.table["norm_scan"][0][[0, 5, -1]]
         assert_allclose(actual, [0.2, 1, 5])
 
-        actual = fp.table["dloglike_scan"][0][[0, 5, -1]]
+        actual = fp.table["stat_scan"][0][[0, 5, -1]]
         assert_allclose(actual, [220.368653, 4.301011, 1881.626454], rtol=1e-2)
 
     @staticmethod
@@ -156,25 +167,25 @@ class TestFluxPointsEstimator:
         fp = fpe_map_pwl.run()
 
         actual = fp.table["norm"].data
-        assert_allclose(actual, [0.97922, 0.94081, 1.074426], rtol=1e-3)
+        assert_allclose(actual, [0.974651, 0.966819, 0.897374], rtol=1e-2)
 
         actual = fp.table["norm_err"].data
-        assert_allclose(actual, [0.069966, 0.052617, 0.092854], rtol=1e-2)
+        assert_allclose(actual, [0.067386, 0.052279, 0.091548], rtol=1e-2)
 
         actual = fp.table["counts"].data
-        assert_allclose(actual, [[44445, 0], [1911, 0], [292, 0]])
+        assert_allclose(actual, [[44609, 0], [1923, 0], [250, 0]])
 
         actual = fp.table["norm_ul"].data
-        assert_allclose(actual, [1.121379, 1.048815, 1.270037], rtol=1e-2)
+        assert_allclose(actual, [1.104696, 1.075925, 1.091443], rtol=1e-2)
 
         actual = fp.table["sqrt_ts"].data
-        assert_allclose(actual, [16.165806, 27.121415, 22.04104], rtol=1e-2)
+        assert_allclose(actual, [16.64713, 28.415688, 18.3774043], rtol=1e-2)
 
         actual = fp.table["norm_scan"][0]
         assert_allclose(actual, [0.2, 1, 5])
 
-        actual = fp.table["dloglike_scan"][0] - fp.table["loglike"][0]
-        assert_allclose(actual, [1.536452e02, 8.762343e-02, 1.883447e03], rtol=1e-2)
+        actual = fp.table["stat_scan"][0] - fp.table["stat"][0]
+        assert_allclose(actual, [1.614884e02, 2.263890e-01, 2.015639e03], rtol=1e-2)
 
     @staticmethod
     @requires_dependency("iminuit")
@@ -183,28 +194,60 @@ class TestFluxPointsEstimator:
         fp = fpe_map_pwl_reoptimize.run(steps=["err", "norm-scan", "ts"])
 
         actual = fp.table["norm"].data
-        assert_allclose(actual, 0.884621, rtol=1e-3)
+        assert_allclose(actual, 0.986422, rtol=1e-3)
 
         actual = fp.table["norm_err"].data
-        assert_allclose(actual, 0.058005, rtol=1e-2)
+        assert_allclose(actual, 0.061124, rtol=1e-2)
 
         actual = fp.table["sqrt_ts"].data
-        assert_allclose(actual, 23.971251, rtol=1e-2)
+        assert_allclose(actual, 25.214661, rtol=1e-2)
 
         actual = fp.table["norm_scan"][0]
         assert_allclose(actual, 1)
 
-        actual = fp.table["dloglike_scan"][0] - fp.table["loglike"][0]
-        assert_allclose(actual, 3.698882, rtol=1e-2)
+        actual = fp.table["stat_scan"][0] - fp.table["stat"][0]
+        assert_allclose(actual, 0.048701, rtol=1e-2)
 
 
 def test_no_likelihood_contribution():
-    dataset = simulate_spectrum_dataset(PowerLaw())
-    dataset.model = PowerLaw()
+    dataset = simulate_spectrum_dataset(
+        SkyModel(spectral_model=PowerLawSpectralModel())
+    )
     dataset.mask_safe = np.zeros(dataset.data_shape, dtype=bool)
 
-    fpe = FluxPointsEstimator([dataset], e_edges=[1, 10] * u.TeV)
+    fpe = FluxPointsEstimator([dataset], e_edges=[1, 3, 10] * u.TeV)
+    fp = fpe.run()
 
-    with pytest.raises(ValueError) as excinfo:
-        fpe.run()
-    assert "No dataset contributes" in str(excinfo.value)
+    assert np.isnan(fp.table["norm"]).all()
+    assert np.isnan(fp.table["norm_err"]).all()
+    assert np.isnan(fp.table["norm_ul"]).all()
+    assert np.isnan(fp.table["norm_scan"]).all()
+    assert_allclose(fp.table["counts"], 0)
+
+
+def test_mask_shape():
+    axis = MapAxis.from_edges([1, 3, 10], unit="TeV", interp="log", name="energy")
+    geom_1 = WcsGeom.create(binsz=1, width=3, axes=[axis])
+    geom_2 = WcsGeom.create(binsz=1, width=5, axes=[axis])
+
+    dataset_1 = MapDataset.create(geom_1)
+    dataset_2 = MapDataset.create(geom_2)
+    dataset_1.psf = None
+    dataset_2.psf = None
+    dataset_1.edisp = None
+    dataset_2.edisp = None
+
+    model = SkyModel(
+        spectral_model=PowerLawSpectralModel(), spatial_model=GaussianSpatialModel()
+    )
+
+    dataset_1.models = model
+    dataset_2.models = model
+
+    fpe = FluxPointsEstimator(
+        datasets=[dataset_2, dataset_1], e_edges=[1, 10] * u.TeV, source="source"
+    )
+
+    fp = fpe.run()
+
+    assert_allclose(fp.table["counts"], 0)
