@@ -1,14 +1,13 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """Interpolation utilities"""
 import numpy as np
-from scipy.interpolate import RegularGridInterpolator, interp1d
+import scipy.interpolate
 from astropy import units as u
 
-
 __all__ = [
     "ScaledRegularGridInterpolator",
     "interpolation_scale",
-    "interpolate_likelihood_profile",
+    "interpolate_profile",
 ]
 
 
@@ -43,7 +42,7 @@ class ScaledRegularGridInterpolator:
         values_scale="lin",
         extrapolate=True,
         axis=None,
-        **kwargs
+        **kwargs,
     ):
 
         if points_scale is None:
@@ -61,18 +60,20 @@ class ScaledRegularGridInterpolator:
             kwargs.setdefault("fill_value", None)
 
         if axis is None:
-            self._interpolate = RegularGridInterpolator(
+            self._interpolate = scipy.interpolate.RegularGridInterpolator(
                 points=points_scaled, values=values_scaled, **kwargs
             )
         else:
-            self._interpolate = interp1d(points_scaled[0], values_scaled, axis=axis)
+            self._interpolate = scipy.interpolate.interp1d(
+                points_scaled[0], values_scaled, axis=axis
+            )
 
     def __call__(self, points, method="linear", clip=True, **kwargs):
         """Interpolate data points.
 
         Parameters
         ----------
-        points : tuple of `np.ndarray` or `~astropy.units.Quantity`
+        points : tuple of `~numpy.ndarray` or `~astropy.units.Quantity`
             Tuple of coordinate arrays of the form (x_1, x_2, x_3, ...). Arrays are
             broadcasted internally.
         method : {"linear", "nearest"}
@@ -80,6 +81,7 @@ class ScaledRegularGridInterpolator:
         clip : bool
             Clip values at zero after interpolation.
         """
+
         points = tuple([scale(p) for scale, p in zip(self.scale_points, points)])
 
         if self.axis is None:
@@ -93,7 +95,6 @@ class ScaledRegularGridInterpolator:
 
         if clip:
             values = np.clip(values, 0, np.inf)
-
         return values
 
 
@@ -112,7 +113,7 @@ def interpolation_scale(scale="lin"):
     elif scale == "sqrt":
         return SqrtScale()
     else:
-        raise ValueError("Not a valid value scaling mode: '{}'.".format(scale))
+        raise ValueError(f"Not a valid value scaling mode: '{scale}'.")
 
 
 class InterpolationScale:
@@ -128,7 +129,7 @@ class InterpolationScale:
         return self._scale(values)
 
     def inverse(self, values):
-        values = self._inverse(values)
+        values = self._inverse(self, values)
         if hasattr(self, "_unit"):
             return u.Quantity(values, self._unit, copy=False)
         else:
@@ -145,8 +146,9 @@ class LogScale(InterpolationScale):
         return np.log(values)
 
     @staticmethod
-    def _inverse(values):
-        return np.exp(values)
+    def _inverse(self, values):
+        output = np.exp(values)
+        return np.where(abs(output) - self.tiny <= self.tiny, 0, output)
 
 
 class SqrtScale(InterpolationScale):
@@ -158,7 +160,7 @@ class SqrtScale(InterpolationScale):
         return sign * np.sqrt(sign * values)
 
     @staticmethod
-    def _inverse(values):
+    def _inverse(self, values):
         return np.power(values, 2)
 
 
@@ -170,35 +172,30 @@ class LinearScale(InterpolationScale):
         return values
 
     @staticmethod
-    def _inverse(values):
+    def _inverse(self, values):
         return values
 
 
-def interpolate_likelihood_profile(value_scan, dloglike_scan, interp_scale="sqrt"):
-    """Helper function to interpolate likelihood profiles.
+def interpolate_profile(x, y, interp_scale="sqrt"):
+    """Helper function to interpolate one-dimensional profiles.
 
     Parameters
     ----------
-    value_scan : `~numpy.ndarray`
-        Array of parameter values.
-    dloglike_scan : `~numpy.ndarray`
-        Array of delta log-likelihood values, with respect to the minimum.
+    x : `~numpy.ndarray`
+        Array of x values
+    y : `~numpy.ndarray`
+        Array of y values
     interp_scale : {"sqrt", "lin"}
-        Interpolation scale applied to the likelihood profile. If the profile is
+        Interpolation scale applied to the profile. If the profile is
         of parabolic shape, a "sqrt" scaling is recommended. In other cases or
         for fine sampled profiles a "lin" can also be used.
 
     Returns
     -------
     interp : `ScaledRegularGridInterpolator`
-        Interpolator instance.
-
+        Interpolator
     """
-    # likelihood profiles are typically of parabolic shape, so we use a
-    # sqrt scaling of the values and perform linear interpolation on the scaled
-    # values
-    sign = np.sign(np.gradient(dloglike_scan))
-    interp = ScaledRegularGridInterpolator(
-        points=(value_scan,), values=sign * dloglike_scan, values_scale=interp_scale
+    sign = np.sign(np.gradient(y))
+    return ScaledRegularGridInterpolator(
+        points=(x,), values=sign * y, values_scale=interp_scale
     )
-    return interp
