@@ -1,16 +1,16 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """Utilities for testing"""
-from __future__ import absolute_import, division, print_function, unicode_literals
-import sys
 import os
+import sys
 from numpy.testing import assert_allclose
 import astropy.units as u
-from astropy.time import Time
 from astropy.coordinates import SkyCoord
+from astropy.time import Time
 
 __all__ = [
     "requires_dependency",
     "requires_data",
+    "mpl_plot_check",
     "assert_quantity_allclose",
     "assert_wcs_allclose",
     "assert_skycoord_allclose",
@@ -49,7 +49,7 @@ def requires_dependency(name):
 
         _requires_dependency_cache[name] = skip_it
 
-    reason = "Missing dependency: {}".format(name)
+    reason = f"Missing dependency: {name}"
     return pytest.mark.skipif(skip_it, reason=reason)
 
 
@@ -64,10 +64,10 @@ def has_data(name):
     elif name == "fermi-lat":
         return "GAMMAPY_FERMI_LAT_DATA" in os.environ
     else:
-        raise ValueError("Invalid name: {}".format(name))
+        raise ValueError(f"Invalid name: {name}")
 
 
-def requires_data(name):
+def requires_data(name="gammapy-data"):
     """Decorator to declare required data for tests.
 
     Examples
@@ -75,18 +75,23 @@ def requires_data(name):
     ::
 
         from gammapy.utils.testing import requires_data
-        from gammapy.datasets import data
 
-        @requires_data('gammapy-data')
+        @requires_data()
         def test_using_data_files():
-            filename = gammapy_data.filename('...')
+            filename = "$GAMMAPY_DATA/..."
             ...
     """
     import pytest
 
+    if not isinstance(name, str):
+        raise TypeError(
+            "You must call @requires_data with a name (str). "
+            "Usually this:  @requires_data()"
+        )
+
     skip_it = not has_data(name)
 
-    reason = "Missing data: {}".format(name)
+    reason = f"Missing data: {name}"
     return pytest.mark.skipif(skip_it, reason=reason)
 
 
@@ -175,7 +180,7 @@ def _unquantify_allclose_arguments(actual, desired, rtol, atol):
         desired = desired.to(actual.unit)
     except u.UnitsError:
         raise u.UnitsError(
-            "Units for 'desired' ({0}) and 'actual' ({1}) "
+            "Units for 'desired' ({}) and 'actual' ({}) "
             "are not convertible".format(desired.unit, actual.unit)
         )
 
@@ -188,7 +193,7 @@ def _unquantify_allclose_arguments(actual, desired, rtol, atol):
             atol = atol.to(actual.unit)
         except u.UnitsError:
             raise u.UnitsError(
-                "Units for 'atol' ({0}) and 'actual' ({1}) "
+                "Units for 'atol' ({}) and 'actual' ({}) "
                 "are not convertible".format(atol.unit, actual.unit)
             )
 
@@ -214,7 +219,7 @@ def mpl_plot_check():
     import matplotlib.pyplot as plt
     from io import BytesIO
 
-    class MPLPlotCheck(object):
+    class MPLPlotCheck:
         def __enter__(self):
             plt.figure()
 
@@ -225,7 +230,7 @@ def mpl_plot_check():
     return MPLPlotCheck()
 
 
-class Checker(object):
+class Checker:
     """Base class for checker classes in Gammapy."""
 
     def run(self, checks="all"):
@@ -234,8 +239,8 @@ class Checker(object):
 
         unknown_checks = sorted(set(checks).difference(self.CHECKS.keys()))
         if unknown_checks:
-            raise ValueError("Unknown checks: {!r}".format(unknown_checks))
+            raise ValueError(f"Unknown checks: {unknown_checks!r}")
 
         for check in checks:
-            for record in getattr(self, self.CHECKS[check])():
-                yield record
+            method = getattr(self, self.CHECKS[check])
+            yield from method()
